@@ -1,6 +1,7 @@
 /*
 	 This file is part of darktable,
-	 copyright (c) 2010-2011 Henrik Andersson.
+	 copyright (c) 2010-2012 Henrik Andersson.
+	 copyright (c) 2012 Tobias Ellinghaus.
 
 	 darktable is free software: you can redistribute it and/or modify
 	 it under the terms of the GNU General Public License as published by
@@ -19,6 +20,9 @@
 #include "config.h"
 #endif
 #include "common/camera_control.h"
+#include "control/control.h"
+#include "libraw/libraw.h"
+#include <gphoto2/gphoto2-file.h>
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -34,6 +38,8 @@ typedef enum _camctl_camera_job_type_t
   _JOB_TYPE_DETECT_DEVICES,
   /** Remotly executes a capture. */
   _JOB_TYPE_EXECUTE_CAPTURE,
+  /** Fetch a preview for live view. */
+  _JOB_TYPE_EXECUTE_LIVE_VIEW,
   /** Read a copy of remote camera config into cache. */
   _JOB_TYPE_READ_CONFIG,
   /** Writes changed properties in cache to camera */
@@ -78,7 +84,7 @@ void _camera_configuration_merge(const dt_camctl_t *c,const dt_camera_t *camera,
 void _camera_add_job(const dt_camctl_t *c,const dt_camera_t *camera, gpointer job);
 /** Get a job from the queue */
 gpointer _camera_get_job( const dt_camctl_t *c, const dt_camera_t *camera );
-void _camera_process_jobb(const dt_camctl_t *c,const dt_camera_t *camera, gpointer job);
+static void _camera_process_job(const dt_camctl_t *c,const dt_camera_t *camera, gpointer job);
 
 /** Dispatch functions for listener interfaces */
 const char *_dispatch_request_image_path(const dt_camctl_t *c,const dt_camera_t *camera);
@@ -210,7 +216,7 @@ gpointer _camera_get_job( const dt_camctl_t *c,const dt_camera_t *camera )
 }
 
 
-void _camera_process_jobb(const dt_camctl_t *c,const dt_camera_t *camera, gpointer job)
+static void _camera_process_job(const dt_camctl_t *c,const dt_camera_t *camera, gpointer job)
 {
   dt_camera_t *cam=(dt_camera_t *)camera;
   _camctl_camera_job_t *j = (_camctl_camera_job_t *)job;
@@ -225,10 +231,13 @@ void _camera_process_jobb(const dt_camctl_t *c,const dt_camera_t *camera, gpoint
       if( (res = gp_camera_capture (camera->gpcam, GP_CAPTURE_IMAGE,&fp, c->gpcontext)) == GP_OK )
       {
         CameraFile *destination;
-        const char *output_path = _dispatch_request_image_path(c,camera);
-        if( !output_path ) output_path="/tmp";
-        const char *fname = _dispatch_request_image_filename(c,fp.name,cam);
-        if( !fname ) fname=fp.name;
+        const char *output_path = _dispatch_request_image_path(c, camera);
+        if(!output_path)
+	  output_path="/tmp";
+
+	const char *fname = _dispatch_request_image_filename(c, fp.name,cam);
+	if(!fname)
+	  break;
 
         char *output = g_build_filename (output_path,fname,(char *)NULL);
 
@@ -248,6 +257,44 @@ void _camera_process_jobb(const dt_camctl_t *c,const dt_camera_t *camera, gpoint
     }
     break;
 
+    case _JOB_TYPE_EXECUTE_LIVE_VIEW:
+    {
+      CameraFile *fp = NULL;
+      int res = GP_OK;
+      const gchar* data = NULL;
+      unsigned long int data_size = 0;
+
+      gp_file_new(&fp);
+
+      if( (res = gp_camera_capture_preview (cam->gpcam, fp, c->gpcontext)) != GP_OK )
+      {
+        dt_print (DT_DEBUG_CAMCTL,"[camera_control] live view failed to capture preview: %s\n", gp_result_as_string(res));
+      }
+      else if( (res = gp_file_get_data_and_size(fp, &data, &data_size)) != GP_OK )
+      {
+        dt_print (DT_DEBUG_CAMCTL,"[camera_control] live view failed to get preview data: %s\n", gp_result_as_string(res));
+      }
+      else
+      {
+        // everything worked
+        GdkPixbufLoader *loader = gdk_pixbuf_loader_new();
+        if(gdk_pixbuf_loader_write(loader, (guchar*)data, data_size, NULL) == TRUE)
+        {
+          dt_pthread_mutex_lock(&cam->live_view_pixbuf_mutex);
+          if(cam->live_view_pixbuf != NULL)
+            g_object_unref(cam->live_view_pixbuf);
+          cam->live_view_pixbuf = gdk_pixbuf_loader_get_pixbuf(loader);
+          dt_pthread_mutex_unlock(&cam->live_view_pixbuf_mutex);
+        }
+        gdk_pixbuf_loader_close(loader, NULL);
+      }
+      if(fp)
+        gp_file_free(fp);
+      dt_pthread_mutex_unlock(&cam->live_view_synch);
+      dt_control_queue_redraw_center();
+    }
+    break;
+
     case _JOB_TYPE_SET_PROPERTY:
     {
       _camctl_camera_set_property_job_t *spj=(_camctl_camera_set_property_job_t *)job;
@@ -281,6 +328,93 @@ void _camera_process_jobb(const dt_camctl_t *c,const dt_camera_t *camera, gpoint
   g_free(j);
 }
 
+/*************/
+/* LIVE VIEW */
+/*************/
+static void *dt_camctl_camera_get_live_view(void* data)
+{
+  dt_camctl_t *camctl = (dt_camctl_t*)data;
+  dt_camera_t *cam = (dt_camera_t*)camctl->active_camera;
+  dt_print (DT_DEBUG_CAMCTL,"[camera_control] live view thread started\n");
+
+  int frames= 0;
+  double capture_time = dt_get_wtime();
+
+  while(cam->is_live_viewing == TRUE)
+  {
+    dt_pthread_mutex_lock(&cam->live_view_synch);
+
+    // calculate FPS
+    double current_time = dt_get_wtime();
+    if(current_time - capture_time >= 1.0)
+    {
+      // a second has passed
+      dt_print(DT_DEBUG_CAMCTL, "%d fps\n", frames+1);
+      frames = 0;
+      capture_time = current_time;
+    }
+    else
+    {
+      // just increase the frame counter
+      frames++;
+    }
+
+    _camctl_camera_job_t *job = g_malloc(sizeof(_camctl_camera_job_t));
+    job->type = _JOB_TYPE_EXECUTE_LIVE_VIEW;
+    _camera_add_job( camctl, cam, job);
+
+    g_usleep((1.0/15)*G_USEC_PER_SEC); // never update faster than 15 FPS. going too fast will result in too many redraws without a real benefit
+  }
+  dt_print (DT_DEBUG_CAMCTL,"[camera_control] live view thread stopped\n");
+  return NULL;
+}
+
+gboolean dt_camctl_camera_start_live_view(const dt_camctl_t *c)
+{
+  dt_camctl_t *camctl = (dt_camctl_t*)c;
+  dt_camera_t *cam = (dt_camera_t*)camctl->active_camera;
+  if( cam == NULL )
+  {
+    dt_print(DT_DEBUG_CAMCTL,"[camera_control] Failed to start live view, camera==NULL\n");
+    return FALSE;
+  }
+  else
+    dt_print(DT_DEBUG_CAMCTL,"[camera_control] Starting live view\n");
+
+  if(cam->can_live_view == FALSE)
+  {
+    dt_print(DT_DEBUG_CAMCTL,"[camera_control] Camera does not support live view\n");
+    return FALSE;
+  }
+  cam->is_live_viewing = TRUE;
+  dt_camctl_camera_set_property(darktable.camctl, NULL, "eosviewfinder", "1");
+  pthread_create(&cam->live_view_thread, NULL, &dt_camctl_camera_get_live_view, (void*)camctl);
+  return TRUE;
+}
+
+void dt_camctl_camera_stop_live_view(const dt_camctl_t *c)
+{
+  dt_camctl_t *camctl = (dt_camctl_t*)c;
+  dt_camera_t *cam = (dt_camera_t*)camctl->active_camera;
+  dt_print(DT_DEBUG_CAMCTL,"[camera_control] Stopping live view\n");
+  cam->is_live_viewing = FALSE;
+  pthread_join(cam->live_view_thread, NULL);
+  //tell camera to get back to normal state (close mirror)
+  // this should work like this:
+//   dt_camctl_camera_set_property(darktable.camctl, NULL, "eosviewfinder", "0");
+  // but it doesn't, passing a string isn't ok in this case. I guess that's a TODO.
+  // for the time being I'll do it manually (not nice, I know).
+  CameraWidget *config;
+  CameraWidget *widget;
+  gp_camera_get_config( cam->gpcam, &config, camctl->gpcontext );
+  if(  gp_widget_get_child_by_name ( config, "eosviewfinder", &widget) == GP_OK)
+  {
+    int zero=0;
+    gp_widget_set_value ( widget , &zero);
+    gp_camera_set_config( cam->gpcam, config, camctl->gpcontext );
+  }
+}
+
 void _camctl_lock(const dt_camctl_t *c,const dt_camera_t *cam)
 {
   dt_camctl_t *camctl=(dt_camctl_t *)c;
@@ -320,6 +454,7 @@ dt_camctl_t *dt_camctl_new()
   dt_print(DT_DEBUG_CAMCTL,"[camera_control] loaded %d camera drivers.\n", gp_abilities_list_count( camctl->gpcams ) );
 
   dt_pthread_mutex_init(&camctl->lock, NULL);
+  dt_pthread_mutex_init(&camctl->listeners_lock, NULL);
 
   // Let's detect cameras connexted
   dt_camctl_detect_cameras(camctl);
@@ -345,7 +480,7 @@ void dt_camctl_register_listener( const dt_camctl_t *c, dt_camctl_listener_t *li
 {
   dt_camctl_t *camctl=(dt_camctl_t *)c;
   // Just locking mutex and prevent signalling CAMERA_CONTROL_BUSY
-  dt_pthread_mutex_lock(&camctl->lock);
+  dt_pthread_mutex_lock(&camctl->listeners_lock);
   if( g_list_find(camctl->listeners,listener) == NULL )
   {
     camctl->listeners=g_list_append(camctl->listeners,listener);
@@ -353,17 +488,17 @@ void dt_camctl_register_listener( const dt_camctl_t *c, dt_camctl_listener_t *li
   }
   else
     dt_print(DT_DEBUG_CAMCTL,"[camera_control] registering already registered listener %lx\n",(unsigned long int)listener);
-  dt_pthread_mutex_unlock(&camctl->lock);
+  dt_pthread_mutex_unlock(&camctl->listeners_lock);
 }
 
 void dt_camctl_unregister_listener( const dt_camctl_t *c, dt_camctl_listener_t *listener)
 {
   dt_camctl_t *camctl=(dt_camctl_t *)c;
   // Just locking mutex and prevent signalling CAMERA_CONTROL_BUSY
-  dt_pthread_mutex_lock(&camctl->lock);
+  dt_pthread_mutex_lock(&camctl->listeners_lock);
   dt_print(DT_DEBUG_CAMCTL,"[camera_control] unregistering listener %lx\n",(unsigned long int)listener);
   camctl->listeners = g_list_remove( camctl->listeners, listener );
-  dt_pthread_mutex_unlock(&camctl->lock);
+  dt_pthread_mutex_unlock(&camctl->listeners_lock);
 }
 
 gint _compare_camera_by_port(gconstpointer a,gconstpointer b)
@@ -402,6 +537,8 @@ void dt_camctl_detect_cameras(const dt_camctl_t *c)
     gp_list_get_name (available_cameras, i, &camera->model);
     gp_list_get_value (available_cameras, i, &camera->port);
     dt_pthread_mutex_init(&camera->config_lock, NULL);
+    dt_pthread_mutex_init(&camera->live_view_pixbuf_mutex, NULL);
+    dt_pthread_mutex_init(&camera->live_view_synch, NULL);
 
     // if(strcmp(camera->port,"usb:")==0) { g_free(camera); continue; }
     GList *citem;
@@ -483,7 +620,7 @@ static void *_camera_event_thread(void *data)
     // Let's check if there are jobs in queue to process
     gpointer job;
     while( (job=_camera_get_job(camctl,camera)) != NULL )
-      _camera_process_jobb(camctl,camera,job);
+      _camera_process_job(camctl,camera,job);
 
     // Check it jobs did change the configuration
     if( camera->config_changed == TRUE )
@@ -514,6 +651,7 @@ gboolean _camera_initialize(const dt_camctl_t *c, dt_camera_t *cam)
 
     // Check for abilities
     if( (a.operations&GP_OPERATION_CAPTURE_IMAGE) ) cam->can_tether=TRUE;
+    if( (a.operations&GP_OPERATION_CAPTURE_PREVIEW) ) cam->can_live_view=TRUE;
     if(  cam->can_tether && (a.operations&GP_OPERATION_CONFIG) ) cam->can_config=TRUE;
     if( !(a.file_operations&GP_FILE_OPERATION_NONE) ) cam->can_import=TRUE;
 
@@ -526,12 +664,16 @@ gboolean _camera_initialize(const dt_camctl_t *c, dt_camera_t *cam)
     // read a full copy of config to configuration cache
     gp_camera_get_config( cam->gpcam, &cam->configuration, c->gpcontext );
 
+    // TODO: find a more robust way for this, once we find out how to do it with non-EOS cameras
+    if(cam->can_live_view && dt_camctl_camera_property_exists(camctl, cam, "eoszoomposition"))
+      cam->can_live_view_advanced = TRUE;
+
     // initialize timeout callbacks eg. keep alive, some cameras needs it.
     cam->gpcontext = camctl->gpcontext;
-    gp_camera_set_timeout_funcs(cam->gpcam, 
-				(CameraTimeoutStartFunc)_camera_start_timeout_func,
-				(CameraTimeoutStopFunc)_camera_stop_timeout_func,
-				cam);
+    gp_camera_set_timeout_funcs(cam->gpcam,
+                                (CameraTimeoutStartFunc)_camera_start_timeout_func,
+                                (CameraTimeoutStopFunc)_camera_stop_timeout_func,
+                                cam);
 
 
     dt_pthread_mutex_init(&cam->jobqueue_lock, NULL);
@@ -559,18 +701,19 @@ void dt_camctl_import(const dt_camctl_t *c,const dt_camera_t *cam,GList *images,
     {
       // Split file into folder and filename
       char *eos;
-      char folder[4096]= {0};
-      char filename[4096]= {0};
+      char folder[DT_MAX_PATH_LEN]= {0};
+      char filename[DT_MAX_PATH_LEN]= {0};
       char *file=(char *)ifile->data;
       eos=file+strlen(file);
       while( --eos>file && *eos!='/' );
       char *_file = g_strndup(file, eos-file);
-      g_strlcat(folder, _file, 4096);
-      g_strlcat(filename, eos+1, 4096);
+      g_strlcat(folder, _file, DT_MAX_PATH_LEN);
+      g_strlcat(filename, eos+1, DT_MAX_PATH_LEN);
       g_free(_file);
 
       const char *fname = _dispatch_request_image_filename(c,filename,cam);
-      if(!fname) fname=filename;
+      if(!fname)
+	continue;
 
       char *output = g_build_filename(output_path,fname,(char *)NULL);
 
@@ -634,12 +777,34 @@ int _camctl_recursive_get_previews(const dt_camctl_t *c,dt_camera_preview_flags_
           if( gp_camera_file_get(c->active_camera->gpcam, path, filename, GP_FILE_TYPE_PREVIEW,preview,c->gpcontext) < GP_OK )
           {
             // No preview for file lets check image size to se if we should download full image for preview...
-            if( cfi.file.size > 0  && cfi.file.size < 512000 )
+            if( cfi.file.size > 0  && cfi.file.size < 512000 ) {
               if( gp_camera_file_get(c->active_camera->gpcam, path, filename, GP_FILE_TYPE_NORMAL,preview,c->gpcontext) < GP_OK )
               {
                 preview=NULL;
                 dt_print(DT_DEBUG_CAMCTL,"[camera_control] failed to retreive preview of file %s\n",filename);
               }
+            } else if (!strncmp(c->active_camera->port, "disk:", 5)) {
+              int ret;
+              char fullpath[512];
+              snprintf(fullpath,512,"%s/%s/%s",c->active_camera->port+5, path, filename);
+              libraw_data_t *raw = libraw_init(0);
+              libraw_processed_image_t *image = NULL;
+              ret = libraw_open_file(raw, fullpath);
+              if(ret) goto libraw_thumb_fail;
+              ret = libraw_unpack_thumb(raw);
+              if(ret) goto libraw_thumb_fail;
+              ret = libraw_adjust_sizes_info_only(raw);
+              if(ret) goto libraw_thumb_fail;
+              image = libraw_dcraw_make_mem_thumb(raw, &ret);
+              if(!image || ret) goto libraw_thumb_fail;
+              char *img = (char *) malloc(image->data_size);
+              if (!img) goto libraw_thumb_fail;
+              memcpy(img, image->data, image->data_size); 
+              gp_file_set_data_and_size(preview, img,(unsigned long int) image->data_size);
+              free(image);
+libraw_thumb_fail:
+              libraw_close(raw);
+            }
           }
         }
 
@@ -671,11 +836,11 @@ int _camctl_recursive_get_previews(const dt_camctl_t *c,dt_camera_preview_flags_
   {
     for(int i=0; i < gp_list_count(folders); i++)
     {
-      char buffer[4096]= {0};
-      g_strlcat(buffer,path, 4096);
-      if(path[1]!='\0') g_strlcat(buffer,"/", 4096);
+      char buffer[DT_MAX_PATH_LEN]= {0};
+      g_strlcat(buffer,path, DT_MAX_PATH_LEN);
+      if(path[1]!='\0') g_strlcat(buffer,"/", DT_MAX_PATH_LEN);
       gp_list_get_name (folders, i, &foldername);
-      g_strlcat(buffer,foldername, 4096);
+      g_strlcat(buffer,foldername, DT_MAX_PATH_LEN);
       if( !_camctl_recursive_get_previews(c,flags,buffer))
         return 0;
     }
@@ -988,58 +1153,45 @@ void _camera_poll_events(const dt_camctl_t *c,const dt_camera_t *cam)
 {
   CameraEventType event;
   gpointer data;
-  gboolean wait_timedout=FALSE;
-  while( !wait_timedout )
+  if( gp_camera_wait_for_event( cam->gpcam, 30, &event, &data, c->gpcontext ) == GP_OK )
   {
-    if( gp_camera_wait_for_event( cam->gpcam, 100, &event, &data, c->gpcontext ) == GP_OK )
+    if( event == GP_EVENT_UNKNOWN )
     {
-      if( event == GP_EVENT_UNKNOWN )
+      /* this is really some undefined behavior, seems like its
+      camera driver dependent... very ugly! */
+      if( strstr( (char *)data, "4006" ) || // Nikon PTP driver
+          (strstr((char *)data, "PTP Property") && strstr((char *)data, "changed"))  // Some Canon driver maybe all ??
+        )
       {
-	/* this is really some undefined behavior, seems like its
-	 camera driver dependent... very ugly! */
-        if( strstr( (char *)data, "4006" ) || // Nikon PTP driver
-	    (strstr((char *)data, "PTP Property") && strstr((char *)data, "changed"))  // Some Canon driver maybe all ??
-
-	    )
-        {
-          // Property change event occured on camera
-          // let's update cache and signalling
-	  dt_print(DT_DEBUG_CAMCTL, "[camera_control] Camera configuration change event, lets update internal configuration cache.\n");
-          _camera_configuration_update(c,cam);
-        }
+        // Property change event occured on camera
+        // let's update cache and signalling
+        dt_print(DT_DEBUG_CAMCTL, "[camera_control] Camera configuration change event, lets update internal configuration cache.\n");
+        _camera_configuration_update(c,cam);
       }
-      else if( event == GP_EVENT_FILE_ADDED )
-      {
-        if( cam->is_tethering )
-        {
-          dt_print(DT_DEBUG_CAMCTL,"[camera_control] Camera file added event\n");
-          CameraFilePath *fp = (CameraFilePath *)data;
-          CameraFile *destination;
-          const char *output_path = _dispatch_request_image_path(c,cam);
-          if( !output_path ) output_path="/tmp";
-          const char *fname = _dispatch_request_image_filename(c,fp->name,cam);
-          if( !fname ) fname=fp->name;
-
-          char *output = g_build_filename(output_path,fname,(char *)NULL);
-
-          int handle = open( output, O_CREAT | O_WRONLY,0666);
-          gp_file_new_from_fd( &destination , handle );
-          gp_camera_file_get( cam->gpcam, fp->folder , fp->name, GP_FILE_TYPE_NORMAL, destination,  c->gpcontext);
-          close( handle );
-
-          // Notify listerners of captured image
-          _dispatch_camera_image_downloaded(c,cam,output);
-          g_free(output);
-        }
-      }
-      else if( event == GP_EVENT_TIMEOUT )
-        wait_timedout=TRUE;
     }
-    else
+    else if( event == GP_EVENT_FILE_ADDED )
     {
-      wait_timedout=TRUE;
-      // dt_print(DT_DEBUG_CAMCTL,"[camera_control] Failed to wait for camera event\n");
+      if( cam->is_tethering )
+      {
+        dt_print(DT_DEBUG_CAMCTL,"[camera_control] Camera file added event\n");
+        CameraFilePath *fp = (CameraFilePath *)data;
+        CameraFile *destination;
+        const char *output_path = _dispatch_request_image_path(c,cam);
+        if( !output_path ) output_path="/tmp";
+        const char *fname = _dispatch_request_image_filename(c,fp->name,cam);
+        if( !fname ) fname=fp->name;
 
+        char *output = g_build_filename(output_path,fname,(char *)NULL);
+
+        int handle = open( output, O_CREAT | O_WRONLY,0666);
+        gp_file_new_from_fd( &destination , handle );
+        gp_camera_file_get( cam->gpcam, fp->folder , fp->name, GP_FILE_TYPE_NORMAL, destination,  c->gpcontext);
+        close( handle );
+
+        // Notify listerners of captured image
+        _dispatch_camera_image_downloaded(c,cam,output);
+        g_free(output);
+      }
     }
   }
 }
@@ -1148,6 +1300,7 @@ const char *_dispatch_request_image_filename(const dt_camctl_t *c,const char *fi
   dt_camctl_t *camctl=(dt_camctl_t *)c;
   GList *listener;
   const char *path=NULL;
+  dt_pthread_mutex_lock(&camctl->listeners_lock);
   if((listener=g_list_first(camctl->listeners))!=NULL)
     do
     {
@@ -1155,6 +1308,7 @@ const char *_dispatch_request_image_filename(const dt_camctl_t *c,const char *fi
         path=((dt_camctl_listener_t*)listener->data)->request_image_filename(camera,filename,((dt_camctl_listener_t*)listener->data)->data);
     }
     while((listener=g_list_next(listener))!=NULL);
+  dt_pthread_mutex_unlock(&camctl->listeners_lock);
   return path;
 }
 
@@ -1164,6 +1318,7 @@ const char *_dispatch_request_image_path(const dt_camctl_t *c,const dt_camera_t
   dt_camctl_t *camctl=(dt_camctl_t *)c;
   GList *listener;
   const char *path=NULL;
+  dt_pthread_mutex_lock(&camctl->listeners_lock);
   if((listener=g_list_first(camctl->listeners))!=NULL)
     do
     {
@@ -1171,6 +1326,7 @@ const char *_dispatch_request_image_path(const dt_camctl_t *c,const dt_camera_t
         path=((dt_camctl_listener_t*)listener->data)->request_image_path(camera,((dt_camctl_listener_t*)listener->data)->data);
     }
     while((listener=g_list_next(listener))!=NULL);
+  dt_pthread_mutex_unlock(&camctl->listeners_lock);
   return path;
 }
 
@@ -1178,6 +1334,7 @@ void _dispatch_camera_connected(const dt_camctl_t *c,const dt_camera_t *camera)
 {
   dt_camctl_t *camctl=(dt_camctl_t *)c;
   GList *listener;
+  dt_pthread_mutex_lock(&camctl->listeners_lock);
   if((listener=g_list_first(camctl->listeners))!=NULL)
     do
     {
@@ -1185,12 +1342,14 @@ void _dispatch_camera_connected(const dt_camctl_t *c,const dt_camera_t *camera)
         ((dt_camctl_listener_t*)listener->data)->camera_connected(camera,((dt_camctl_listener_t*)listener->data)->data);
     }
     while((listener=g_list_next(listener))!=NULL);
+  dt_pthread_mutex_unlock(&camctl->listeners_lock);
 }
 
 void _dispatch_camera_disconnected(const dt_camctl_t *c,const dt_camera_t *camera)
 {
   dt_camctl_t *camctl=(dt_camctl_t *)c;
   GList *listener;
+  dt_pthread_mutex_lock(&camctl->listeners_lock);
   if((listener=g_list_first(camctl->listeners))!=NULL)
     do
     {
@@ -1198,6 +1357,7 @@ void _dispatch_camera_disconnected(const dt_camctl_t *c,const dt_camera_t *camer
         ((dt_camctl_listener_t*)listener->data)->camera_disconnected(camera,((dt_camctl_listener_t*)listener->data)->data);
     }
     while((listener=g_list_next(listener))!=NULL);
+  dt_pthread_mutex_unlock(&camctl->listeners_lock);
 }
 
 
@@ -1205,6 +1365,7 @@ void _dispatch_camera_image_downloaded(const dt_camctl_t *c,const dt_camera_t *c
 {
   dt_camctl_t *camctl=(dt_camctl_t *)c;
   GList *listener;
+  dt_pthread_mutex_lock(&camctl->listeners_lock);
   if((listener=g_list_first(camctl->listeners))!=NULL)
     do
     {
@@ -1212,6 +1373,7 @@ void _dispatch_camera_image_downloaded(const dt_camctl_t *c,const dt_camera_t *c
         ((dt_camctl_listener_t*)listener->data)->image_downloaded(camera,filename,((dt_camctl_listener_t*)listener->data)->data);
     }
     while((listener=g_list_next(listener))!=NULL);
+  dt_pthread_mutex_unlock(&camctl->listeners_lock);
 }
 
 int _dispatch_camera_storage_image_filename(const dt_camctl_t *c,const dt_camera_t *camera,const char *filename,CameraFile *preview,CameraFile *exif)
@@ -1219,6 +1381,7 @@ int _dispatch_camera_storage_image_filename(const dt_camctl_t *c,const dt_camera
   int res=0;
   dt_camctl_t *camctl=(dt_camctl_t *)c;
   GList *listener;
+  dt_pthread_mutex_lock(&camctl->listeners_lock);
   if((listener=g_list_first(camctl->listeners))!=NULL)
     do
     {
@@ -1226,6 +1389,7 @@ int _dispatch_camera_storage_image_filename(const dt_camctl_t *c,const dt_camera
         res=((dt_camctl_listener_t*)listener->data)->camera_storage_image_filename(camera,filename,preview,exif,((dt_camctl_listener_t*)listener->data)->data);
     }
     while((listener=g_list_next(listener))!=NULL);
+  dt_pthread_mutex_unlock(&camctl->listeners_lock);
   return res;
 }
 
@@ -1233,6 +1397,7 @@ void _dispatch_control_status(const dt_camctl_t *c,dt_camctl_status_t status)
 {
   dt_camctl_t *camctl=(dt_camctl_t *)c;
   GList *listener;
+  dt_pthread_mutex_lock(&camctl->listeners_lock);
   if((listener=g_list_first(camctl->listeners))!=NULL)
     do
     {
@@ -1240,12 +1405,14 @@ void _dispatch_control_status(const dt_camctl_t *c,dt_camctl_status_t status)
         ((dt_camctl_listener_t*)listener->data)->control_status(status,((dt_camctl_listener_t*)listener->data)->data);
     }
     while((listener=g_list_next(listener))!=NULL);
+  dt_pthread_mutex_unlock(&camctl->listeners_lock);
 }
 
 void _dispatch_camera_property_value_changed(const dt_camctl_t *c,const dt_camera_t *camera,const char *name,const char *value)
 {
   dt_camctl_t *camctl=(dt_camctl_t *)c;
   GList *listener;
+  dt_pthread_mutex_lock(&camctl->listeners_lock);
   if((listener=g_list_first(camctl->listeners))!=NULL)
     do
     {
@@ -1253,12 +1420,14 @@ void _dispatch_camera_property_value_changed(const dt_camctl_t *c,const dt_camer
         ((dt_camctl_listener_t*)listener->data)->camera_property_value_changed(camera,name,value,((dt_camctl_listener_t*)listener->data)->data);
     }
     while((listener=g_list_next(listener))!=NULL);
+  dt_pthread_mutex_unlock(&camctl->listeners_lock);
 }
 
 void _dispatch_camera_property_accessibility_changed(const dt_camctl_t *c,const dt_camera_t *camera,const char *name,gboolean read_only)
 {
   dt_camctl_t *camctl=(dt_camctl_t *)c;
   GList *listener;
+  dt_pthread_mutex_lock(&camctl->listeners_lock);
   if((listener=g_list_first(camctl->listeners))!=NULL)
     do
     {
@@ -1266,12 +1435,14 @@ void _dispatch_camera_property_accessibility_changed(const dt_camctl_t *c,const
         ((dt_camctl_listener_t*)listener->data)->camera_property_accessibility_changed(camera,name,read_only,((dt_camctl_listener_t*)listener->data)->data);
     }
     while((listener=g_list_next(listener))!=NULL);
+  dt_pthread_mutex_unlock(&camctl->listeners_lock);
 }
 
 void _dispatch_camera_error(const dt_camctl_t *c,const dt_camera_t *camera,dt_camera_error_t error)
 {
   dt_camctl_t *camctl=(dt_camctl_t *)c;
   GList *listener;
+  dt_pthread_mutex_lock(&camctl->listeners_lock);
   if((listener=g_list_first(camctl->listeners))!=NULL)
     do
     {
@@ -1279,6 +1450,9 @@ void _dispatch_camera_error(const dt_camctl_t *c,const dt_camera_t *camera,dt_ca
         ((dt_camctl_listener_t*)listener->data)->camera_error(camera,error,((dt_camctl_listener_t*)listener->data)->data);
     }
     while((listener=g_list_next(listener))!=NULL);
+  dt_pthread_mutex_unlock(&camctl->listeners_lock);
 }
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
