@@ -1,6 +1,6 @@
 /*
    This file is part of darktable,
-   copyright (c) 2009--2010 johannes hanika.
+   copyright (c) 2009--2013 johannes hanika.
    copyright (c) 2011 henrik andersson.
    copyright (c) 2012 tobias ellinghaus.
 
@@ -438,9 +438,12 @@ static bool dt_exif_read_exif_data(dt_image_t *img, Exiv2::ExifData &exifData)
     {
       img->orientation = dt_image_orientation_to_flip_bits(pos->toLong());
     }
-    /* sony has its own rotation */
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.MinoltaCs7D.Rotation")))
-         != exifData.end() && pos->size())
+
+    /* minolta and sony have their own rotation */
+    if ( ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.MinoltaCs5D.Rotation")))
+           != exifData.end() && pos->size()) ||
+         ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.MinoltaCs7D.Rotation")))
+           != exifData.end() && pos->size()) )
     {
       switch(pos->toLong())
       {
@@ -460,24 +463,30 @@ static bool dt_exif_read_exif_data(dt_image_t *img, Exiv2::ExifData &exifData)
          != exifData.end() && pos->size())
     {
       Exiv2::ExifData::const_iterator ref = exifData.findKey(Exiv2::ExifKey("Exif.GPSInfo.GPSLatitudeRef"));
-      const char *sign = ref->toString().c_str();
-      double latitude = 0.0;
-      if(_gps_rationale_to_number(pos->toRational(0).first, pos->toRational(0).second,
-                                  pos->toRational(1).first, pos->toRational(1).second,
-                                  pos->toRational(2).first, pos->toRational(2).second, sign[0], &latitude))
-        img->latitude = latitude;
+      if(ref != exifData.end() && ref->size())
+      {
+        const char *sign = ref->toString().c_str();
+        double latitude = 0.0;
+        if(_gps_rationale_to_number(pos->toRational(0).first, pos->toRational(0).second,
+                                    pos->toRational(1).first, pos->toRational(1).second,
+                                    pos->toRational(2).first, pos->toRational(2).second, sign[0], &latitude))
+          img->latitude = latitude;
+      }
     }
 
     if ( (pos = exifData.findKey(Exiv2::ExifKey("Exif.GPSInfo.GPSLongitude")))
          != exifData.end() && pos->size())
     {
       Exiv2::ExifData::const_iterator ref = exifData.findKey(Exiv2::ExifKey("Exif.GPSInfo.GPSLongitudeRef"));
-      const char *sign = ref->toString().c_str();
-      double longitude = 0.0;
-      if(_gps_rationale_to_number(pos->toRational(0).first, pos->toRational(0).second,
-                                  pos->toRational(1).first, pos->toRational(1).second,
-                                  pos->toRational(2).first, pos->toRational(2).second, sign[0], &longitude))
-        img->longitude = longitude;
+      if(ref != exifData.end() && ref->size())
+      {
+        const char *sign = ref->toString().c_str();
+        double longitude = 0.0;
+        if(_gps_rationale_to_number(pos->toRational(0).first, pos->toRational(0).second,
+                                    pos->toRational(1).first, pos->toRational(1).second,
+                                    pos->toRational(2).first, pos->toRational(2).second, sign[0], &longitude))
+          img->longitude = longitude;
+      }
     }
 
     /* Read lens name */
@@ -802,7 +811,14 @@ int dt_exif_write_blob(uint8_t *blob,uint32_t size, const char* path)
   return 1;
 }
 
-int dt_exif_read_blob(uint8_t *buf, const char* path, const int imgid, const int sRGB, const int out_width, const int out_height)
+int dt_exif_read_blob(
+    uint8_t    *buf,
+    const char *path,
+    const int   imgid,
+    const int   sRGB,
+    const int   out_width,
+    const int   out_height,
+    const int   dng_mode)
 {
   try
   {
@@ -811,16 +827,7 @@ int dt_exif_read_blob(uint8_t *buf, const char* path, const int imgid, const int
     assert(image.get() != 0);
     image->readMetadata();
     Exiv2::ExifData &exifData = image->exifData();
-    //     Exiv2::XmpData &xmpData = image->xmpData();  // TODO: I'm not sure how to embed xmp data into the blob.
-
-    /* Dont bail, lets return a blob with UserComment and Software
-       if (exifData.empty())
-       {
-       std::string error(path);
-       error += ": no exif data found in ";
-       error += path;
-       throw Exiv2::Error(1, error);
-       }*/
+    // needs to be reset, even in dng mode, as the buffers are flipped during raw import
     exifData["Exif.Image.Orientation"] = uint16_t(1);
 
     // ufraw-style exif stripping:
@@ -869,111 +876,114 @@ int dt_exif_read_blob(uint8_t *buf, const char* path, const int imgid, const int
          != exifData.end() )
       exifData.erase(pos);
 
-    /* Delete various MakerNote fields only applicable to the raw file */
+    if(!dng_mode)
+    {
+      /* Delete various MakerNote fields only applicable to the raw file */
 
-    // Nikon thumbnail data
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Nikon3.Preview")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.NikonPreview.JPEGInterchangeFormat")))
-         != exifData.end() )
-      exifData.erase(pos);
+      // Nikon thumbnail data
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Nikon3.Preview")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.NikonPreview.JPEGInterchangeFormat")))
+           != exifData.end() )
+        exifData.erase(pos);
 
-    // DNG private data
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Image.DNGPrivateData")))
-         != exifData.end() )
-      exifData.erase(pos);
+      // DNG private data
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Image.DNGPrivateData")))
+           != exifData.end() )
+        exifData.erase(pos);
 
-    // Pentax thumbnail data
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Pentax.PreviewResolution")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Pentax.PreviewLength")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Pentax.PreviewOffset")))
-         != exifData.end() )
-      exifData.erase(pos);
+      // Pentax thumbnail data
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Pentax.PreviewResolution")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Pentax.PreviewLength")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Pentax.PreviewOffset")))
+           != exifData.end() )
+        exifData.erase(pos);
 
-    // Minolta thumbnail data
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Minolta.Thumbnail")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Minolta.ThumbnailOffset")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Minolta.ThumbnailLength")))
-         != exifData.end() )
-      exifData.erase(pos);
+      // Minolta thumbnail data
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Minolta.Thumbnail")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Minolta.ThumbnailOffset")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Minolta.ThumbnailLength")))
+           != exifData.end() )
+        exifData.erase(pos);
 
-    // Olympus thumbnail data
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Olympus.Thumbnail")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Olympus.ThumbnailOffset")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Olympus.ThumbnailLength")))
-         != exifData.end() )
-      exifData.erase(pos);
+      // Olympus thumbnail data
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Olympus.Thumbnail")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Olympus.ThumbnailOffset")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Olympus.ThumbnailLength")))
+           != exifData.end() )
+        exifData.erase(pos);
 
 #if EXIV2_MINOR_VERSION>=23
-    // Exiv2 versions older than 0.23 drop all EXIF if the code below is executed
-    // Samsung makernote cleanup, the entries below have no relevance for exported images
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.SensorAreas")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.ColorSpace")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.EncryptionKey")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.WB_RGGBLevelsUncorrected")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.WB_RGGBLevelsAuto")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.WB_RGGBLevelsIlluminator1")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.WB_RGGBLevelsIlluminator2")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.WB_RGGBLevelsBlack")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.ColorMatrix")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.ColorMatrixSRGB")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.ColorMatrixAdobeRGB")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.ToneCurve1")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.ToneCurve2")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.ToneCurve3")))
-         != exifData.end() )
-      exifData.erase(pos);
-    if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.ToneCurve4")))
-         != exifData.end() )
-      exifData.erase(pos);
+      // Exiv2 versions older than 0.23 drop all EXIF if the code below is executed
+      // Samsung makernote cleanup, the entries below have no relevance for exported images
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.SensorAreas")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.ColorSpace")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.EncryptionKey")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.WB_RGGBLevelsUncorrected")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.WB_RGGBLevelsAuto")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.WB_RGGBLevelsIlluminator1")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.WB_RGGBLevelsIlluminator2")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.WB_RGGBLevelsBlack")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.ColorMatrix")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.ColorMatrixSRGB")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.ColorMatrixAdobeRGB")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.ToneCurve1")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.ToneCurve2")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.ToneCurve3")))
+           != exifData.end() )
+        exifData.erase(pos);
+      if ( (pos=exifData.findKey(Exiv2::ExifKey("Exif.Samsung2.ToneCurve4")))
+           != exifData.end() )
+        exifData.erase(pos);
 #endif
 
-    /* Write appropriate color space tag if using sRGB output */
-    if (sRGB)
-      exifData["Exif.Photo.ColorSpace"] = uint16_t(1);      /* sRGB */
-    else
-      exifData["Exif.Photo.ColorSpace"] = uint16_t(0xFFFF); /* Uncalibrated */
+      /* Write appropriate color space tag if using sRGB output */
+      if (sRGB)
+        exifData["Exif.Photo.ColorSpace"] = uint16_t(1);      /* sRGB */
+      else
+        exifData["Exif.Photo.ColorSpace"] = uint16_t(0xFFFF); /* Uncalibrated */
+    }
 
-    /* Replace RAW dimension with output dimensions (for example after crop/scale) */
+    /* Replace RAW dimension with output dimensions (for example after crop/scale, or orientation for dng mode) */
     if (out_width > 0)
       exifData["Exif.Photo.PixelXDimension"] = out_width;
     if (out_height > 0)
@@ -989,7 +999,6 @@ int dt_exif_read_blob(uint8_t *buf, const char* path, const int imgid, const int
       if(res != NULL)
       {
         exifData["Exif.Image.Artist"] = (char*)res->data;
-        //         xmpData["Xmp.dc.creator"] = (char*)res->data;
         g_free(res->data);
         g_list_free(res);
       }
@@ -998,7 +1007,6 @@ int dt_exif_read_blob(uint8_t *buf, const char* path, const int imgid, const int
       if(res != NULL)
       {
         exifData["Exif.Image.ImageDescription"] = (char*)res->data;
-        //         xmpData["Xmp.dc.title"] = (char*)res->data;
         g_free(res->data);
         g_list_free(res);
       }
@@ -1007,7 +1015,6 @@ int dt_exif_read_blob(uint8_t *buf, const char* path, const int imgid, const int
       if(res != NULL)
       {
         exifData["Exif.Photo.UserComment"] = (char*)res->data;
-        //         xmpData["Xmp.dc.description"] = (char*)res->data;
         g_free(res->data);
         g_list_free(res);
       }
@@ -1016,7 +1023,6 @@ int dt_exif_read_blob(uint8_t *buf, const char* path, const int imgid, const int
       if(res != NULL)
       {
         exifData["Exif.Image.Copyright"] = (char*)res->data;
-        //         xmpData["Xmp.dc.rights"] = (char*)res->data;
         g_free(res->data);
         g_list_free(res);
       }
@@ -1027,7 +1033,6 @@ int dt_exif_read_blob(uint8_t *buf, const char* path, const int imgid, const int
         int rating = GPOINTER_TO_INT(res->data)+1;
         exifData["Exif.Image.Rating"] = rating;
         exifData["Exif.Image.RatingPercent"] = int(rating/5.*100.);
-        //         xmpData["Xmp.xmp.Rating"] = rating;
         g_list_free(res);
       }
 
@@ -1344,7 +1349,7 @@ int dt_exif_xmp_read (dt_image_t *img, const char* filename, const int history_o
           /* check if we got blendop from xmp */
           unsigned char *blendop_params = NULL;
           unsigned int blendop_size = 0;
-          if(blendop != xmpData.end() && blendop->size() > 0 && blendop->toString(i).c_str() != NULL)
+          if(blendop != xmpData.end() && blendop->size() > 0 && blendop->count () > i && blendop->toString(i).c_str() != NULL)
           {
             blendop_size = strlen(blendop->toString(i).c_str())/2;
             blendop_params = (unsigned char *)malloc(blendop_size);
@@ -1356,7 +1361,7 @@ int dt_exif_xmp_read (dt_image_t *img, const char* filename, const int history_o
 
           /* check if we got blendop_version from xmp; if not assume 1 as default */
           int blversion = 1;
-          if(blendop_version != xmpData.end())
+          if(blendop_version != xmpData.end() && blendop_version->count() > i)
           {
             blversion = blendop_version->toLong(i);
           }
@@ -1562,6 +1567,7 @@ dt_exif_xmp_read_data(Exiv2::XmpData &xmpData, const int imgid)
     xmpData.add(Exiv2::XmpKey(key), &tv);
 
     const char *op = (const char *)sqlite3_column_text(stmt, 3);
+    if(!op) continue; // no op is fatal.
     tv.read(op);
     snprintf(key, 1024, "Xmp.darktable.history_operation[%d]", num);
     xmpData.add(Exiv2::XmpKey(key), &tv);
@@ -1576,9 +1582,11 @@ dt_exif_xmp_read_data(Exiv2::XmpData &xmpData, const int imgid)
     free(vparams);
 
     /* read and add blendop params */
+    const void *blob = sqlite3_column_blob(stmt, 6);
+    if(!blob) continue; // no params, no history item.
     len = sqlite3_column_bytes(stmt, 6);
     vparams = (char *)malloc(2*len + 1);
-    dt_exif_xmp_encode ((const unsigned char *)sqlite3_column_blob(stmt, 6), vparams, len);
+    dt_exif_xmp_encode ((const unsigned char *)blob, vparams, len);
     tv.read(vparams);
     snprintf(key, 1024, "Xmp.darktable.blendop_params[%d]", num);
     xmpData.add(Exiv2::XmpKey(key), &tv);
