@@ -22,6 +22,9 @@
 #include <gtk/gtk.h>
 #include <inttypes.h>
 
+/** Flag for the format modules */
+#define FORMAT_FLAGS_SUPPORT_XMP   1
+
 /**
  * defines the plugin structure for image import and export.
  *
@@ -98,6 +101,9 @@ typedef struct dt_imageio_module_format_t
   /* write to file, with exif if not NULL, and icc profile if supported. */
   int (*write_image)(dt_imageio_module_data_t *data, const char *filename, const void *in, void *exif, int exif_len, int imgid);
 
+  // sometimes we want to tell the world about what we can do
+  int (*flags)();
+
   // reading functions:
   /* read header from file, get width and height */
   int (*read_header)(const char *filename, dt_imageio_module_data_t *data);
@@ -137,7 +143,7 @@ typedef struct dt_imageio_module_storage_t
   int (*recommended_dimension)    (struct dt_imageio_module_storage_t *self, uint32_t *width, uint32_t *height);
 
   /* this actually does the work */
-  int (*store)(struct dt_imageio_module_data_t *self, const int imgid, dt_imageio_module_format_t *format, dt_imageio_module_data_t *fdata, const int num, const int total);
+  int (*store)(struct dt_imageio_module_data_t *self, const int imgid, dt_imageio_module_format_t *format, dt_imageio_module_data_t *fdata, const int num, const int total, const gboolean high_quality);
   /* called once at the end (after exporting all images), if implemented. */
   int (*finalize_store) (struct dt_imageio_module_storage_t *self, dt_imageio_module_data_t *data);
 
@@ -172,4 +178,11 @@ dt_imageio_module_storage_t *dt_imageio_get_storage();
 dt_imageio_module_format_t *dt_imageio_get_format_by_name(const char *name);
 dt_imageio_module_storage_t *dt_imageio_get_storage_by_name(const char *name);
 
+/* get by index */
+dt_imageio_module_format_t *dt_imageio_get_format_by_index(int index);
+dt_imageio_module_storage_t *dt_imageio_get_storage_by_index(int index);
+
 #endif
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
