@@ -22,7 +22,9 @@
 
 // lamer version for obsolete archs:
 
-#ifndef _XOPEN_SOURCE
+#if !defined _XOPEN_SOURCE && \
+    !defined(__DragonFly__) && !defined(__FreeBSD__) && \
+    !defined(__NetBSD__) && !defined(__OpenBSD__)
 #define _XOPEN_SOURCE
 #endif
 
@@ -291,13 +293,13 @@ typedef unsigned __int64 uint64_t;
 
 static inline uint32_t gen_rand32(struct sfmt_state_t *s);
 static inline uint64_t gen_rand64(struct sfmt_state_t *s);
-static inline void fill_array32(struct sfmt_state_t *s, uint32_t *array, int size);
-static inline void fill_array64(struct sfmt_state_t *s, uint64_t *array, int size);
-static inline void init_gen_rand(struct sfmt_state_t *s, uint32_t seed);
-static inline void init_by_array(struct sfmt_state_t *s, uint32_t *init_key, int key_length);
-static inline const char *get_idstring(void);
-static inline int get_min_array_size32(void);
-static inline int get_min_array_size64(void);
+static inline void fill_array32(struct sfmt_state_t *s, uint32_t *array, int size) __attribute__ ((unused));
+static inline void fill_array64(struct sfmt_state_t *s, uint64_t *array, int size) __attribute__ ((unused));
+static inline void init_gen_rand(struct sfmt_state_t *s, uint32_t seed) __attribute__ ((unused));
+static inline void init_by_array(struct sfmt_state_t *s, uint32_t *init_key, int key_length) __attribute__ ((unused));
+static inline const char *get_idstring(void) __attribute__ ((unused));
+static inline int get_min_array_size32(void) __attribute__ ((unused));
+static inline int get_min_array_size64(void) __attribute__ ((unused));
 
 /* These real versions are due to Isaku Wada */
 /** generates a random number on [0,1]-real-interval */
@@ -1133,3 +1135,6 @@ static inline float dt_points_get()
 
 #endif
 #endif
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
