@@ -43,9 +43,12 @@ gboolean dt_tag_exists(const char *name,guint *tagid);
 /** attach a list of tags on selected images. \param[in] tagid id of tag to attach. \param[in] imgid the image id to attach tag to, if < 0 selected images are used. */
 void dt_tag_attach(guint tagid,gint imgid);
 
-/** attach a list of tags on selected images. \param[in] tags a list of strings of tags. \param[in] imgid the image id to attach tag to, if < 0 selected images are used. \note If tag not exists it's created.*/
+/** attach a list of tags on selected images. \param[in] tags a list of ids of tags. \param[in] imgid the image id to attach tag to, if < 0 selected images are used. \note If tag not exists it's created.*/
 void dt_tag_attach_list(GList *tags,gint imgid);
 
+/** attach a list of tags on selected images. \param[in] tags a comma separated string of tags. \param[in] imgid the image id to attach tag to, if < 0 selected images are used. \note If tag not exists it's created.*/
+void dt_tag_attach_string_list(const gchar *tags, gint imgid);
+
 /** detach tag from images. \param[in] tagid if of tag to deattach. \param[in] imgid the image id to attach tag from, if < 0 selected images are used. */
 void dt_tag_detach(guint tagid,gint imgid);
 
@@ -74,3 +77,6 @@ void dt_tag_free_result(GList **result);
 void dt_tag_reorganize(const gchar *source, const gchar *dest);
 
 #endif
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
