@@ -96,7 +96,7 @@ static inline void _HSL_2_RGB(const float *HSL, float *RGB)
     if (L < 0.5f) var_2 = L * (1.0f + S);
     else          var_2 = (L + S) - (S * L);
 
-    var_1 = 2 * L - var_2;
+    var_1 = 2.0f * L - var_2;
 
     RGB[0] = _Hue_2_RGB(var_1, var_2, H + (1.0f / 3.0f));
     RGB[1] = _Hue_2_RGB(var_1, var_2, H);
@@ -133,6 +133,12 @@ static inline void _CLAMP_XYZ(float *XYZ, const float *min, const float *max)
   XYZ[2] = CLAMP_RANGE(XYZ[2], min[2], max[2]);
 }
 
+static inline void _PX_COPY(const float *src, float *dst)
+{
+  dst[0] = src[0];
+  dst[1] = src[1];
+  dst[2] = src[2];
+}
 
 
 
@@ -1273,7 +1279,12 @@ static void _blend_lightness(dt_iop_colorspace_type_t cst,const float *a, float
     }
     else if(cst==iop_cs_rgb)
     {
-      _RGB_2_HSL(&a[j], tta);
+      _PX_COPY(&a[j], ta);
+
+      _CLAMP_XYZ(ta, min, max);
+      _CLAMP_XYZ(&b[j], min, max);
+
+      _RGB_2_HSL(ta, tta);
       _RGB_2_HSL(&b[j], ttb);
 
       ttb[0] = tta[0];
@@ -1309,8 +1320,11 @@ static void _blend_chroma(dt_iop_colorspace_type_t cst,const float *a, float *b,
     if(cst==iop_cs_Lab)
     {
       _blend_Lab_scale(&a[j], ta);
-      _blend_Lab_scale(&b[j], tb);
+      _CLAMP_XYZ(ta, min, max);
       _Lab_2_LCH(ta, tta);
+
+      _blend_Lab_scale(&b[j], tb);
+      _CLAMP_XYZ(tb, min, max);
       _Lab_2_LCH(tb, ttb);
 
       ttb[0] = tta[0];
@@ -1323,7 +1337,12 @@ static void _blend_chroma(dt_iop_colorspace_type_t cst,const float *a, float *b,
     }
     else if(cst==iop_cs_rgb)
     {
-      _RGB_2_HSL(&a[j], tta);
+      _PX_COPY(&a[j], ta);
+
+      _CLAMP_XYZ(ta, min, max);
+      _CLAMP_XYZ(&b[j], min, max);
+
+      _RGB_2_HSL(ta, tta);
       _RGB_2_HSL(&b[j], ttb);
 
       ttb[0] = tta[0];
@@ -1359,9 +1378,11 @@ static void _blend_hue(dt_iop_colorspace_type_t cst,const float *a, float *b, co
     if(cst==iop_cs_Lab)
     {
       _blend_Lab_scale(&a[j], ta);
-      _blend_Lab_scale(&b[j], tb);
-
+      _CLAMP_XYZ(ta, min, max);
       _Lab_2_LCH(ta, tta);
+
+      _blend_Lab_scale(&b[j], tb);
+      _CLAMP_XYZ(tb, min, max);
       _Lab_2_LCH(tb, ttb);
 
       ttb[0] = tta[0];
@@ -1377,7 +1398,12 @@ static void _blend_hue(dt_iop_colorspace_type_t cst,const float *a, float *b, co
     }
     else if(cst==iop_cs_rgb)
     {
-      _RGB_2_HSL(&a[j], tta);
+      _PX_COPY(&a[j], ta);
+
+      _CLAMP_XYZ(ta, min, max);
+      _CLAMP_XYZ(&b[j], min, max);
+
+      _RGB_2_HSL(ta, tta);
       _RGB_2_HSL(&b[j], ttb);
 
       /* blend hue along shortest distance on color circle */
@@ -1416,9 +1442,11 @@ static void _blend_color(dt_iop_colorspace_type_t cst,const float *a, float *b,
     if(cst==iop_cs_Lab)
     {
       _blend_Lab_scale(&a[j], ta);
-      _blend_Lab_scale(&b[j], tb);
-
+      _CLAMP_XYZ(ta, min, max);
       _Lab_2_LCH(ta, tta);
+
+      _blend_Lab_scale(&b[j], tb);
+      _CLAMP_XYZ(tb, min, max);
       _Lab_2_LCH(tb, ttb);
 
       ttb[0] = tta[0];
@@ -1435,7 +1463,12 @@ static void _blend_color(dt_iop_colorspace_type_t cst,const float *a, float *b,
     }
     else if(cst==iop_cs_rgb)
     {
-      _RGB_2_HSL(&a[j], tta);
+      _PX_COPY(&a[j], ta);
+
+      _CLAMP_XYZ(ta, min, max);
+      _CLAMP_XYZ(&b[j], min, max);
+
+      _RGB_2_HSL(ta, tta);
       _RGB_2_HSL(&b[j], ttb);
 
       /* blend hue along shortest distance on color circle */
@@ -1474,9 +1507,11 @@ static void _blend_coloradjust(dt_iop_colorspace_type_t cst,const float *a, floa
     if(cst==iop_cs_Lab)
     {
       _blend_Lab_scale(&a[j], ta);
-      _blend_Lab_scale(&b[j], tb);
-
+      _CLAMP_XYZ(ta, min, max);
       _Lab_2_LCH(ta, tta);
+
+      _blend_Lab_scale(&b[j], tb);
+      _CLAMP_XYZ(tb, min, max);
       _Lab_2_LCH(tb, ttb);
 
       // ttb[0] (output lightness) unchanged
@@ -1493,6 +1528,11 @@ static void _blend_coloradjust(dt_iop_colorspace_type_t cst,const float *a, floa
     }
     else if(cst==iop_cs_rgb)
     {
+      _PX_COPY(&a[j], ta);
+
+      _CLAMP_XYZ(ta, min, max);
+      _CLAMP_XYZ(&b[j], min, max);
+
       _RGB_2_HSL(&a[j], tta);
       _RGB_2_HSL(&b[j], ttb);
 
