@@ -27,6 +27,7 @@
 #include "common/imageio.h"
 #include "common/tags.h"
 #include "common/debug.h"
+#include "common/similarity.h"
 #include "gui/gtk.h"
 
 #include <glib/gprintf.h>
@@ -38,12 +39,18 @@
 #include <assert.h>
 
 
+#define DT_DEV_AVERAGE_DELAY_START            250
+#define DT_DEV_PREVIEW_AVERAGE_DELAY_START     50
+#define DT_DEV_AVERAGE_DELAY_COUNT              5
+
 void dt_dev_init(dt_develop_t *dev, int32_t gui_attached)
 {
   memset(dev,0,sizeof(dt_develop_t));
   dev->preview_downsampling = 1.0f;
   dev->gui_module = NULL;
   dev->timestamp = 0;
+  dev->average_delay = DT_DEV_AVERAGE_DELAY_START;
+  dev->preview_average_delay = DT_DEV_PREVIEW_AVERAGE_DELAY_START;
   dev->gui_leaving = 0;
   dev->gui_synch = 0;
   dt_pthread_mutex_init(&dev->history_mutex, NULL);
@@ -64,13 +71,17 @@ void dt_dev_init(dt_develop_t *dev, int32_t gui_attached)
   dev->histogram = NULL;
   dev->histogram_pre_tonecurve = NULL;
   dev->histogram_pre_levels = NULL;
+  if(g_strcmp0(dt_conf_get_string("plugins/darkroom/histogram/mode"), "linear") == 0)
+    dev->histogram_linear = TRUE;
+  else
+    dev->histogram_linear = FALSE;
 
   if(dev->gui_attached)
   {
     dev->pipe = (dt_dev_pixelpipe_t *)malloc(sizeof(dt_dev_pixelpipe_t));
     dev->preview_pipe = (dt_dev_pixelpipe_t *)malloc(sizeof(dt_dev_pixelpipe_t));
     dt_dev_pixelpipe_init(dev->pipe);
-    dt_dev_pixelpipe_init(dev->preview_pipe);
+    dt_dev_pixelpipe_init_preview(dev->preview_pipe);
 
     dev->histogram = (float *)malloc(sizeof(float)*4*256);
     dev->histogram_pre_tonecurve = (float *)malloc(sizeof(float)*4*256);
@@ -164,7 +175,7 @@ void dt_dev_process_preview_job(dt_develop_t *dev)
   dev->preview_pipe->input_timestamp = dev->timestamp;
   dev->preview_dirty = 1;
 
-  // lock if there, issue a background load, if not (best-effort for mip f). 
+  // lock if there, issue a background load, if not (best-effort for mip f).
   dt_mipmap_cache_read_get(darktable.mipmap_cache, &buf, dev->image_storage.id, DT_MIPMAP_F, 0);
   if(!buf.buf)
   {
@@ -213,8 +224,12 @@ restart:
     else goto restart;
   }
   dt_show_times(&start, "[dev_process_preview] pixel pipeline processing", NULL);
+  dt_dev_average_delay_update(&start, &dev->preview_average_delay);
 
   dev->preview_dirty = 0;
+  // redraw the whole thing, to also update color picker values and histograms etc.
+  if(dev->gui_attached)
+    dt_control_queue_redraw();
   dt_control_log_busy_leave();
   dt_mipmap_cache_read_release(darktable.mipmap_cache, &buf);
 }
@@ -240,7 +255,7 @@ void dt_dev_process_image_job(dt_develop_t *dev)
 
   // failed to load raw?
   if(!buf.buf) return;
-  
+
   dt_dev_pixelpipe_set_input(dev->pipe, dev, (float *)buf.buf, buf.width, buf.height, 1.0);
 
   if(dev->image_loading)
@@ -302,6 +317,7 @@ restart:
     else goto restart;
   }
   dt_show_times(&start, "[dev_process_image] pixel pipeline processing", NULL);
+  dt_dev_average_delay_update(&start, &dev->average_delay);
 
   // maybe we got zoomed/panned in the meantime?
   if(dev->pipe->changed != DT_DEV_PIPE_UNCHANGED) goto restart;
@@ -311,7 +327,9 @@ restart:
   dev->image_loading = 0;
 
   dt_mipmap_cache_read_release(darktable.mipmap_cache, &buf);
-  dt_control_queue_redraw_center();
+  // redraw the whole thing, to also update color picker values and histograms etc.
+  if(dev->gui_attached)
+    dt_control_queue_redraw();
   dt_control_log_busy_leave();
 }
 
@@ -403,7 +421,7 @@ int dt_dev_write_history_item(const dt_image_t *image, dt_dev_history_item_t *h,
   }
   // printf("[dev write history item] writing %d - %s params %f %f\n", h->module->instance, h->module->op, *(float *)h->params, *(((float *)h->params)+1));
   sqlite3_finalize (stmt);
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "update history set operation = ?1, op_params = ?2, module = ?3, enabled = ?4, blendop_params = ?7 where imgid = ?5 and num = ?6", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "update history set operation = ?1, op_params = ?2, module = ?3, enabled = ?4, blendop_params = ?7, blendop_version = ?8 where imgid = ?5 and num = ?6", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, h->module->op, strlen(h->module->op), SQLITE_TRANSIENT);
   DT_DEBUG_SQLITE3_BIND_BLOB(stmt, 2, h->params, h->module->params_size, SQLITE_TRANSIENT);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 3, h->module->version());
@@ -411,6 +429,7 @@ int dt_dev_write_history_item(const dt_image_t *image, dt_dev_history_item_t *h,
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 5, image->id);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 6, num);
   DT_DEBUG_SQLITE3_BIND_BLOB(stmt, 7, h->blend_params, sizeof(dt_develop_blend_params_t), SQLITE_TRANSIENT);
+  DT_DEBUG_SQLITE3_BIND_INT(stmt, 8, dt_develop_blend_version());
 
   sqlite3_step (stmt);
   sqlite3_finalize (stmt);
@@ -511,6 +530,9 @@ void dt_dev_add_history_item(dt_develop_t *dev, dt_iop_module_t *module, gboolea
   }
 #endif
 
+  /* invalidate image data*/
+  dt_similarity_image_dirty(dev->image_storage.id);
+
   // invalidate buffers and force redraw of darkroom
   dt_dev_invalidate_all(dev);
   dt_pthread_mutex_unlock(&dev->history_mutex);
@@ -604,10 +626,10 @@ void dt_dev_write_history(dt_develop_t *dev)
     history = g_list_next(history);
     changed = TRUE;
   }
-  
+
   /* attach / detach changed tag reflecting actual change */
   guint tagid = 0;
-  dt_tag_new("darktable|changed",&tagid); 
+  dt_tag_new("darktable|changed",&tagid);
   if(changed)
     dt_tag_attach(tagid, dev->image_storage.id);
   else
@@ -615,17 +637,117 @@ void dt_dev_write_history(dt_develop_t *dev)
 
 }
 
+static void
+auto_apply_presets(const int imgid)
+{
+  if(imgid <= 0) return;
+  int run = 0;
+  const dt_image_t *cimg = dt_image_cache_read_get(darktable.image_cache, imgid);
+  if(!(cimg->flags & DT_IMAGE_AUTO_PRESETS_APPLIED))
+    run = 1;
+
+  // flag was already set? only apply presets once in the lifetime of a history stack.
+  // (the flag will be cleared when removing it)
+  if(!run || cimg->id <= 0)
+  {
+    dt_image_cache_read_release(darktable.image_cache, cimg);
+    return;
+  }
+
+  // keep locked, we want to be alone messing with the history of the poor fellow:
+  dt_image_t *image = dt_image_cache_write_get(darktable.image_cache, cimg);
+  // be extra sure that we don't mess up history in separate threads:
+  dt_pthread_mutex_lock(&darktable.db_insert);
+
+  // cleanup
+  DT_DEBUG_SQLITE3_EXEC(dt_database_get(darktable.db),
+      "delete from memory.history",
+      NULL, NULL, NULL);
+
+  const char *preset_table[2] = {"presets", "legacy_presets"};
+  const int legacy = (image->flags & DT_IMAGE_NO_LEGACY_PRESETS) ? 0 : 1;
+  char query[1024];
+  snprintf(query, 1024, 
+      "insert into memory.history select ?1, 0, op_version, operation, op_params, enabled, blendop_params, blendop_version "
+      "from %s where autoapply=1 and "
+                              "?2 like model and ?3 like maker and ?4 like lens and "
+                              "?5 between iso_min and iso_max and "
+                              "?6 between exposure_min and exposure_max and "
+                              "?7 between aperture_min and aperture_max and "
+                              "?8 between focal_length_min and focal_length_max and "
+                              "(isldr = 0 or isldr=?9) order by writeprotect desc, "
+                              "length(model), length(maker), length(lens)", preset_table[legacy]);
+
+  // query for all modules at once:
+  sqlite3_stmt *stmt;
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), query, -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, imgid);
+  DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 2, cimg->exif_model, strlen(cimg->exif_model), SQLITE_TRANSIENT);
+  DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 3, cimg->exif_maker, strlen(cimg->exif_maker), SQLITE_TRANSIENT);
+  DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 4, cimg->exif_lens,  strlen(cimg->exif_lens),  SQLITE_TRANSIENT);
+  DT_DEBUG_SQLITE3_BIND_DOUBLE(stmt, 5, fmaxf(0.0f, fminf(1000000, cimg->exif_iso)));
+  DT_DEBUG_SQLITE3_BIND_DOUBLE(stmt, 6, fmaxf(0.0f, fminf(1000000, cimg->exif_exposure)));
+  DT_DEBUG_SQLITE3_BIND_DOUBLE(stmt, 7, fmaxf(0.0f, fminf(1000000, cimg->exif_aperture)));
+  DT_DEBUG_SQLITE3_BIND_DOUBLE(stmt, 8, fmaxf(0.0f, fminf(1000000, cimg->exif_focal_length)));
+  // 0: dontcare, 1: ldr, 2: raw
+  DT_DEBUG_SQLITE3_BIND_DOUBLE(stmt, 9, 2-dt_image_is_ldr(cimg));
+
+  if(sqlite3_step(stmt) == SQLITE_DONE)
+  {
+    sqlite3_finalize(stmt);
+    int cnt = 0;
+    // count what we found:
+    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+        "select count(*) from memory.history",
+        -1, &stmt, NULL);
+    if(sqlite3_step(stmt) == SQLITE_ROW)
+    {
+      // if there is anything..
+      cnt = sqlite3_column_int(stmt, 0);
+      sqlite3_finalize(stmt);
+      // fprintf(stderr, "[auto_apply_presets] imageid %d found %d matching presets (legacy %d)\n", imgid, cnt, legacy);
+      // advance the current history by that amount:
+      DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+          "update history set num=num+?1 where imgid=?2",
+          -1, &stmt, NULL);
+      DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, cnt);
+      DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, imgid);
+      if(sqlite3_step(stmt) == SQLITE_DONE)
+      {
+        // and finally prepend the rest with increasing numbers (starting at 0)
+        sqlite3_finalize(stmt);
+        DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+            "insert into history select imgid, rowid-1, module, operation, op_params, enabled, "
+            "blendop_params, blendop_version from memory.history",
+          -1, &stmt, NULL);
+        sqlite3_step(stmt);
+      }
+    }
+  }
+  sqlite3_finalize(stmt);
+
+  image->flags |= DT_IMAGE_AUTO_PRESETS_APPLIED | DT_IMAGE_NO_LEGACY_PRESETS;
+  dt_pthread_mutex_unlock(&darktable.db_insert);
+  // make sure these end up in the image_cache + xmp (sync through here if we set the flag)
+  dt_image_cache_write_release(darktable.image_cache, image, DT_IMAGE_CACHE_SAFE);
+  dt_image_cache_read_release(darktable.image_cache, cimg);
+}
+
 void dt_dev_read_history(dt_develop_t *dev)
 {
-  if(!dev->image_storage.id) return;
+  if(dev->image_storage.id <= 0) return;
+
+  // maybe prepend auto-presets to history before loading it:
+  auto_apply_presets(dev->image_storage.id);
+
   sqlite3_stmt *stmt;
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select * from history where imgid = ?1 order by num", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select imgid, num, module, operation, op_params, enabled, blendop_params, blendop_version from history where imgid = ?1 order by num", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, dev->image_storage.id);
   dev->history_end = 0;
   while(sqlite3_step(stmt) == SQLITE_ROW)
   {
     // db record:
-    // 0-img, 1-num, 2-module_instance, 3-operation char, 4-params blob, 5-enabled, 6-blend_params
+    // 0-img, 1-num, 2-module_instance, 3-operation char, 4-params blob, 5-enabled, 6-blend_params, 7-blendop_version
     dt_dev_history_item_t *hist = (dt_dev_history_item_t *)malloc(sizeof(dt_dev_history_item_t));
     hist->enabled = sqlite3_column_int(stmt, 5);
 
@@ -637,9 +759,9 @@ void dt_dev_read_history(dt_develop_t *dev)
       free(hist);
       continue;
     }
-   
+
     hist->module = NULL;
-    while(modules)
+    while(opname && modules)
     {
       dt_iop_module_t *module = (dt_iop_module_t *)modules->data;
       if(!strcmp(module->op, opname))
@@ -649,7 +771,7 @@ void dt_dev_read_history(dt_develop_t *dev)
       }
       modules = g_list_next(modules);
     }
-    if(!hist->module)
+    if(!hist->module && opname)
     {
       fprintf(stderr, "[dev_read_history] the module `%s' requested by image `%s' is not installed on this computer!\n", opname, dev->image_storage.filename);
       free(hist);
@@ -681,10 +803,22 @@ void dt_dev_read_history(dt_develop_t *dev)
       memcpy(hist->params, sqlite3_column_blob(stmt, 4), hist->module->params_size);
     }
 
-    if(sqlite3_column_bytes(stmt, 6) == sizeof(dt_develop_blend_params_t))
-      memcpy(hist->blend_params, sqlite3_column_blob(stmt, 6), sizeof(dt_develop_blend_params_t));
+    const void *blendop_params = sqlite3_column_blob(stmt, 6);
+    int bl_length = sqlite3_column_bytes(stmt, 6);
+    int blendop_version = sqlite3_column_int(stmt, 7);
+
+    if (blendop_params && (blendop_version == dt_develop_blend_version()) && (bl_length == sizeof(dt_develop_blend_params_t)))
+    {
+      memcpy(hist->blend_params, blendop_params, sizeof(dt_develop_blend_params_t));
+    }
+    else if (blendop_params && dt_develop_blend_legacy_params(hist->module, blendop_params, blendop_version, hist->blend_params, dt_develop_blend_version(), bl_length) == 0)
+    {
+      // do nothing
+    }
     else
-      memset(hist->blend_params, 0, sizeof(dt_develop_blend_params_t));
+    {
+      memcpy(hist->blend_params, hist->module->default_blendop_params, sizeof(dt_develop_blend_params_t));
+    }
 
     // memcpy(hist->module->params, hist->params, hist->module->params_size);
     // hist->module->enabled = hist->enabled;
@@ -705,6 +839,46 @@ void dt_dev_read_history(dt_develop_t *dev)
   sqlite3_finalize (stmt);
 }
 
+
+void dt_dev_reprocess_all(dt_develop_t *dev)
+{
+  if(darktable.gui->reset) return;
+  if(dev && dev->gui_attached)
+  {
+    dev->pipe->changed |= DT_DEV_PIPE_SYNCH;
+    dev->preview_pipe->changed |= DT_DEV_PIPE_SYNCH;
+    dev->pipe->cache_obsolete = 1;
+    dev->preview_pipe->cache_obsolete = 1;
+
+    dt_similarity_image_dirty(dev->image_storage.id);
+
+    // invalidate buffers and force redraw of darkroom
+    dt_dev_invalidate_all(dev);
+
+    /* redraw */
+    dt_control_queue_redraw_center();
+  }
+}
+
+void dt_dev_reprocess_center(dt_develop_t *dev)
+{
+  if(darktable.gui->reset) return;
+  if(dev && dev->gui_attached)
+  {
+    dev->pipe->changed |= DT_DEV_PIPE_SYNCH;
+    dev->pipe->cache_obsolete = 1;
+
+    dt_similarity_image_dirty(dev->image_storage.id);
+
+    // invalidate buffers and force redraw of darkroom
+    dt_dev_invalidate_all(dev);
+
+    /* redraw */
+    dt_control_queue_redraw_center();
+  }
+}
+
+
 void dt_dev_check_zoom_bounds(dt_develop_t *dev, float *zoom_x, float *zoom_y, dt_dev_zoom_t zoom, int closeup, float *boxww, float *boxhh)
 {
   int procw, proch;
@@ -748,10 +922,29 @@ void dt_dev_check_zoom_bounds(dt_develop_t *dev, float *zoom_x, float *zoom_y, d
 
 void dt_dev_get_processed_size(const dt_develop_t *dev, int *procw, int *proch)
 {
-  // [jo] TODO: i hate this whole function. thread-safe? get rid of it!
-  const float scale = dev->preview_pipe ? (dev->preview_pipe->iscale / dev->preview_downsampling) : 1.0f;
-  *procw = dev->pipe && dev->pipe->processed_width  ? dev->pipe->processed_width  : scale * dev->preview_pipe->processed_width;
-  *proch = dev->pipe && dev->pipe->processed_height ? dev->pipe->processed_height : scale * dev->preview_pipe->processed_height;
+  if (!dev)
+    return;
+
+  // if pipe is processed, lets return its size
+  if (dev->pipe && dev->pipe->processed_width)
+  {
+    *procw = dev->pipe->processed_width;
+    *proch = dev->pipe->processed_height;
+    return;
+  }
+
+  // fallback on preview pipe
+  if (dev->preview_pipe && dev->preview_pipe->processed_width)
+  {
+    const float scale = (dev->preview_pipe->iscale / dev->preview_downsampling);
+    *procw = scale * dev->preview_pipe->processed_width;
+    *proch = scale * dev->preview_pipe->processed_height;
+    return;
+  }
+
+  // no processed pipes, lets return 0 size
+  *procw = *proch = 0;
+  return;
 }
 
 void dt_dev_get_pointer_zoom_pos(dt_develop_t *dev, const float px, const float py, float *zoom_x, float *zoom_y)
@@ -787,7 +980,7 @@ dt_dev_is_current_image (dt_develop_t *dev, uint32_t imgid)
 gboolean dt_dev_exposure_hooks_available(dt_develop_t *dev)
 {
   /* check if exposure iop module has registered its hooks */
-  if( dev->proxy.exposure.module && 
+  if( dev->proxy.exposure.module &&
       dev->proxy.exposure.set_black && dev->proxy.exposure.get_black &&
       dev->proxy.exposure.set_white && dev->proxy.exposure.get_white)
     return TRUE;
@@ -807,14 +1000,14 @@ void dt_dev_exposure_reset_defaults(dt_develop_t *dev)
 
 void dt_dev_exposure_set_white(dt_develop_t *dev, const float white)
 {
-   if (dev->proxy.exposure.module && dev->proxy.exposure.set_white)
-     dev->proxy.exposure.set_white(dev->proxy.exposure.module, white);
+  if (dev->proxy.exposure.module && dev->proxy.exposure.set_white)
+    dev->proxy.exposure.set_white(dev->proxy.exposure.module, white);
 }
 
 float dt_dev_exposure_get_white(dt_develop_t *dev)
 {
   if (dev->proxy.exposure.module && dev->proxy.exposure.set_white)
-     return dev->proxy.exposure.get_white(dev->proxy.exposure.module);
+    return dev->proxy.exposure.get_white(dev->proxy.exposure.module);
 
   return 0.0;
 }
@@ -822,18 +1015,18 @@ float dt_dev_exposure_get_white(dt_develop_t *dev)
 void dt_dev_exposure_set_black(dt_develop_t *dev, const float black)
 {
   if (dev->proxy.exposure.module && dev->proxy.exposure.set_black)
-     dev->proxy.exposure.set_black(dev->proxy.exposure.module, black);
+    dev->proxy.exposure.set_black(dev->proxy.exposure.module, black);
 }
 
 float dt_dev_exposure_get_black(dt_develop_t *dev)
 {
   if (dev->proxy.exposure.module && dev->proxy.exposure.set_black)
-     return dev->proxy.exposure.get_black(dev->proxy.exposure.module);
+    return dev->proxy.exposure.get_black(dev->proxy.exposure.module);
 
   return 0.0;
 }
 
-gboolean dt_dev_modulegroups_available(dt_develop_t *dev) 
+gboolean dt_dev_modulegroups_available(dt_develop_t *dev)
 {
   if(dev->proxy.modulegroups.module && dev->proxy.modulegroups.set && dev->proxy.modulegroups.get)
     return TRUE;
@@ -858,7 +1051,7 @@ uint32_t dt_dev_modulegroups_get(dt_develop_t *dev)
 gboolean dt_dev_modulegroups_test(dt_develop_t *dev, uint32_t group, uint32_t iop_group)
 {
   if(dev->proxy.modulegroups.module && dev->proxy.modulegroups.test)
-    return dev->proxy.modulegroups.test(dev->proxy.modulegroups.module, group, iop_group); 
+    return dev->proxy.modulegroups.test(dev->proxy.modulegroups.module, group, iop_group);
   return FALSE;
 }
 
@@ -881,4 +1074,14 @@ void dt_dev_invalidate_from_gui (dt_develop_t *dev)
   dt_dev_pop_history_items(darktable.develop, darktable.develop->history_end);
 }
 
+void dt_dev_average_delay_update(const dt_times_t *start, uint32_t *average_delay)
+{
+  dt_times_t end;
+  dt_get_times(&end);
+
+  *average_delay += ((end.clock - start->clock)*1000/DT_DEV_AVERAGE_DELAY_COUNT - *average_delay/DT_DEV_AVERAGE_DELAY_COUNT);
+}
+
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
