@@ -50,6 +50,8 @@ typedef struct dt_develop_t
   int32_t image_force_reload;
   int32_t preview_loading, preview_dirty, preview_input_changed;
   uint32_t timestamp;
+  uint32_t average_delay;
+  uint32_t preview_average_delay;
   struct dt_iop_module_t *gui_module; // this module claims gui expose/event callbacks.
   float preview_downsampling; // < 1.0: optionally downsample preview
 
@@ -80,11 +82,12 @@ typedef struct dt_develop_t
   // histogram for display.
   float *histogram, *histogram_pre_tonecurve, *histogram_pre_levels;
   float histogram_max, histogram_pre_tonecurve_max, histogram_pre_levels_max;
+  gboolean histogram_linear;
 
   /* proxy for communication between plugins and develop/darkroom */
   struct
   {
-    // exposure plugin hooks, used by histogram dragging functions 
+    // exposure plugin hooks, used by histogram dragging functions
     struct
     {
       struct dt_iop_module_t *module;
@@ -110,7 +113,7 @@ typedef struct dt_develop_t
     }
     modulegroups;
 
-    // snapshots plugin hooks 
+    // snapshots plugin hooks
     struct
     {
       // this flag is set by snapshot plugin to signal that expose of darkroom
@@ -150,7 +153,8 @@ void dt_dev_invalidate_all(dt_develop_t *dev);
 void dt_dev_set_histogram(dt_develop_t *dev);
 void dt_dev_set_histogram_pre(dt_develop_t *dev);
 void dt_dev_get_history_item_label(dt_dev_history_item_t *hist, char *label, const int cnt);
-
+void dt_dev_reprocess_all(dt_develop_t *dev);
+void dt_dev_reprocess_center(dt_develop_t *dev);
 
 void dt_dev_get_processed_size(const dt_develop_t *dev, int *procw, int *proch);
 void dt_dev_check_zoom_bounds(dt_develop_t *dev, float *zoom_x, float *zoom_y, dt_dev_zoom_t zoom, int closeup, float *boxw, float *boxh);
@@ -161,8 +165,8 @@ void dt_dev_get_pointer_zoom_pos(dt_develop_t *dev, const float px, const float
 void dt_dev_configure (dt_develop_t *dev, int wd, int ht);
 void dt_dev_invalidate_from_gui (dt_develop_t *dev);
 
-/* 
- * exposure plugin hook, set the white level 
+/*
+ * exposure plugin hook, set the white level
  */
 
 /** check if exposure iop hooks are available */
@@ -195,4 +199,10 @@ gboolean dt_dev_modulegroups_test(dt_develop_t *dev, uint32_t group, uint32_t io
 /** request snapshot */
 void dt_dev_snapshot_request(dt_develop_t *dev, const char *filename);
 
+/** update gliding average for pixelpipe delay */
+void dt_dev_average_delay_update(const dt_times_t *start, uint32_t *average_delay);
+
 #endif
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
