@@ -1,20 +1,21 @@
 /*
-		This file is part of darktable,
-		copyright (c) 2009--2011 johannes hanika.
-		copyright (c) 2011 henrik andersson.
+    This file is part of darktable,
+    copyright (c) 2009--2012 johannes hanika.
+    copyright (c) 2011 henrik andersson.
+    copyright (c) 2012 tobias ellinghaus.
 
-		darktable is free software: you can redistribute it and/or modify
-		it under the terms of the GNU General Public License as published by
-		the Free Software Foundation, either version 3 of the License, or
-		(at your option) any later version.
+    darktable is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
 
-		darktable is distributed in the hope that it will be useful,
-		but WITHOUT ANY WARRANTY; without even the implied warranty of
-		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-		GNU General Public License for more details.
+    darktable is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
 
-		You should have received a copy of the GNU General Public License
-		along with darktable.  If not, see <http://www.gnu.org/licenses/>.
+    You should have received a copy of the GNU General Public License
+    along with darktable.  If not, see <http://www.gnu.org/licenses/>.
 */
 #ifndef DT_DEVELOP_IMAGEOP_H
 #define DT_DEVELOP_IMAGEOP_H
@@ -23,6 +24,7 @@
 #include "control/settings.h"
 #include "develop/pixelpipe.h"
 #include "dtgtk/togglebutton.h"
+#include "gui/simple_gui.h"
 #include <gmodule.h>
 #include <gtk/gtk.h>
 #include <sched.h>
@@ -55,9 +57,19 @@ struct dt_develop_tiling_t;
 #define	IOP_FLAGS_SUPPORTS_BLENDING	2			// Does provide blending modes
 #define	IOP_FLAGS_DEPRECATED	        4
 #define IOP_FLAGS_BLEND_ONLY_LIGHTNESS	8			// Does only blend with L-channel in Lab space. Keeps a, b of original image.
-#define IOP_FLAGS_ALLOW_TILING         16                       // Does allow tile-wise processing (currently only via opencl)
+#define IOP_FLAGS_ALLOW_TILING         16                       // Does allow tile-wise processing (valid for CPU and GPU processing)
 #define IOP_FLAGS_HIDDEN               32                       // Hide the iop from userinterface
+#define IOP_FLAGS_TILING_FULL_ROI      64                       // Tiling code has to expect arbitrary roi's for this module (incl. flipping, mirroring etc.)
+/** status of a module*/
+typedef enum dt_iop_module_state_t
+{
+  dt_iop_state_HIDDEN = 0, // keep first
+  dt_iop_state_ACTIVE,
+  dt_iop_state_FAVORITE,
+  dt_iop_state_LAST
 
+}
+dt_iop_module_state_t;
 typedef struct dt_iop_params_t
 {
   int keep;
@@ -78,6 +90,10 @@ typedef struct dt_iop_module_so_t
   dt_dev_operation_t op;
   /** other stuff that may be needed by the module, not only in gui mode. inited only once, has to be read-only then. */
   dt_iop_global_data_t *data;
+  /** gui is also only inited once at startup. */
+  dt_iop_gui_data_t *gui_data;
+  /** which results in this widget here, too. */
+  GtkWidget *widget;
 
   /** this initializes static, hardcoded presets for this module and is called only once per run of dt. */
   void (*init_presets)    (struct dt_iop_module_so_t *self);
@@ -92,8 +108,8 @@ typedef struct dt_iop_module_so_t
   int (*groups)           ();
   int (*flags)            ();
 
-  int (*operation_tags)         (); 
-  int (*operation_tags_filter)  (); 
+  int (*operation_tags)         ();
+  int (*operation_tags_filter)  ();
 
   int (*output_bpp)       (struct dt_iop_module_t *self, struct dt_dev_pixelpipe_t *pipe, struct dt_dev_pixelpipe_iop_t *piece);
   void (*tiling_callback) (struct dt_iop_module_t *self, struct dt_dev_pixelpipe_iop_t *piece, const struct dt_iop_roi_t *roi_in, const struct dt_iop_roi_t *roi_out, struct dt_develop_tiling_t *tiling);
@@ -101,12 +117,15 @@ typedef struct dt_iop_module_so_t
   void (*gui_reset)       (struct dt_iop_module_t *self);
   void (*gui_update)      (struct dt_iop_module_t *self);
   void (*gui_init)        (struct dt_iop_module_t *self);
+  dt_gui_simple_t* (*gui_init_simple) (struct dt_iop_module_so_t *self);
   void (*gui_cleanup)     (struct dt_iop_module_t *self);
   void (*gui_post_expose) (struct dt_iop_module_t *self, cairo_t *cr, int32_t width, int32_t height, int32_t pointerx, int32_t pointery);
   void (*gui_focus)       (struct dt_iop_module_t *self, gboolean in);
   /** Optional callback for keyboard accelerators */
   void (*init_key_accels)(struct dt_iop_module_so_t *so);
+  void (*original_init_key_accels)(struct dt_iop_module_so_t *so);
   void (*connect_key_accels)(struct dt_iop_module_t *self);
+  void (*original_connect_key_accels)(struct dt_iop_module_t *self);
   void (*disconnect_key_accels)(struct dt_iop_module_t *self);
 
   int  (*mouse_leave)     (struct dt_iop_module_t *self);
@@ -117,6 +136,7 @@ typedef struct dt_iop_module_so_t
   void (*configure)       (struct dt_iop_module_t *self, int width, int height);
 
   void (*init)            (struct dt_iop_module_t *self); // this MUST set params_size!
+  void (*original_init)   (struct dt_iop_module_t *self);
   void (*cleanup)         (struct dt_iop_module_t *self);
   void (*init_pipe)       (struct dt_iop_module_t *self, struct dt_dev_pixelpipe_t *pipe, struct dt_dev_pixelpipe_iop_t *piece);
   void (*commit_params)   (struct dt_iop_module_t *self, struct dt_iop_params_t *params, struct dt_dev_pixelpipe_t *pipe, struct dt_dev_pixelpipe_iop_t *piece);
@@ -147,12 +167,18 @@ typedef struct dt_iop_module_t
   int32_t hide_enable_button;
   /** set to 1 if you want an input color picked during next eval. gui mode only. */
   int32_t request_color_pick;
+  /** set to 1 if you want the mask to be transfered into alpha channel during next eval. gui mode only. */
+  int32_t request_mask_display;
+  /** set to 1 if you want the blendif mask to be suppressed in the module in focus. gui mode only. */
+  int32_t suppress_mask;
   /** bounding box in which the mean color is requested. */
   float color_picker_box[4];
   /** single point to pick if in point mode */
   float color_picker_point[2];
-  /** place to store the picked color. */
+  /** place to store the picked color of module input. */
   float picked_color[3], picked_color_min[3], picked_color_max[3];
+  /** place to store the picked color of module output (before blending). */
+  float picked_output_color[3], picked_output_color_min[3], picked_output_color_max[3];
   /** reference for dlopened libs. */
   darktable_t *dt;
   /** the module is used in this develop module. */
@@ -171,7 +197,7 @@ typedef struct dt_iop_module_t
   struct dt_develop_blend_params_t *blend_params, *default_blendop_params;
   /** holder for blending ui control */
   gpointer blend_data;
-  /** child widget which is added to the GtkExpander. */
+  /** child widget which is added to the GtkExpander. copied from module_so_t. */
   GtkWidget *widget;
   /** off button, somewhere in header, common to all plug-ins. */
   GtkDarktableToggleButton *off;
@@ -179,9 +205,10 @@ typedef struct dt_iop_module_t
   GtkWidget *header;
 
   /** button used to show/hide this module in the plugin list. */
-  GtkWidget *showhide;
-  /** expander containing the widget. */
+  dt_iop_module_state_t state;
+  /** expander containing the widget and flag to store expanded state */
   GtkWidget *expander;
+  gboolean expanded;
   /** reset parameters button */
   GtkWidget *reset_button;
   /** show preset menu button */
@@ -205,9 +232,9 @@ typedef struct dt_iop_module_t
   /** get the iop module flags. */
   int (*flags)            ();
 
-  int (*operation_tags)         (); 
+  int (*operation_tags)         ();
 
-  int (*operation_tags_filter)  (); 
+  int (*operation_tags_filter)  ();
   /** how many bytes per pixel in the output. */
   int (*output_bpp)       (struct dt_iop_module_t *self, struct dt_dev_pixelpipe_t *pipe, struct dt_dev_pixelpipe_iop_t *piece);
   /** report back info for tiling: memory usage and overlap. Memory usage: factor * intput_size + overhead */
@@ -220,6 +247,8 @@ typedef struct dt_iop_module_t
   void (*gui_reset)       (struct dt_iop_module_t *self);
   /** construct widget. */
   void (*gui_init)        (struct dt_iop_module_t *self);
+  /** construct widget using the simple api. */
+  dt_gui_simple_t* (*gui_init_simple) (struct dt_iop_module_so_t *self);
   /** destroy widget. */
   void (*gui_cleanup)     (struct dt_iop_module_t *self);
   /** optional method called after darkroom expose. */
@@ -237,6 +266,7 @@ typedef struct dt_iop_module_t
   void (*configure)       (struct dt_iop_module_t *self, int width, int height);
 
   void (*init)            (struct dt_iop_module_t *self); // this MUST set params_size!
+  void (*original_init)   (struct dt_iop_module_t *self);
   void (*cleanup)         (struct dt_iop_module_t *self);
   /** this inits the piece of the pipe, allocing piece->data as necessary. */
   void (*init_pipe)       (struct dt_iop_module_t *self, struct dt_dev_pixelpipe_t *pipe, struct dt_dev_pixelpipe_iop_t *piece);
@@ -265,6 +295,7 @@ typedef struct dt_iop_module_t
 
   /** Key accelerator registration callbacks */
   void (*connect_key_accels)(struct dt_iop_module_t *self);
+  void (*original_connect_key_accels)(struct dt_iop_module_t *self);
   void (*disconnect_key_accels)(struct dt_iop_module_t *self);
 }
 dt_iop_module_t;
@@ -281,12 +312,18 @@ void dt_iop_cleanup_module(dt_iop_module_t *module);
 void dt_iop_init_pipe(struct dt_iop_module_t *module, struct dt_dev_pixelpipe_t *pipe, struct dt_dev_pixelpipe_iop_t *piece);
 /** checks if iop do have an ui */
 gboolean dt_iop_is_hidden(dt_iop_module_t *module);
+/** cleans up gui of module and of blendops */
+void dt_iop_gui_cleanup_module(dt_iop_module_t *module);
 /** updates the gui params and the enabled switch. */
 void dt_iop_gui_update(dt_iop_module_t *module);
 /** reset the ui to its defaults */
 void dt_iop_gui_reset(dt_iop_module_t *module);
 /** set expanded state of iop */
 void dt_iop_gui_set_expanded(dt_iop_module_t *module, gboolean expanded);
+/** refresh iop according to set expanded state */
+void dt_iop_gui_update_expanded(dt_iop_module_t *module);
+/** change module state */
+void dt_iop_gui_set_state(dt_iop_module_t *module,dt_iop_module_state_t state);
 
 /** commits params and updates piece hash. */
 void dt_iop_commit_params(dt_iop_module_t *module, struct dt_iop_params_t *params, struct dt_develop_blend_params_t * blendop_params, struct dt_dev_pixelpipe_t *pipe, struct dt_dev_pixelpipe_iop_t *piece);
@@ -307,6 +344,9 @@ void dt_iop_reload_defaults(dt_iop_module_t *module);
 /** let plugins have breakpoints: */
 int dt_iop_breakpoint(struct dt_develop_t *dev, struct dt_dev_pixelpipe_t *pipe);
 
+/** allow plugins to relinquish CPU and go to sleep for some time */
+void dt_iop_nap(uint32_t usec);
+
 /** colorspace enums */
 typedef enum dt_iop_colorspace_type_t
 {
@@ -328,24 +368,24 @@ void dt_iop_clip_and_zoom(float *out, const float *const in, const struct dt_iop
 /** clip and zoom mosaiced image without demosaicing it uint16_t -> float4 */
 void
 dt_iop_clip_and_zoom_demosaic_half_size(
-    float *out,
-    const uint16_t *const in,
-    const struct dt_iop_roi_t *const roi_out,
-    const struct dt_iop_roi_t *const roi_in,
-    const int32_t out_stride,
-    const int32_t in_stride,
-    const uint32_t filters);
+  float *out,
+  const uint16_t *const in,
+  const struct dt_iop_roi_t *const roi_out,
+  const struct dt_iop_roi_t *const roi_in,
+  const int32_t out_stride,
+  const int32_t in_stride,
+  const uint32_t filters);
 
 void
 dt_iop_clip_and_zoom_demosaic_half_size_f(
-    float *out,
-    const float *const in,
-    const struct dt_iop_roi_t *const roi_out,
-    const struct dt_iop_roi_t *const roi_in,
-    const int32_t out_stride,
-    const int32_t in_stride,
-    const uint32_t filters,
-    const float clip);
+  float *out,
+  const float *const in,
+  const struct dt_iop_roi_t *const roi_out,
+  const struct dt_iop_roi_t *const roi_in,
+  const int32_t out_stride,
+  const int32_t in_stride,
+  const uint32_t filters,
+  const float clip);
 
 /** as dt_iop_clip_and_zoom, but for rgba 8-bit channels. */
 void dt_iop_clip_and_zoom_8(const uint8_t *i, int32_t ix, int32_t iy, int32_t iw, int32_t ih, int32_t ibw, int32_t ibh,
@@ -375,43 +415,66 @@ static inline float dt_iop_eval_cubic(const float *const a, const float x)
  *  the largest point should be (1.0, y) to really get good data. */
 static inline void dt_iop_estimate_exp(const float *const x, const float *const y, const int num, float *coeff)
 {
-  // first find normalization constant a:
-  float xm = 0.0f, ym = 1.0f;
-  for(int k=0;k<num;k++)
-  {
-    if(x[k] > xm)
-    {
-      xm = x[k];
-      ym = y[k];
-    }
-  }
-  const float a = ym;
+  // map every thing to y = y0*(x/x0)^g
+  // and fix (x0,y0) as the last point.
+  // assume (x,y) pairs are ordered by ascending x, so this is the last point:
+  float x0 = x[num-1], y0 = y[num-1];
 
-  // y = a*x^g => g = log(y/a)/log(x);
   float g = 0.0f;
   int cnt = 0;
-  for(int k=0;k<num;k++)
+  // solving for g yields
+  // g = log(y/y0)/log(x/x0)
+  //
+  // average that over the course of the other samples:
+  for(int k=0; k<num-1; k++)
   {
-    if(x[k] < 0.999f)
+    const float yy = y[k]/y0, xx = x[k]/x0;
+    if(yy > 0.0f && xx > 0.0f)
     {
-      g += logf(y[k]/a)/logf(x[k]);
+      const float gg = logf(y[k]/y0)/log(x[k]/x0);
+      g += gg;
       cnt ++;
     }
   }
-  g *= 1.0f/cnt;
-  coeff[0] = a;
-  coeff[1] = g;
+  if(cnt)
+    g *= 1.0f/cnt;
+  else
+    g = 1.0f;
+  coeff[0] = 1.0f/x0;
+  coeff[1] = y0;
+  coeff[2] = g;
 }
 
 /** evaluates the exp fit. */
 static inline float dt_iop_eval_exp(const float *const coeff, const float x)
 {
-  return coeff[0] * powf(x, coeff[1]);
+  return coeff[1] * powf(x*coeff[0], coeff[2]);
 }
 
 /** Connects common accelerators to an iop module */
 void dt_iop_connect_common_accels(dt_iop_module_t *module);
 
+/** Copy alpha channel 1:1 from input to output */
+static inline void dt_iop_alpha_copy(const void *ivoid, void *ovoid, const int width, const int height)
+{
+#ifdef _OPENMP
+  #pragma omp parallel for schedule(static) default(none) shared(ovoid,ivoid)
+#endif
+  for(int j=0; j<height; j++)
+  {
+    const float *in  = ((const float *)ivoid)+4*width*j+3;
+    float *out = ((float *)ovoid)+4*width*j+3;
+    for(int i=0; i<width; i++)
+    {
+      *out = *in;
+      out += 4;
+      in += 4;
+    }
+  }
+}
+
 #endif
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
