@@ -594,7 +594,7 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
        step is anyhow done on cpu. we assume that blending itself will never require tiling in cpu path,
        because memory requirements will still be low enough. */
 
-    assert(tiling.factor > 0.0f && tiling.factor < 100.0f);
+    assert(tiling.factor > 0.0f);
 
     if(pipe->shutdown)
     {
@@ -1580,6 +1580,12 @@ void dt_dev_pixelpipe_get_dimensions(dt_dev_pixelpipe_t *pipe, struct dt_develop
       piece->buf_out = roi_out;
       roi_in = roi_out;
     }
+    else
+    {
+      // pass through regions of interest for gui post expose events
+      piece->buf_in = roi_in;
+      piece->buf_out = roi_in;
+    }
     modules = g_list_next(modules);
     pieces = g_list_next(pieces);
   }
