@@ -23,10 +23,12 @@
 #include "develop/develop.h"
 #include "control/control.h"
 #include "gui/gtk.h"
+#include "bauhaus/bauhaus.h"
 #include "common/colorspaces.h"
 #include "common/colormatrices.c"
 #include "common/opencl.h"
-#include "dtgtk/resetlabel.h"
+#include "common/image_cache.h"
+#include "common/imageio_jpeg.h"
 #include "external/adobe_coeff.c"
 #include <xmmintrin.h>
 #include <stdlib.h>
@@ -34,7 +36,6 @@
 #include <assert.h>
 #include <string.h>
 
-#define ROUNDUP(a, n)		((a) % (n) == 0 ? (a) : ((a) / (n) + 1) * (n))
 
 DT_MODULE(1)
 
@@ -83,25 +84,25 @@ cleanup_global(dt_iop_module_so_t *module)
 }
 
 #if 0
-static void intent_changed (GtkComboBox *widget, gpointer user_data)
+static void intent_changed (GtkWidget *widget, gpointer user_data)
 {
   dt_iop_module_t *self = (dt_iop_module_t *)user_data;
   if(self->dt->gui->reset) return;
   dt_iop_colorin_params_t *p = (dt_iop_colorin_params_t *)self->params;
-  p->intent = (dt_iop_color_intent_t)gtk_combo_box_get_active(widget);
+  p->intent = (dt_iop_color_intent_t)dt_bauhaus_combobox_get(widget);
   dt_dev_add_history_item(darktable.develop, self, TRUE);
 }
 #endif
 
 static void
-profile_changed (GtkComboBox *widget, gpointer user_data)
+profile_changed (GtkWidget *widget, gpointer user_data)
 {
   dt_iop_module_t *self = (dt_iop_module_t *)user_data;
   if(self->dt->gui->reset) return;
   dt_iop_request_focus(self);
   dt_iop_colorin_params_t *p = (dt_iop_colorin_params_t *)self->params;
   dt_iop_colorin_gui_data_t *g = (dt_iop_colorin_gui_data_t *)self->gui_data;
-  int pos = gtk_combo_box_get_active(widget);
+  int pos = dt_bauhaus_combobox_get(widget);
   GList *prof = g->profiles;
   while(prof)
   {
@@ -145,7 +146,7 @@ process_cl (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem
   const int width = roi_in->width;
   const int height = roi_in->height;
 
-  size_t sizes[] = { ROUNDUP(width, 4), ROUNDUP(height, 4), 1};
+  size_t sizes[] = { ROUNDUPWD(width), ROUNDUPHT(height), 1};
   dev_m = dt_opencl_copy_host_to_device_constant(devid, sizeof(float)*9, d->cmatrix);
   if (dev_m == NULL) goto error;
   dev_r = dt_opencl_copy_host_to_device(devid, d->lut[0], 256, 256, sizeof(float));
@@ -154,7 +155,7 @@ process_cl (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem
   if (dev_g == NULL) goto error;
   dev_b = dt_opencl_copy_host_to_device(devid, d->lut[2], 256, 256, sizeof(float));
   if (dev_b == NULL) goto error;
-  dev_coeffs = dt_opencl_copy_host_to_device_constant(devid, sizeof(float)*2*3, (float *)d->unbounded_coeffs);
+  dev_coeffs = dt_opencl_copy_host_to_device_constant(devid, sizeof(float)*3*3, (float *)d->unbounded_coeffs);
   if (dev_coeffs == NULL) goto error;
   dt_opencl_set_kernel_arg(devid, gd->kernel_colorin, 0, sizeof(cl_mem), (void *)&dev_in);
   dt_opencl_set_kernel_arg(devid, gd->kernel_colorin, 1, sizeof(cl_mem), (void *)&dev_out);
@@ -215,7 +216,7 @@ dt_XYZ_to_Lab_SSE(const __m128 XYZ)
   // because d50_inv.z is 0.0f, lab_f(0) == 16/116, so Lab[0] = 116*f[0] - 16 equal to 116*(f[0]-f[3])
   return _mm_mul_ps(coef,_mm_sub_ps(_mm_shuffle_ps(f,f,_MM_SHUFFLE(3,1,0,1)),_mm_shuffle_ps(f,f,_MM_SHUFFLE(3,2,1,3))));
 }
- 
+
 void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void *i, void *o, const dt_iop_roi_t *roi_in, const dt_iop_roi_t *roi_out)
 {
   const dt_iop_colorin_data_t *const d = (dt_iop_colorin_data_t *)piece->data;
@@ -240,25 +241,25 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
       const __m128 m0 = _mm_set_ps(0.0f,mat[6],mat[3],mat[0]);
       const __m128 m1 = _mm_set_ps(0.0f,mat[7],mat[4],mat[1]);
       const __m128 m2 = _mm_set_ps(0.0f,mat[8],mat[5],mat[2]);
-  
+
       for(int i=0; i<roi_out->width; i++, buf_in+=ch, buf_out+=ch )
       {
-  
+
         // memcpy(cam, buf_in, sizeof(float)*3);
         // avoid calling this for linear profiles (marked with negative entries), assures unbounded
         // color management without extrapolation.
         for(int i=0; i<3; i++) cam[i] = (d->lut[i][0] >= 0.0f) ?
-          ((buf_in[i] < 1.0f) ? lerp_lut(d->lut[i], buf_in[i])
-          : dt_iop_eval_exp(d->unbounded_coeffs[i], buf_in[i]))
-          : buf_in[i];
-  
-        if(map_blues)
+                                          ((buf_in[i] < 1.0f) ? lerp_lut(d->lut[i], buf_in[i])
+                                           : dt_iop_eval_exp(d->unbounded_coeffs[i], buf_in[i]))
+                                            : buf_in[i];
+
+        const float YY = cam[0]+cam[1]+cam[2];
+        if(map_blues && YY > 0.0f)
         {
           // manual gamut mapping. these values cause trouble when converting back from Lab to sRGB.
           // deeply saturated blues turn into purple fringes, so dampen them before conversion.
           // this is off for non-raw images, which don't seem to have this problem.
           // might be caused by too loose clipping bounds during highlight clipping?
-          const float YY = cam[0]+cam[1]+cam[2];
           const float zz = cam[2]/YY;
           // lower amount and higher bound_z make the effect smaller.
           // the effect is weakened the darker input values are, saturating at bound_Y
@@ -271,7 +272,7 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
             cam[2] -= t*amount;
           }
         }
-  
+
 #if 0
         __attribute__((aligned(16))) float XYZ[4];
         _mm_store_ps(XYZ,_mm_add_ps(_mm_add_ps( _mm_mul_ps(m0,_mm_set1_ps(cam[0])), _mm_mul_ps(m1,_mm_set1_ps(cam[1]))), _mm_mul_ps(m2,_mm_set1_ps(cam[2]))));
@@ -330,6 +331,9 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
       }
     }
   }
+
+  if(piece->pipe->mask_display)
+    dt_iop_alpha_copy(i, o, roi_out->width, roi_out->height);
 }
 
 void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
@@ -349,29 +353,49 @@ void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pi
   d->lut[1][0] = -1.0f;
   d->lut[2][0] = -1.0f;
   piece->process_cl_ready = 1;
-  char datadir[1024];
-  char filename[1024];
-  dt_util_get_datadir(datadir, 1024);
+  char datadir[DT_MAX_PATH_LEN];
+  char filename[DT_MAX_PATH_LEN];
+  dt_loc_get_datadir(datadir, DT_MAX_PATH_LEN);
+  if(!strcmp(p->iccprofile, "Lab"))
+  {
+    piece->enabled = 0;
+    return;
+  }
+  piece->enabled = 1;
   if(!strcmp(p->iccprofile, "darktable"))
   {
     char makermodel[1024];
     dt_colorspaces_get_makermodel(makermodel, 1024, pipe->image.exif_maker, pipe->image.exif_model);
     d->input = dt_colorspaces_create_darktable_profile(makermodel);
-    if(!d->input) sprintf(p->iccprofile, "cmatrix");
+    if(!d->input) sprintf(p->iccprofile, "eprofile");
   }
   if(!strcmp(p->iccprofile, "vendor"))
   {
     char makermodel[1024];
     dt_colorspaces_get_makermodel(makermodel, 1024, pipe->image.exif_maker, pipe->image.exif_model);
     d->input = dt_colorspaces_create_vendor_profile(makermodel);
-    if(!d->input) sprintf(p->iccprofile, "cmatrix");
+    if(!d->input) sprintf(p->iccprofile, "eprofile");
   }
   if(!strcmp(p->iccprofile, "alternate"))
   {
     char makermodel[1024];
     dt_colorspaces_get_makermodel(makermodel, 1024, pipe->image.exif_maker, pipe->image.exif_model);
     d->input = dt_colorspaces_create_alternate_profile(makermodel);
-    if(!d->input) sprintf(p->iccprofile, "cmatrix");
+    if(!d->input) sprintf(p->iccprofile, "eprofile");
+  }
+  if(!strcmp(p->iccprofile, "eprofile"))
+  {
+    // embedded color profile
+    const dt_image_t *cimg = dt_image_cache_read_get(darktable.image_cache, pipe->image.id);
+    if(cimg->profile == NULL) sprintf(p->iccprofile, "ematrix");
+    else d->input = cmsOpenProfileFromMem(cimg->profile, cimg->profile_size);
+    dt_image_cache_read_release(darktable.image_cache, cimg);
+  }
+  if(!strcmp(p->iccprofile, "ematrix"))
+  {
+    // embedded matrix, hopefully D65
+    if(isnan(pipe->image.d65_color_matrix[0])) sprintf(p->iccprofile, "cmatrix");
+    else d->input = dt_colorspaces_create_xyzimatrix_profile((float (*)[3])pipe->image.d65_color_matrix);
   }
   if(!strcmp(p->iccprofile, "cmatrix"))
   {
@@ -406,7 +430,7 @@ void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pi
   }
   else if(!d->input)
   {
-    dt_colorspaces_find_profile(filename, 1024, p->iccprofile, "in");
+    dt_colorspaces_find_profile(filename, DT_MAX_PATH_LEN, p->iccprofile, "in");
     d->input = cmsOpenProfileFromFile(filename, "r");
   }
   if(d->input)
@@ -446,12 +470,12 @@ void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pi
       for(int t=0; t<num_threads; t++) d->xform[t] = cmsCreateTransform(d->Lab, TYPE_RGB_FLT, d->input, TYPE_Lab_FLT, p->intent, 0);
     }
   }
-  
+
   // now try to initialize unbounded mode:
   // we do a extrapolation for input values above 1.0f.
   // unfortunately we can only do this if we got the computation
   // in our hands, i.e. for the fast builtin-dt-matrix-profile path.
-  for(int k=0;k<3;k++)
+  for(int k=0; k<3; k++)
   {
     // omit luts marked as linear (negative as marker)
     if(d->lut[k][0] >= 0.0f)
@@ -460,7 +484,8 @@ void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pi
       const float y[4] = {lerp_lut(d->lut[k], x[0]),
                           lerp_lut(d->lut[k], x[1]),
                           lerp_lut(d->lut[k], x[2]),
-                          lerp_lut(d->lut[k], x[3])};
+                          lerp_lut(d->lut[k], x[3])
+                         };
       dt_iop_estimate_exp(x, y, 4, d->unbounded_coeffs[k]);
     }
     else d->unbounded_coeffs[k][0] = -1.0f;
@@ -480,19 +505,12 @@ void init_pipe (struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_p
 
 void cleanup_pipe (struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
 {
-#ifdef HAVE_GEGL
-  // clean up everything again.
-  // (void)gegl_node_remove_child(pipe->gegl, piece->input);
-#else
-  // pthread_mutex_lock(&darktable.plugin_threadsafe);
   dt_iop_colorin_data_t *d = (dt_iop_colorin_data_t *)piece->data;
   if(d->input) dt_colorspaces_cleanup_profile(d->input);
   dt_colorspaces_cleanup_profile(d->Lab);
   for(int t=0; t<dt_get_num_threads(); t++) if(d->xform[t]) cmsDeleteTransform(d->xform[t]);
   free(d->xform);
   free(piece->data);
-  // pthread_mutex_unlock(&darktable.plugin_threadsafe);
-#endif
 }
 
 void gui_update(struct dt_iop_module_t *self)
@@ -500,29 +518,62 @@ void gui_update(struct dt_iop_module_t *self)
   dt_iop_module_t *module = (dt_iop_module_t *)self;
   dt_iop_colorin_gui_data_t *g = (dt_iop_colorin_gui_data_t *)self->gui_data;
   dt_iop_colorin_params_t *p = (dt_iop_colorin_params_t *)module->params;
-  // gtk_combo_box_set_active(g->cbox1, (int)p->intent);
+  // dt_bauhaus_combobox_set(g->cbox1, (int)p->intent);
   GList *prof = g->profiles;
   while(prof)
   {
     dt_iop_color_profile_t *pp = (dt_iop_color_profile_t *)prof->data;
     if(!strcmp(pp->filename, p->iccprofile))
     {
-      gtk_combo_box_set_active(g->cbox2, pp->pos);
+      dt_bauhaus_combobox_set(g->cbox2, pp->pos);
       return;
     }
     prof = g_list_next(prof);
   }
-  gtk_combo_box_set_active(g->cbox2, 0);
+  dt_bauhaus_combobox_set(g->cbox2, 0);
   if(strcmp(p->iccprofile, "darktable")) fprintf(stderr, "[colorin] could not find requested profile `%s'!\n", p->iccprofile);
 }
 
+// FIXME: update the gui when we add/remove the eprofile or ematrix
 void reload_defaults(dt_iop_module_t *module)
 {
+  gboolean use_eprofile = FALSE;
+  // some file formats like jpeg can have an embedded color profile
+  // currently we only support jpeg
+  const dt_image_t *cimg = dt_image_cache_read_get(darktable.image_cache, module->dev->image_storage.id);
+  if(!cimg->profile)
+  {
+    char filename[DT_MAX_PATH_LEN];
+    dt_image_full_path(cimg->id, filename, DT_MAX_PATH_LEN);
+    const gchar *cc = filename + strlen(filename);
+    for(; *cc!='.'&&cc>filename; cc--);
+    gchar *ext = g_ascii_strdown(cc+1, -1);
+    if(!strcmp(ext, "jpg") || !strcmp(ext, "jpeg"))
+    {
+      dt_imageio_jpeg_t jpg;
+      if(!dt_imageio_jpeg_read_header(filename, &jpg))
+      {
+        dt_image_t *img = dt_image_cache_write_get(darktable.image_cache, cimg);
+        img->profile_size = dt_imageio_jpeg_read_profile(&jpg, &img->profile);
+        use_eprofile = (img->profile_size > 0);
+        dt_image_cache_write_release(darktable.image_cache, img, DT_IMAGE_CACHE_RELAXED);
+      }
+    }
+    g_free(ext);
+  }
+  else
+    use_eprofile = TRUE; // the image has a profile assigned
+  dt_image_cache_read_release(darktable.image_cache, cimg);
+
   dt_iop_colorin_params_t tmp = (dt_iop_colorin_params_t)
   {
     "darktable", DT_INTENT_PERCEPTUAL
   };
-  if(dt_image_is_ldr(&module->dev->image_storage)) g_strlcpy(tmp.iccprofile, "sRGB", sizeof(tmp.iccprofile));
+
+  if(use_eprofile) g_strlcpy(tmp.iccprofile, "eprofile", sizeof(tmp.iccprofile));
+  else if(module->dev->image_storage.colorspace == DT_IMAGE_COLORSPACE_SRGB) g_strlcpy(tmp.iccprofile, "sRGB", sizeof(tmp.iccprofile));
+  else if(module->dev->image_storage.colorspace == DT_IMAGE_COLORSPACE_ADOBE_RGB) g_strlcpy(tmp.iccprofile, "adobergb", sizeof(tmp.iccprofile));
+  else if(dt_image_is_ldr(&module->dev->image_storage)) g_strlcpy(tmp.iccprofile, "sRGB", sizeof(tmp.iccprofile));
   memcpy(module->params, &tmp, sizeof(dt_iop_colorin_params_t));
   memcpy(module->default_params, &tmp, sizeof(dt_iop_colorin_params_t));
 }
@@ -534,8 +585,9 @@ void init(dt_iop_module_t *module)
   module->default_params = malloc(sizeof(dt_iop_colorin_params_t));
   module->params_size = sizeof(dt_iop_colorin_params_t);
   module->gui_data = NULL;
-  module->priority = 352; // module order created by iop_dependencies.py, do not edit!
+  module->priority = 346; // module order created by iop_dependencies.py, do not edit!
   module->hide_enable_button = 1;
+  module->default_enabled = 1;
 }
 
 void cleanup(dt_iop_module_t *module)
@@ -551,12 +603,31 @@ void gui_init(struct dt_iop_module_t *self)
   // pthread_mutex_lock(&darktable.plugin_threadsafe);
   self->gui_data = malloc(sizeof(dt_iop_colorin_gui_data_t));
   dt_iop_colorin_gui_data_t *g = (dt_iop_colorin_gui_data_t *)self->gui_data;
-  dt_iop_colorin_params_t *p   = (dt_iop_colorin_params_t *)self->params;
 
   g->profiles = NULL;
   dt_iop_color_profile_t *prof;
   int pos = -1;
-
+  // some file formats like jpeg can have an embedded color profile
+  // currently we only support jpeg
+  const dt_image_t *cimg = dt_image_cache_read_get(darktable.image_cache, self->dev->image_storage.id);
+  if(cimg->profile)
+  {
+    prof = (dt_iop_color_profile_t *)g_malloc0(sizeof(dt_iop_color_profile_t));
+    g_strlcpy(prof->filename, "eprofile", sizeof(prof->filename));
+    g_strlcpy(prof->name, "eprofile", sizeof(prof->name));
+    g->profiles = g_list_append(g->profiles, prof);
+    prof->pos = ++pos;
+  }
+  dt_image_cache_read_release(darktable.image_cache, cimg);
+  // use the matrix embedded in some DNGs
+  if(!isnan(self->dev->image_storage.d65_color_matrix[0]))
+  {
+    prof = (dt_iop_color_profile_t *)g_malloc0(sizeof(dt_iop_color_profile_t));
+    g_strlcpy(prof->filename, "ematrix", sizeof(prof->filename));
+    g_strlcpy(prof->name, "ematrix", sizeof(prof->name));
+    g->profiles = g_list_append(g->profiles, prof);
+    prof->pos = ++pos;
+  }
   // get color matrix from raw image:
   char makermodel[1024];
   dt_colorspaces_get_makermodel(makermodel, 1024, self->dev->image_storage.exif_maker, self->dev->image_storage.exif_model);
@@ -642,6 +713,13 @@ void gui_init(struct dt_iop_module_t *self)
   g->profiles = g_list_append(g->profiles, prof);
   prof->pos = ++pos;
 
+  // Lab built-in
+  prof = (dt_iop_color_profile_t *)g_malloc0(sizeof(dt_iop_color_profile_t));
+  g_strlcpy(prof->filename, "Lab", sizeof(prof->filename));
+  g_strlcpy(prof->name, "Lab", sizeof(prof->name));
+  g->profiles = g_list_append(g->profiles, prof);
+  prof->pos = ++pos;
+
   // infrared built-in
   prof = (dt_iop_color_profile_t *)g_malloc0(sizeof(dt_iop_color_profile_t));
   g_strlcpy(prof->filename, "infrared", sizeof(prof->filename));
@@ -650,12 +728,15 @@ void gui_init(struct dt_iop_module_t *self)
   prof->pos = ++pos;
 
   // read {userconfig,datadir}/color/in/*.icc, in this order.
-  char datadir[1024], confdir[1024], dirname[1024], filename[1024];
-  dt_util_get_user_config_dir(confdir, 1024);
-  dt_util_get_datadir(datadir, 1024);
-  snprintf(dirname, 1024, "%s/color/in", confdir);
+  char datadir[DT_MAX_PATH_LEN];
+  char confdir[DT_MAX_PATH_LEN];
+  char dirname[DT_MAX_PATH_LEN];
+  char filename[DT_MAX_PATH_LEN];
+  dt_loc_get_user_config_dir(confdir, DT_MAX_PATH_LEN);
+  dt_loc_get_datadir(datadir, DT_MAX_PATH_LEN);
+  snprintf(dirname, DT_MAX_PATH_LEN, "%s/color/in", confdir);
   if(!g_file_test(dirname, G_FILE_TEST_IS_DIR))
-    snprintf(dirname, 1024, "%s/color/in", datadir);
+    snprintf(dirname, DT_MAX_PATH_LEN, "%s/color/in", datadir);
   cmsHPROFILE tmpprof;
   const gchar *d_name;
   GDir *dir = g_dir_open(dirname, 0, NULL);
@@ -664,12 +745,12 @@ void gui_init(struct dt_iop_module_t *self)
     while((d_name = g_dir_read_name(dir)))
     {
       if(!strcmp(d_name, "linear_rgb")) continue;
-      snprintf(filename, 1024, "%s/%s", dirname, d_name);
+      snprintf(filename, DT_MAX_PATH_LEN, "%s/%s", dirname, d_name);
       tmpprof = cmsOpenProfileFromFile(filename, "r");
       if(tmpprof)
       {
-	char *lang = getenv("LANG");
-	if (!lang) lang = "en_US";
+        char *lang = getenv("LANG");
+        if (!lang) lang = "en_US";
 
         dt_iop_color_profile_t *prof = (dt_iop_color_profile_t *)g_malloc0(sizeof(dt_iop_color_profile_t));
         char name[1024];
@@ -685,51 +766,51 @@ void gui_init(struct dt_iop_module_t *self)
     g_dir_close(dir);
   }
 
-  self->widget = GTK_WIDGET(gtk_hbox_new(FALSE, 0));
-  g->vbox1 = GTK_VBOX(gtk_vbox_new(TRUE, DT_GUI_IOP_MODULE_CONTROL_SPACING));
-  g->vbox2 = GTK_VBOX(gtk_vbox_new(TRUE, DT_GUI_IOP_MODULE_CONTROL_SPACING));
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->vbox1), FALSE, FALSE, 5);
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->vbox2), TRUE, TRUE, 5);
-  GtkWidget *label = dtgtk_reset_label_new(_("profile"), self, &p->iccprofile, sizeof(char)*DT_IOP_COLOR_ICC_LEN);
-  gtk_box_pack_start(GTK_BOX(g->vbox1), GTK_WIDGET(label), TRUE, TRUE, 0);
-  g->cbox2 = GTK_COMBO_BOX(gtk_combo_box_new_text());
+  self->widget = gtk_vbox_new(TRUE, DT_BAUHAUS_SPACE);
+  g->cbox2 = dt_bauhaus_combobox_new(self);
+  dt_bauhaus_widget_set_label(g->cbox2, _("profile"));
+  gtk_box_pack_start(GTK_BOX(self->widget), g->cbox2, TRUE, TRUE, 0);
   GList *l = g->profiles;
   while(l)
   {
     dt_iop_color_profile_t *prof = (dt_iop_color_profile_t *)l->data;
-    if(!strcmp(prof->name, "cmatrix"))
-      gtk_combo_box_append_text(g->cbox2, _("standard color matrix"));
+    if(!strcmp(prof->name, "eprofile"))
+      dt_bauhaus_combobox_add(g->cbox2, _("embedded icc profile"));
+    else if(!strcmp(prof->name, "ematrix"))
+      dt_bauhaus_combobox_add(g->cbox2, _("dng embedded matrix"));
+    else if(!strcmp(prof->name, "cmatrix"))
+      dt_bauhaus_combobox_add(g->cbox2, _("standard color matrix"));
     else if(!strcmp(prof->name, "darktable"))
-      gtk_combo_box_append_text(g->cbox2, _("enhanced color matrix"));
+      dt_bauhaus_combobox_add(g->cbox2, _("enhanced color matrix"));
     else if(!strcmp(prof->name, "vendor"))
-      gtk_combo_box_append_text(g->cbox2, _("vendor color matrix"));
+      dt_bauhaus_combobox_add(g->cbox2, _("vendor color matrix"));
     else if(!strcmp(prof->name, "alternate"))
-      gtk_combo_box_append_text(g->cbox2, _("alternate color matrix"));
+      dt_bauhaus_combobox_add(g->cbox2, _("alternate color matrix"));
     else if(!strcmp(prof->name, "sRGB"))
-      gtk_combo_box_append_text(g->cbox2, _("sRGB (e.g. jpg)"));
+      dt_bauhaus_combobox_add(g->cbox2, _("sRGB (e.g. jpg)"));
     else if(!strcmp(prof->name, "adobergb"))
-      gtk_combo_box_append_text(g->cbox2, _("Adobe RGB"));
+      dt_bauhaus_combobox_add(g->cbox2, _("Adobe RGB"));
     else if(!strcmp(prof->name, "linear_rgb"))
-      gtk_combo_box_append_text(g->cbox2, _("linear RGB"));
+      dt_bauhaus_combobox_add(g->cbox2, _("linear RGB"));
     else if(!strcmp(prof->name, "infrared"))
-      gtk_combo_box_append_text(g->cbox2, _("linear infrared BGR"));
+      dt_bauhaus_combobox_add(g->cbox2, _("linear infrared BGR"));
     else if(!strcmp(prof->name, "XYZ"))
-      gtk_combo_box_append_text(g->cbox2, _("linear XYZ"));
+      dt_bauhaus_combobox_add(g->cbox2, _("linear XYZ"));
+    else if(!strcmp(prof->name, "Lab"))
+      dt_bauhaus_combobox_add(g->cbox2, _("Lab"));
     else
-      gtk_combo_box_append_text(g->cbox2, prof->name);
+      dt_bauhaus_combobox_add(g->cbox2, prof->name);
     l = g_list_next(l);
   }
-  gtk_combo_box_set_active(g->cbox2, 0);
-  gtk_box_pack_start(GTK_BOX(g->vbox2), GTK_WIDGET(g->cbox2), TRUE, TRUE, 0);
+  dt_bauhaus_combobox_set(g->cbox2, 0);
 
   char tooltip[1024];
   snprintf(tooltip, 1024, _("icc profiles in %s/color/in or %s/color/in"), confdir, datadir);
   g_object_set(G_OBJECT(g->cbox2), "tooltip-text", tooltip, (char *)NULL);
 
-  g_signal_connect (G_OBJECT (g->cbox2), "changed",
+  g_signal_connect (G_OBJECT (g->cbox2), "value-changed",
                     G_CALLBACK (profile_changed),
                     (gpointer)self);
-  // pthread_mutex_unlock(&darktable.plugin_threadsafe);
 }
 
 void gui_cleanup(struct dt_iop_module_t *self)
@@ -744,4 +825,6 @@ void gui_cleanup(struct dt_iop_module_t *self)
   self->gui_data = NULL;
 }
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
