@@ -300,6 +300,10 @@ autoexp_callback (GtkToggleButton *button, dt_iop_module_t *self)
     dt_lib_colorpicker_set_area(darktable.lib, 0.99);
     dt_dev_reprocess_all(self->dev);
   }
+  else
+  {
+    dt_control_queue_redraw();
+  }
 
   gtk_widget_set_sensitive(GTK_WIDGET(g->autoexpp), gtk_toggle_button_get_active(button));
 
