@@ -53,17 +53,43 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
 {
   dt_iop_gamma_data_t *d = (dt_iop_gamma_data_t *)piece->data;
   const int ch = piece->colors;
-#ifdef _OPENMP
-  #pragma omp parallel for default(none) shared(roi_out, o, i, d) schedule(static)
-#endif
-  for(int k=0; k<roi_out->height; k++)
+
+  if(piece->pipe->mask_display)
   {
-    const float *in = ((float *)i) + ch*k*roi_out->width;
-    uint8_t *out = ((uint8_t *)o) + ch*k*roi_out->width;
-    for (int j=0; j<roi_out->width; j++,in+=ch,out+=ch)
+    const float yellow[3] = { 1.0f, 1.0f, 0.0f };
+#ifdef _OPENMP
+    #pragma omp parallel for default(none) shared(roi_out, o, i, d) schedule(static)
+#endif
+    for(int k=0; k<roi_out->height; k++)
     {
-      for(int c=0; c<3; c++)
-        out[2-c] = d->table[(uint16_t)CLAMP((int)(0xfffful*in[c]), 0, 0xffff)];
+      const float *in = ((float *)i) + ch*k*roi_out->width;
+      uint8_t *out = ((uint8_t *)o) + ch*k*roi_out->width;
+      for (int j=0; j<roi_out->width; j++,in+=ch,out+=ch)
+      {
+        float gray = 0.3f*in[0] + 0.59f*in[1] + 0.11f*in[2];
+        float alpha = in[3];
+        for(int c=0; c<3; c++)
+        {
+          float value = gray * (1.0f - alpha) + yellow[c] * alpha;
+          out[2-c] = d->table[(uint16_t)CLAMP((int)(0xfffful*value), 0, 0xffff)];
+        }
+      }
+    }
+  }
+  else
+  {
+#ifdef _OPENMP
+    #pragma omp parallel for default(none) shared(roi_out, o, i, d) schedule(static)
+#endif
+    for(int k=0; k<roi_out->height; k++)
+    {
+      const float *in = ((float *)i) + ch*k*roi_out->width;
+      uint8_t *out = ((uint8_t *)o) + ch*k*roi_out->width;
+      for (int j=0; j<roi_out->width; j++,in+=ch,out+=ch)
+      {
+        for(int c=0; c<3; c++)
+          out[2-c] = d->table[(uint16_t)CLAMP((int)(0xfffful*in[c]), 0, 0xffff)];
+      }
     }
   }
 }
@@ -135,6 +161,7 @@ void init(dt_iop_module_t *module)
   module->gui_data = NULL;
   module->priority = 1000; // module order created by iop_dependencies.py, do not edit!
   module->hide_enable_button = 1;
+  module->default_enabled = 1;
   dt_iop_gamma_params_t tmp = (dt_iop_gamma_params_t)
   {
     1.0, 1.0
@@ -152,3 +179,6 @@ void cleanup(dt_iop_module_t *module)
 }
 
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
