@@ -1,6 +1,7 @@
 /*
     This file is part of darktable,
     copyright (c) 2011 Rostyslav Pidgornyi
+    copyright (c) 2012 Henrik Andersson
 
     and the initial plugin `stuck pixels' was
     copyright (c) 2011 bruce guenter
@@ -22,9 +23,9 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+#include "bauhaus/bauhaus.h"
 #include "control/control.h"
 #include "develop/imageop.h"
-#include "dtgtk/slider.h"
 #include "dtgtk/resetlabel.h"
 #include "gui/accelerators.h"
 #include "gui/gtk.h"
@@ -44,11 +45,11 @@ dt_iop_hotpixels_params_t;
 
 typedef struct dt_iop_hotpixels_gui_data_t
 {
-  GtkDarktableSlider *threshold;
-  GtkDarktableSlider *strength;
+  GtkWidget *threshold, *strength;
   GtkToggleButton *markfixed;
   GtkToggleButton *permissive;
   GtkLabel *message;
+  int pixels_fixed;
 }
 dt_iop_hotpixels_gui_data_t;
 
@@ -82,7 +83,7 @@ void init_key_accels(dt_iop_module_so_t *self)
 void connect_key_accels(dt_iop_module_t *self)
 {
   dt_iop_hotpixels_gui_data_t *g =
-      (dt_iop_hotpixels_gui_data_t*)self->gui_data;
+    (dt_iop_hotpixels_gui_data_t*)self->gui_data;
 
   dt_accel_connect_slider_iop(self, "threshold", GTK_WIDGET(g->threshold));
   dt_accel_connect_slider_iop(self, "strength", GTK_WIDGET(g->strength));
@@ -95,7 +96,7 @@ output_bpp(dt_iop_module_t *module, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_i
 }
 
 /* Detect hot sensor pixels based on the 4 surrounding sites. Pixels
- * having 3 or 4 (depending on permissive setting) surrounding pixels that 
+ * having 3 or 4 (depending on permissive setting) surrounding pixels that
  * than value*multiplier are considered "hot", and are replaced by the maximum of
  * the neighbour pixels. The permissive variant allows for
  * correcting pairs of hot pixels in adjacent sites. Replacement using
@@ -158,14 +159,10 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
       }
     }
   }
-  if (g != NULL)
+
+  if(g != NULL && self->dev->gui_attached && piece->pipe->type == DT_DEV_PIXELPIPE_FULL)
   {
-    // lock gui thread mutex, if we are not called from gui thread (very likely)
-    gboolean i_have_lock = dt_control_gdk_lock();
-    char buf[256];
-    snprintf(buf, sizeof buf, _("fixed %d pixels"), fixed);
-    gtk_label_set_text(g->message, buf);
-    if(i_have_lock) dt_control_gdk_unlock();
+    g->pixels_fixed = fixed;
   }
 }
 
@@ -175,7 +172,7 @@ void init(dt_iop_module_t *module)
   module->params = malloc(sizeof(dt_iop_hotpixels_params_t));
   module->default_params = malloc(sizeof(dt_iop_hotpixels_params_t));
   module->default_enabled = 0;
-  module->priority = 58; // module order created by iop_dependencies.py, do not edit!
+  module->priority = 96; // module order created by iop_dependencies.py, do not edit!
   module->params_size = sizeof(dt_iop_hotpixels_params_t);
   module->gui_data = NULL;
   const dt_iop_hotpixels_params_t tmp =
@@ -227,7 +224,7 @@ strength_callback(GtkRange *range, dt_iop_module_t *self)
   if(darktable.gui->reset) return;
   dt_iop_hotpixels_gui_data_t *g = (dt_iop_hotpixels_gui_data_t *)self->gui_data;
   dt_iop_hotpixels_params_t *p = (dt_iop_hotpixels_params_t *)self->params;
-  p->strength = dtgtk_slider_get_value(g->strength);
+  p->strength = dt_bauhaus_slider_get(g->strength);
   dt_dev_add_history_item(darktable.develop, self, TRUE);
 }
 
@@ -237,7 +234,7 @@ threshold_callback(GtkRange *range, dt_iop_module_t *self)
   if(darktable.gui->reset) return;
   dt_iop_hotpixels_gui_data_t *g = (dt_iop_hotpixels_gui_data_t *)self->gui_data;
   dt_iop_hotpixels_params_t *p = (dt_iop_hotpixels_params_t *)self->params;
-  p->threshold = dtgtk_slider_get_value(g->threshold);
+  p->threshold = dt_bauhaus_slider_get(g->threshold);
   dt_dev_add_history_item(darktable.develop, self, TRUE);
 }
 
@@ -265,10 +262,31 @@ void gui_update    (dt_iop_module_t *self)
 {
   dt_iop_hotpixels_gui_data_t *g = (dt_iop_hotpixels_gui_data_t *)self->gui_data;
   dt_iop_hotpixels_params_t *p = (dt_iop_hotpixels_params_t *)self->params;
-  dtgtk_slider_set_value(g->strength, p->strength);
-  dtgtk_slider_set_value(g->threshold, p->threshold);
+  dt_bauhaus_slider_set(g->strength, p->strength);
+  dt_bauhaus_slider_set(g->threshold, p->threshold);
   gtk_toggle_button_set_active(g->markfixed, p->markfixed);
   gtk_toggle_button_set_active(g->permissive, p->permissive);
+  g->pixels_fixed = -1;
+  gtk_label_set_text(g->message, "");
+}
+
+static gboolean
+expose (GtkWidget *widget, GdkEventExpose *event, dt_iop_module_t *self)
+{
+  dt_iop_hotpixels_gui_data_t *g = (dt_iop_hotpixels_gui_data_t *)self->gui_data;
+  if(darktable.gui->reset) return FALSE;
+
+  if(g->pixels_fixed < 0) return FALSE;
+
+  char buf[256];
+  snprintf(buf, sizeof buf, _("fixed %d pixels"), g->pixels_fixed);
+  g->pixels_fixed = -1;
+
+  darktable.gui->reset = 1;
+  gtk_label_set_text(g->message, buf);
+  darktable.gui->reset = 0;
+
+  return FALSE;
 }
 
 void gui_init     (dt_iop_module_t *self)
@@ -276,32 +294,35 @@ void gui_init     (dt_iop_module_t *self)
   self->gui_data = malloc(sizeof(dt_iop_hotpixels_gui_data_t));
   dt_iop_hotpixels_gui_data_t *g = (dt_iop_hotpixels_gui_data_t*)self->gui_data;
   dt_iop_hotpixels_params_t *p = (dt_iop_hotpixels_params_t*)self->params;
+  g->pixels_fixed = -1;
 
-  self->widget = GTK_WIDGET(gtk_hbox_new(FALSE, 0));
-  GtkVBox *vbox = GTK_VBOX(gtk_vbox_new(FALSE, DT_GUI_IOP_MODULE_CONTROL_SPACING));
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(vbox), TRUE, TRUE, 5);
+  self->widget = gtk_vbox_new(FALSE, DT_BAUHAUS_SPACE);
+  g_signal_connect(G_OBJECT(self->widget), "expose-event", G_CALLBACK(expose), self);
 
-  g->threshold = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR, 0.0, 1.0, 0.005, p->threshold, 4));
-  dtgtk_slider_set_label(g->threshold,_("threshold"));
+  /* threshold */
+  g->threshold = dt_bauhaus_slider_new_with_range(self, 0.0, 1.0, 0.005, p->threshold, 4);
+  dt_bauhaus_slider_set_format(g->threshold,"%.4f");
+  dt_bauhaus_widget_set_label(g->threshold,_("threshold"));
   g_object_set(G_OBJECT(g->threshold), "tooltip-text", _("lower threshold for hot pixel"), NULL);
-  dtgtk_slider_set_format_type(DTGTK_SLIDER(g->threshold),DARKTABLE_SLIDER_FORMAT_FLOAT);
-  gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(g->threshold), TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->threshold), TRUE, TRUE, 0);
   g_signal_connect(G_OBJECT (g->threshold), "value-changed", G_CALLBACK (threshold_callback), self);
 
-  g->strength = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR, 0.0, 1.0, 0.01, p->strength, 4));
-  dtgtk_slider_set_label(g->strength,_("strength"));
+  /* strength */
+  g->strength = dt_bauhaus_slider_new_with_range(self, 0.0, 1.0, 0.01, p->strength, 4);
+  dt_bauhaus_slider_set_format(g->threshold,"%.4f");
+  dt_bauhaus_widget_set_label(g->strength,_("strength"));
   g_object_set(G_OBJECT(g->strength), "tooltip-text", _("strength of hot pixel correction"), NULL);
-  dtgtk_slider_set_format_type(DTGTK_SLIDER(g->strength),DARKTABLE_SLIDER_FORMAT_FLOAT);
-  gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(g->strength), TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->strength), TRUE, TRUE, 0);
   g_signal_connect(G_OBJECT (g->strength), "value-changed", G_CALLBACK (strength_callback), self);
 
+  /* 3 neighbours */
   g->permissive  = GTK_TOGGLE_BUTTON(gtk_check_button_new_with_label(_("detect by 3 neighbours")));
   gtk_toggle_button_set_active(g->permissive, p->permissive);
-  gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(g->permissive), TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->permissive), TRUE, TRUE, 0);
   g_signal_connect(G_OBJECT(g->permissive), "toggled", G_CALLBACK(permissive_callback), self);
 
-  GtkHBox *hbox1 = GTK_HBOX(gtk_hbox_new(FALSE, 0));
 
+  GtkHBox *hbox1 = GTK_HBOX(gtk_hbox_new(FALSE, 0));
   g->markfixed  = GTK_TOGGLE_BUTTON(gtk_check_button_new_with_label(_("mark fixed pixels")));
   gtk_toggle_button_set_active(g->markfixed, p->markfixed);
   gtk_box_pack_start(GTK_BOX(hbox1), GTK_WIDGET(g->markfixed), TRUE, TRUE, 0);
@@ -310,7 +331,7 @@ void gui_init     (dt_iop_module_t *self)
   g->message = GTK_LABEL(gtk_label_new ("")); // This gets filled in by process
   gtk_box_pack_start(GTK_BOX(hbox1), GTK_WIDGET(g->message), TRUE, TRUE, 0);
 
-  gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(hbox1), TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(hbox1), TRUE, TRUE, 0);
 }
 
 void gui_cleanup  (dt_iop_module_t *self)
@@ -319,4 +340,6 @@ void gui_cleanup  (dt_iop_module_t *self)
   self->gui_data = NULL;
 }
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
