@@ -19,11 +19,33 @@
 #define DT_IOP_LENS_H
 
 #include "develop/imageop.h"
-#include "dtgtk/slider.h"
+#include "bauhaus/bauhaus.h"
 #include <lensfun.h>
 #include <gtk/gtk.h>
 #include <inttypes.h>
 
+typedef enum dt_iop_lensfun_modflag_t
+{
+  LENSFUN_MODFLAG_NONE        = 0,
+  LENSFUN_MODFLAG_ALL         = LF_MODIFY_DISTORTION | LF_MODIFY_TCA | LF_MODIFY_VIGNETTING,
+  LENSFUN_MODFLAG_DIST_TCA    = LF_MODIFY_DISTORTION | LF_MODIFY_TCA,
+  LENSFUN_MODFLAG_DIST_VIGN   = LF_MODIFY_DISTORTION | LF_MODIFY_VIGNETTING,
+  LENSFUN_MODFLAG_TCA_VIGN    = LF_MODIFY_TCA | LF_MODIFY_VIGNETTING,
+  LENSFUN_MODFLAG_DIST        = LF_MODIFY_DISTORTION,
+  LENSFUN_MODFLAG_TCA         = LF_MODIFY_TCA,
+  LENSFUN_MODFLAG_VIGN        = LF_MODIFY_VIGNETTING,
+  LENSFUN_MODFLAG_MASK        = LF_MODIFY_DISTORTION | LF_MODIFY_TCA | LF_MODIFY_VIGNETTING
+}
+dt_iop_lensfun_modflag_t;
+
+typedef struct dt_iop_lensfun_modifier_t
+{
+  char name[40];
+  int  pos;           // position in combo box
+  int  modflag;
+}
+dt_iop_lensfun_modifier_t;
+
 typedef struct dt_iop_lensfun_params_t
 {
   int modify_flags;
@@ -45,22 +67,31 @@ typedef struct dt_iop_lensfun_gui_data_t
 {
   const lfCamera *camera;
   GtkWidget *lens_param_box;
-  GtkComboBoxEntry *cbe[3];
+  GtkWidget *cbe[3];
   GtkButton *camera_model;
   GtkMenu *camera_menu;
   GtkButton *lens_model;
   GtkMenu *lens_menu;
-  GtkComboBox *target_geom;
-  GtkCheckButton *reverse;
-  GtkDarktableSlider *tca_r;
-  GtkDarktableSlider *tca_b;
-  GtkDarktableSlider *scale;
-  GtkWidget *auto_scale_button;
+  GtkWidget *modflags, *target_geom, *reverse, *tca_r, *tca_b, *scale;
   GtkWidget *find_lens_button;
   GtkWidget *find_camera_button;
+  GList *modifiers;
+  GtkLabel *message;
+  int corrections_done;
 }
 dt_iop_lensfun_gui_data_t;
 
+typedef struct dt_iop_lensfun_global_data_t
+{
+  lfDatabase *db;
+  int kernel_lens_distort_bilinear;
+  int kernel_lens_distort_bicubic;
+  int kernel_lens_distort_lanczos2;
+  int kernel_lens_distort_lanczos3;
+  int kernel_lens_vignette;
+}
+dt_iop_lensfun_global_data_t;
+
 typedef struct dt_iop_lensfun_data_t
 {
   lfLens *lens;
@@ -79,6 +110,7 @@ typedef struct dt_iop_lensfun_data_t
 }
 dt_iop_lensfun_data_t;
 
+
 void init(dt_iop_module_t *module);
 void cleanup(dt_iop_module_t *module);
 
@@ -98,3 +130,6 @@ void gui_cleanup  (struct dt_iop_module_t *self);
 void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void *i, void *o, const dt_iop_roi_t *roi_in, const dt_iop_roi_t *roi_out);
 
 #endif
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
