@@ -58,7 +58,7 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
   const int ch = piece->colors;
   dt_iop_levels_data_t *d = (dt_iop_levels_data_t*)(piece->data);
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(roi_out, i, o, d) schedule(static)
+  #pragma omp parallel for default(none) shared(roi_out, i, o, d) schedule(static)
 #endif
   for(int k=0; k<roi_out->height; k++)
   {
@@ -98,11 +98,54 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
         out[2] = in[2] * out[0]/0.01f;
       }
 
+      out[3] = in[3];
+
     }
   }
 
 }
 
+#ifdef HAVE_OPENCL
+int
+process_cl (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem dev_in, cl_mem dev_out, const dt_iop_roi_t *roi_in, const dt_iop_roi_t *roi_out)
+{
+  dt_iop_levels_data_t *d = (dt_iop_levels_data_t *)piece->data;
+  dt_iop_levels_global_data_t *gd = (dt_iop_levels_global_data_t *)self->data;
+
+  cl_mem dev_lut = NULL;
+  cl_int err = -999;
+  const int devid = piece->pipe->devid;
+
+  const int width = roi_out->width;
+  const int height = roi_out->height;
+
+  dev_lut = dt_opencl_copy_host_to_device(devid, d->lut, 256, 256, sizeof(float));
+  if(dev_lut == NULL) goto error;
+
+  size_t sizes[2] = { ROUNDUPWD(width), ROUNDUPHT(height) };
+  dt_opencl_set_kernel_arg(devid, gd->kernel_levels, 0, sizeof(cl_mem), &dev_in);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_levels, 1, sizeof(cl_mem), &dev_out);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_levels, 2, sizeof(int), &width);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_levels, 3, sizeof(int), &height);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_levels, 4, sizeof(cl_mem), &dev_lut);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_levels, 5, sizeof(float), &d->in_low);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_levels, 6, sizeof(float), &d->in_high);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_levels, 7, sizeof(float), &d->in_inv_gamma);
+  err = dt_opencl_enqueue_kernel_2d(devid, gd->kernel_levels, sizes);
+  if(err != CL_SUCCESS) goto error;
+
+  dt_opencl_release_mem_object(dev_lut);
+  return TRUE;
+
+error:
+  if (dev_lut != NULL) dt_opencl_release_mem_object(dev_lut);
+  dt_print(DT_DEBUG_OPENCL, "[opencl_levels] couldn't enqueue kernel! %d\n", err);
+  return FALSE;
+}
+#endif
+
+
+
 //void init_presets (dt_iop_module_so_t *self)
 //{
 //  dt_iop_levels_params_t p;
@@ -140,7 +183,7 @@ void init_pipe (struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe,
 {
   // create part of the gegl pipeline
   dt_iop_levels_data_t *d =
-      (dt_iop_levels_data_t *)malloc(sizeof(dt_iop_levels_data_t));
+    (dt_iop_levels_data_t *)malloc(sizeof(dt_iop_levels_data_t));
   piece->data = (void *)d;
 }
 
@@ -167,7 +210,7 @@ void init(dt_iop_module_t *module)
   module->params = malloc(sizeof(dt_iop_levels_params_t));
   module->default_params = malloc(sizeof(dt_iop_levels_params_t));
   module->default_enabled = 0;
-  module->priority = 627; // module order created by iop_dependencies.py, do not edit!
+  module->priority = 634; // module order created by iop_dependencies.py, do not edit!
   module->params_size = sizeof(dt_iop_levels_params_t);
   module->gui_data = NULL;
   dt_iop_levels_params_t tmp = (dt_iop_levels_params_t)
@@ -181,10 +224,10 @@ void init(dt_iop_module_t *module)
 
 void init_global(dt_iop_module_so_t *module)
 {
-  //const int program = 2; // basic.cl, from programs.conf
+  const int program = 2; // basic.cl, from programs.conf
   dt_iop_levels_global_data_t *gd = (dt_iop_levels_global_data_t *)malloc(sizeof(dt_iop_levels_global_data_t));
   module->data = gd;
-  //gd->kernel_levels = dt_opencl_create_kernel(program, "levels");      do not try to load kernel unless we have one
+  gd->kernel_levels = dt_opencl_create_kernel(program, "levels");
 }
 
 void cleanup_global(dt_iop_module_so_t *module)
@@ -230,7 +273,7 @@ void gui_init(struct dt_iop_module_t *self)
   g_signal_connect (G_OBJECT (c->area), "leave-notify-event",
                     G_CALLBACK (dt_iop_levels_leave_notify), self);
   g_signal_connect (G_OBJECT (c->area), "scroll-event",
-		    G_CALLBACK (dt_iop_levels_scroll), self);
+                    G_CALLBACK (dt_iop_levels_scroll), self);
 }
 
 void gui_cleanup(struct dt_iop_module_t *self)
@@ -302,17 +345,17 @@ static gboolean dt_iop_levels_expose(GtkWidget *widget, GdkEventExpose *event, g
   {
     switch(k)
     {
-    case 0:
-      cairo_set_source_rgb(cr, 0, 0, 0);
-      break;
+      case 0:
+        cairo_set_source_rgb(cr, 0, 0, 0);
+        break;
 
-    case 1:
-      cairo_set_source_rgb(cr, 0.5, 0.5, 0.5);
-      break;
+      case 1:
+        cairo_set_source_rgb(cr, 0.5, 0.5, 0.5);
+        break;
 
-    default:
-      cairo_set_source_rgb(cr, 1, 1, 1);
-      break;
+      default:
+        cairo_set_source_rgb(cr, 1, 1, 1);
+        break;
     }
 
     cairo_move_to(cr, width*p->levels[k], height+inset-1);
@@ -335,7 +378,7 @@ static gboolean dt_iop_levels_expose(GtkWidget *widget, GdkEventExpose *event, g
     dt_develop_t *dev = darktable.develop;
     float *hist, hist_max;
     hist = dev->histogram_pre_levels;
-    hist_max = dev->histogram_pre_levels_max;
+    hist_max = dev->histogram_linear?dev->histogram_pre_levels_max:logf(1.0 + dev->histogram_pre_levels_max);
     if(hist_max > 0)
     {
       cairo_save(cr);
@@ -382,30 +425,30 @@ static void dt_iop_levels_move_handle(int handle_move, float new_pos, float *lev
   // Determining the minimum and maximum bounds for the drag handles
   switch(handle_move)
   {
-  case 0:
-    max_x = fminf(levels[2] - (0.05 / drag_start_percentage),
-                  1);
-    max_x = fminf((levels[2] * (1 - drag_start_percentage) - 0.05)
-                  / (1 - drag_start_percentage),
-                  max_x);
-    break;
+    case 0:
+      max_x = fminf(levels[2] - (0.05 / drag_start_percentage),
+                    1);
+      max_x = fminf((levels[2] * (1 - drag_start_percentage) - 0.05)
+                    / (1 - drag_start_percentage),
+                    max_x);
+      break;
 
-  case 1:
-    min_x = levels[0] + 0.05;
-    max_x = levels[2] - 0.05;
-    break;
+    case 1:
+      min_x = levels[0] + 0.05;
+      max_x = levels[2] - 0.05;
+      break;
 
-  case 2:
-    min_x = fmaxf((0.05 / drag_start_percentage) + levels[0],
-                  0);
-    min_x = fmaxf((levels[0] * (1 - drag_start_percentage) + 0.05)
-                  / (1 - drag_start_percentage),
-                  min_x);
-    break;
+    case 2:
+      min_x = fmaxf((0.05 / drag_start_percentage) + levels[0],
+                    0);
+      min_x = fmaxf((levels[0] * (1 - drag_start_percentage) + 0.05)
+                    / (1 - drag_start_percentage),
+                    min_x);
+      break;
   }
 
   levels[handle_move] =
-      fminf(max_x, fmaxf(min_x, new_pos));
+    fminf(max_x, fmaxf(min_x, new_pos));
 
   if(handle_move != 1)
     levels[1] = levels[0] + (drag_start_percentage
@@ -468,7 +511,8 @@ static gboolean dt_iop_levels_button_press(GtkWidget *widget, GdkEventButton *ev
   {
     dt_iop_module_t *self = (dt_iop_module_t *)user_data;
 
-    if(event->type == GDK_2BUTTON_PRESS) {
+    if(event->type == GDK_2BUTTON_PRESS)
+    {
       // Reset
       dt_iop_levels_gui_data_t *c = (dt_iop_levels_gui_data_t *)self->gui_data;
       memcpy(self->params, self->default_params, self->params_size);
@@ -479,7 +523,9 @@ static gboolean dt_iop_levels_button_press(GtkWidget *widget, GdkEventButton *ev
 
       dt_dev_add_history_item(darktable.develop, self, TRUE);
       gtk_widget_queue_draw(self->widget);
-    } else {
+    }
+    else
+    {
       dt_iop_levels_gui_data_t *c = (dt_iop_levels_gui_data_t *)self->gui_data;
       c->dragging = 1;
     }
@@ -510,7 +556,8 @@ static gboolean dt_iop_levels_scroll(GtkWidget *widget, GdkEventScroll *event, g
   gboolean updated = FALSE;
   float new_position = 0;
 
-  if (c->dragging) {
+  if (c->dragging)
+  {
     return FALSE;
   }
 
@@ -525,12 +572,17 @@ static gboolean dt_iop_levels_scroll(GtkWidget *widget, GdkEventScroll *event, g
     updated = TRUE;
   }
 
-  if (updated) {
+  if (updated)
+  {
     dt_iop_levels_move_handle(c->handle_move, new_position,
-		              p->levels, c->drag_start_percentage);
+                              p->levels, c->drag_start_percentage);
     dt_dev_add_history_item(darktable.develop, self, TRUE);
     return TRUE;
   }
 
   return FALSE;
 }
+
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
