@@ -430,6 +430,8 @@ pick_toggled(GtkToggleButton *togglebutton, dt_iop_module_t *self)
   /* set the area sample size*/
   if (self->request_color_pick)
     dt_lib_colorpicker_set_point(darktable.lib, 0.5, 0.5);
+  else
+    dt_control_queue_redraw();
 
   if(self->off) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(self->off), 1);
   dt_iop_request_focus(self);
@@ -943,6 +945,17 @@ static gboolean dt_iop_tonecurve_button_press(GtkWidget *widget, GdkEventButton
       dt_dev_add_history_item(darktable.develop, self, TRUE);
       gtk_widget_queue_draw(self->widget);
     }
+    else
+    {
+      if (ch != ch_L)
+      {
+        p->tonecurve_autoscale_ab = 0;
+        c->selected = -2; // avoid motion notify re-inserting immediately.
+        dt_bauhaus_combobox_set(c->autoscale_ab, 1-p->tonecurve_autoscale_ab);
+        dt_dev_add_history_item(darktable.develop, self, TRUE);
+        gtk_widget_queue_draw(self->widget);
+      }
+    }
     return TRUE;
   }
   return FALSE;
