@@ -1,6 +1,7 @@
 /*
     This file is part of darktable,
     copyright (c) 2009--2010 Thierry Leconte.
+    copyright (c) 2012 Henrik Andersson.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,11 +38,10 @@ extern "C"
 #ifdef HAVE_GEGL
 #include <gegl.h>
 #endif
+#include "bauhaus/bauhaus.h"
 #include "develop/develop.h"
 #include "develop/imageop.h"
 #include "control/control.h"
-#include "dtgtk/slider.h"
-#include "dtgtk/resetlabel.h"
 #include "gui/accelerators.h"
 #include "gui/gtk.h"
 #include <gtk/gtk.h>
@@ -62,7 +62,7 @@ extern "C"
 
   typedef struct dt_iop_tonemapping_gui_data_t
   {
-    GtkDarktableSlider *contrast, *Fsize;
+    GtkWidget *contrast, *Fsize;
   }
   dt_iop_tonemapping_gui_data_t;
 
@@ -83,30 +83,30 @@ extern "C"
   {
     return IOP_GROUP_TONE;
   }
- 
+
   int
   flags ()
   {
     return IOP_FLAGS_SUPPORTS_BLENDING;
   }
 
-void init_key_accels(dt_iop_module_so_t *self)
-{
-  dt_accel_register_slider_iop(self, FALSE,
-                               NC_("accel", "contrast compression"));
-  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "spatial extent"));
-}
+  void init_key_accels(dt_iop_module_so_t *self)
+  {
+    dt_accel_register_slider_iop(self, FALSE,
+                                 NC_("accel", "contrast compression"));
+    dt_accel_register_slider_iop(self, FALSE, NC_("accel", "spatial extent"));
+  }
 
-void connect_key_accels(dt_iop_module_t *self)
-{
-  dt_iop_tonemapping_gui_data_t *g =
+  void connect_key_accels(dt_iop_module_t *self)
+  {
+    dt_iop_tonemapping_gui_data_t *g =
       (dt_iop_tonemapping_gui_data_t*)self->gui_data;
 
-  dt_accel_connect_slider_iop(self, "contrast compression",
-                              GTK_WIDGET(g->contrast));
-  dt_accel_connect_slider_iop(self, "spatial extent",
-                              GTK_WIDGET(g->Fsize));
-}
+    dt_accel_connect_slider_iop(self, "contrast compression",
+                                GTK_WIDGET(g->contrast));
+    dt_accel_connect_slider_iop(self, "spatial extent",
+                                GTK_WIDGET(g->Fsize));
+  }
 
   void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void *ivoid, void *ovoid, const dt_iop_roi_t *roi_in, const dt_iop_roi_t *roi_out)
   {
@@ -132,7 +132,7 @@ void connect_key_accels(dt_iop_module_t *self)
     // Build I=log(L)
     // and splat into the lattice
 #ifdef _OPENMP
-#pragma omp parallel for shared(lattice)
+    #pragma omp parallel for shared(lattice)
 #endif
     for(int j=0; j<height; j++)
     {
@@ -177,7 +177,7 @@ void connect_key_accels(dt_iop_module_t *self)
 
     const float contr = 1./data->contrast;
 #ifdef _OPENMP
-#pragma omp parallel for
+    #pragma omp parallel for
 #endif
     for(int j=0; j<height; j++)
     {
@@ -198,6 +198,7 @@ void connect_key_accels(dt_iop_module_t *self)
         out[0]=in[0]*Ln;
         out[1]=in[1]*Ln;
         out[2]=in[2]*Ln;
+        out[3]=in[3];
       }
     }
     // also process the clipping point, as good as we can without knowing
@@ -214,22 +215,22 @@ void connect_key_accels(dt_iop_module_t *self)
 // GUI
 //
   static void
-  contrast_callback (GtkDarktableSlider *slider, gpointer user_data)
+  contrast_callback (GtkWidget *slider, gpointer user_data)
   {
     dt_iop_module_t *self = (dt_iop_module_t *)user_data;
     if(self->dt->gui->reset) return;
     dt_iop_tonemapping_params_t *p = (dt_iop_tonemapping_params_t *)self->params;
-    p->contrast = dtgtk_slider_get_value(slider);
+    p->contrast = dt_bauhaus_slider_get(slider);
     dt_dev_add_history_item(darktable.develop, self, TRUE);
   }
 
   static void
-  Fsize_callback (GtkDarktableSlider *slider, gpointer user_data)
+  Fsize_callback (GtkWidget *slider, gpointer user_data)
   {
     dt_iop_module_t *self = (dt_iop_module_t *)user_data;
     if(self->dt->gui->reset) return;
     dt_iop_tonemapping_params_t *p = (dt_iop_tonemapping_params_t *)self->params;
-    p->Fsize = dtgtk_slider_get_value(slider);
+    p->Fsize = dt_bauhaus_slider_get(slider);
     dt_dev_add_history_item(darktable.develop, self, TRUE);
   }
 
@@ -257,17 +258,12 @@ void connect_key_accels(dt_iop_module_t *self)
     dt_iop_module_t *module = (dt_iop_module_t *)self;
     dt_iop_tonemapping_gui_data_t *g = (dt_iop_tonemapping_gui_data_t *)self->gui_data;
     dt_iop_tonemapping_params_t *p = (dt_iop_tonemapping_params_t *)module->params;
-    dtgtk_slider_set_value(g->contrast, p->contrast);
-    dtgtk_slider_set_value(g->Fsize, p->Fsize);
+    dt_bauhaus_slider_set(g->contrast, p->contrast);
+    dt_bauhaus_slider_set(g->Fsize, p->Fsize);
   }
 
   void reload_defaults(dt_iop_module_t *module)
   {
-    if(dt_image_is_hdr(&module->dev->image_storage))
-      module->default_enabled = 1;
-    else
-      module->default_enabled = 0;
-
     dt_iop_tonemapping_params_t tmp = (dt_iop_tonemapping_params_t)
     {
       2.5,30
@@ -281,8 +277,8 @@ void connect_key_accels(dt_iop_module_t *self)
     // module->data = malloc(sizeof(dt_iop_tonemapping_data_t));
     module->params = (dt_iop_params_t*)malloc(sizeof(dt_iop_tonemapping_params_t));
     module->default_params = (dt_iop_params_t*)malloc(sizeof(dt_iop_tonemapping_params_t));
-    module->default_enabled = 1;
-  module->priority = 156; // module order created by iop_dependencies.py, do not edit!
+    module->default_enabled = 0;
+  module->priority = 153; // module order created by iop_dependencies.py, do not edit!
     module->params_size = sizeof(dt_iop_tonemapping_params_t);
     module->gui_data = NULL;
   }
@@ -301,20 +297,21 @@ void connect_key_accels(dt_iop_module_t *self)
     dt_iop_tonemapping_gui_data_t *g = (dt_iop_tonemapping_gui_data_t *)self->gui_data;
     dt_iop_tonemapping_params_t *p = (dt_iop_tonemapping_params_t *)self->params;
 
-    self->widget = gtk_hbox_new(FALSE, 0);
-    GtkWidget *vbox = gtk_vbox_new(FALSE, DT_GUI_IOP_MODULE_CONTROL_SPACING);
-    gtk_box_pack_start(GTK_BOX(self->widget), vbox, TRUE, TRUE, 5);
+    self->widget = gtk_vbox_new(FALSE, DT_BAUHAUS_SPACE);
 
-    g->contrast = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR,1.0, 5.0000, 0.1, p->contrast, 3));
-    gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(g->contrast), TRUE, TRUE, 0);
-    dtgtk_slider_set_label(g->contrast,_("contrast compression"));
-    g_signal_connect (G_OBJECT (g->contrast), "value-changed",G_CALLBACK (contrast_callback), self);
+    /* contrast */
+    g->contrast = dt_bauhaus_slider_new_with_range(self,1.0, 5.0000, 0.1, p->contrast, 3);
 
-    g->Fsize = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR,0.0,100.0, 1.0, p->Fsize, 1));
-    dtgtk_slider_set_format_type(g->Fsize, DARKTABLE_SLIDER_FORMAT_PERCENT);
-    dtgtk_slider_set_label(g->Fsize,_("spatial extent"));
-    dtgtk_slider_set_unit(g->Fsize,(gchar *)"%");
-    gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(g->Fsize), TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->contrast), TRUE, TRUE, 0);
+    dt_bauhaus_widget_set_label(g->contrast,_("contrast compression"));
+    g_signal_connect (G_OBJECT (g->contrast), "value-changed",
+                      G_CALLBACK (contrast_callback), self);
+
+    /* spatial extent */
+    g->Fsize = dt_bauhaus_slider_new_with_range(self, 0.0, 100.0, 1.0, p->Fsize, 1);
+    dt_bauhaus_slider_set_format(g->Fsize,"%.0f%%");
+    dt_bauhaus_widget_set_label(g->Fsize,_("spatial extent"));
+    gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->Fsize), TRUE, TRUE, 0);
     g_signal_connect (G_OBJECT (g->Fsize), "value-changed",G_CALLBACK (Fsize_callback), self);
   }
 
@@ -326,4 +323,6 @@ void connect_key_accels(dt_iop_module_t *self)
 
 }
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
