@@ -25,11 +25,11 @@
 #ifdef HAVE_GEGL
 #include <gegl.h>
 #endif
+#include "bauhaus/bauhaus.h"
 #include "develop/develop.h"
 #include "develop/imageop.h"
+#include "common/opencl.h"
 #include "control/control.h"
-#include "dtgtk/slider.h"
-#include "dtgtk/resetlabel.h"
 #include "gui/accelerators.h"
 #include "gui/gtk.h"
 #include <gtk/gtk.h>
@@ -37,7 +37,8 @@
 #include <xmmintrin.h>
 
 // NaN-safe clip: NaN compares false and will result in 0.0
-#define CLIP(x) (((x)>=0.0)?((x)<=1.0?(x):1.0):0.0)
+#define CLIP(x)                 (((x)>=0.0)?((x)<=1.0?(x):1.0):0.0)
+
 DT_MODULE(2)
 
 typedef struct dt_iop_velvia_params_t
@@ -60,8 +61,8 @@ dt_iop_velvia_params1_t;
 typedef struct dt_iop_velvia_gui_data_t
 {
   GtkVBox   *vbox;
-  GtkDarktableSlider *strength_scale;
-  GtkDarktableSlider *bias_scale;
+  GtkWidget *strength_scale;
+  GtkWidget *bias_scale;
 }
 dt_iop_velvia_gui_data_t;
 
@@ -72,6 +73,12 @@ typedef struct dt_iop_velvia_data_t
 }
 dt_iop_velvia_data_t;
 
+typedef struct dt_iop_velvia_global_data_t
+{
+  int kernel_velvia;
+}
+dt_iop_velvia_global_data_t;
+
 const char *name()
 {
   return _("velvia");
@@ -79,7 +86,7 @@ const char *name()
 
 int flags()
 {
-  return IOP_FLAGS_INCLUDE_IN_STYLES | IOP_FLAGS_SUPPORTS_BLENDING;
+  return IOP_FLAGS_INCLUDE_IN_STYLES | IOP_FLAGS_SUPPORTS_BLENDING | IOP_FLAGS_ALLOW_TILING;
 }
 
 int
@@ -88,7 +95,7 @@ groups ()
   return IOP_GROUP_COLOR;
 }
 
-
+#if 0 // BAUHAUS doesnt support keyaccels yet...
 void init_key_accels(dt_iop_module_so_t *self)
 {
   dt_accel_register_slider_iop(self, FALSE, NC_("accel", "vibrance"));
@@ -104,6 +111,7 @@ void connect_key_accels(dt_iop_module_t *self)
   dt_accel_connect_slider_iop(self, "mid-tones bias",
                               GTK_WIDGET(g->bias_scale));
 }
+#endif
 
 int
 legacy_params (dt_iop_module_t *self, const void *const old_params, const int old_version, void *new_params, const int new_version)
@@ -167,25 +175,90 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
     }
   }
   _mm_sfence();
+
+  if(piece->pipe->mask_display)
+    dt_iop_alpha_copy(ivoid, ovoid, roi_out->width, roi_out->height);
+}
+
+
+#ifdef HAVE_OPENCL
+int
+process_cl (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem dev_in, cl_mem dev_out, const dt_iop_roi_t *roi_in, const dt_iop_roi_t *roi_out)
+{
+  dt_iop_velvia_data_t *data = (dt_iop_velvia_data_t *)piece->data;
+  dt_iop_velvia_global_data_t *gd = (dt_iop_velvia_global_data_t *)self->data;
+
+  cl_int err = -999;
+
+  const int devid = piece->pipe->devid;
+  const int width = roi_in->width;
+  const int height = roi_in->height;
+
+  const float strength = data->strength/100.0f;
+  const float bias = data->bias;
+
+  size_t sizes[] = { ROUNDUPWD(width), ROUNDUPHT(height), 1};
+
+  if(strength <= 0.0f)
+  {
+    size_t origin[] = {0, 0, 0};
+    size_t region[] = {width, height, 1};
+    err = dt_opencl_enqueue_copy_image(devid, dev_in, dev_out, origin, origin, region);
+    if (err != CL_SUCCESS) goto error;
+  }
+  else
+  {
+    dt_opencl_set_kernel_arg(devid, gd->kernel_velvia, 0, sizeof(cl_mem), (void *)&dev_in);
+    dt_opencl_set_kernel_arg(devid, gd->kernel_velvia, 1, sizeof(cl_mem), (void *)&dev_out);
+    dt_opencl_set_kernel_arg(devid, gd->kernel_velvia, 2, sizeof(int), (void *)&width);
+    dt_opencl_set_kernel_arg(devid, gd->kernel_velvia, 3, sizeof(int), (void *)&height);
+    dt_opencl_set_kernel_arg(devid, gd->kernel_velvia, 4, sizeof(float), (void *)&strength);
+    dt_opencl_set_kernel_arg(devid, gd->kernel_velvia, 5, sizeof(float), (void *)&bias);
+    err = dt_opencl_enqueue_kernel_2d(devid, gd->kernel_velvia, sizes);
+    if(err != CL_SUCCESS) goto error;
+  }
+
+  return TRUE;
+
+error:
+  dt_print(DT_DEBUG_OPENCL, "[opencl_velvia] couldn't enqueue kernel! %d\n", err);
+  return FALSE;
+}
+#endif
+
+void init_global(dt_iop_module_so_t *module)
+{
+  const int program = 8; // extended.cl, from programs.conf
+  dt_iop_velvia_global_data_t *gd = (dt_iop_velvia_global_data_t *)malloc(sizeof(dt_iop_velvia_global_data_t));
+  module->data = gd;
+  gd->kernel_velvia = dt_opencl_create_kernel(program, "velvia");
+}
+
+void cleanup_global(dt_iop_module_so_t *module)
+{
+  dt_iop_velvia_global_data_t *gd = (dt_iop_velvia_global_data_t *)module->data;
+  dt_opencl_free_kernel(gd->kernel_velvia);
+  free(module->data);
+  module->data = NULL;
 }
 
 static void
-strength_callback (GtkDarktableSlider *slider, gpointer user_data)
+strength_callback (GtkWidget *slider, gpointer user_data)
 {
   dt_iop_module_t *self = (dt_iop_module_t *)user_data;
   if(self->dt->gui->reset) return;
   dt_iop_velvia_params_t *p = (dt_iop_velvia_params_t *)self->params;
-  p->strength = dtgtk_slider_get_value(slider);
+  p->strength = dt_bauhaus_slider_get(slider);
   dt_dev_add_history_item(darktable.develop, self, TRUE);
 }
 
 static void
-bias_callback (GtkDarktableSlider *slider, gpointer user_data)
+bias_callback (GtkWidget *slider, gpointer user_data)
 {
   dt_iop_module_t *self = (dt_iop_module_t *)user_data;
   if(self->dt->gui->reset) return;
   dt_iop_velvia_params_t *p = (dt_iop_velvia_params_t *)self->params;
-  p->bias= dtgtk_slider_get_value(slider);
+  p->bias= dt_bauhaus_slider_get(slider);
   dt_dev_add_history_item(darktable.develop, self, TRUE);
 }
 
@@ -231,8 +304,8 @@ void gui_update(struct dt_iop_module_t *self)
   dt_iop_module_t *module = (dt_iop_module_t *)self;
   dt_iop_velvia_gui_data_t *g = (dt_iop_velvia_gui_data_t *)self->gui_data;
   dt_iop_velvia_params_t *p = (dt_iop_velvia_params_t *)module->params;
-  dtgtk_slider_set_value(g->strength_scale, p->strength);
-  dtgtk_slider_set_value(g->bias_scale, p->bias);
+  dt_bauhaus_slider_set(g->strength_scale, p->strength);
+  dt_bauhaus_slider_set(g->bias_scale, p->bias);
 }
 
 void init(dt_iop_module_t *module)
@@ -240,7 +313,7 @@ void init(dt_iop_module_t *module)
   module->params = malloc(sizeof(dt_iop_velvia_params_t));
   module->default_params = malloc(sizeof(dt_iop_velvia_params_t));
   module->default_enabled = 0;
-  module->priority = 862; // module order created by iop_dependencies.py, do not edit!
+  module->priority = 865; // module order created by iop_dependencies.py, do not edit!
   module->params_size = sizeof(dt_iop_velvia_params_t);
   module->gui_data = NULL;
   dt_iop_velvia_params_t tmp = (dt_iop_velvia_params_t)
@@ -265,25 +338,23 @@ void gui_init(struct dt_iop_module_t *self)
   dt_iop_velvia_gui_data_t *g = (dt_iop_velvia_gui_data_t *)self->gui_data;
   dt_iop_velvia_params_t *p = (dt_iop_velvia_params_t *)self->params;
 
-  self->widget = GTK_WIDGET(gtk_hbox_new(FALSE, 0));
-  g->vbox = GTK_VBOX(gtk_vbox_new(FALSE, DT_GUI_IOP_MODULE_CONTROL_SPACING));
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->vbox), TRUE, TRUE, 5);
+  self->widget = gtk_vbox_new(FALSE, DT_BAUHAUS_SPACE);
 
-  g->strength_scale = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR,0.0, 100.0, 1, p->strength, 0));
-  dtgtk_slider_set_format_type(g->strength_scale,DARKTABLE_SLIDER_FORMAT_PERCENT);
-  dtgtk_slider_set_label(g->strength_scale,_("vibrance"));
-  dtgtk_slider_set_unit(g->strength_scale,"%");
-  g->bias_scale = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR,0.0, 1.0, 0.01, p->bias, 2));
-  dtgtk_slider_set_format_type(g->bias_scale,DARKTABLE_SLIDER_FORMAT_RATIO);
-  dtgtk_slider_set_label(g->bias_scale,_("mid-tones bias"));
-
-  gtk_box_pack_start(GTK_BOX(g->vbox), GTK_WIDGET(g->strength_scale), TRUE, TRUE, 0);
-  gtk_box_pack_start(GTK_BOX(g->vbox), GTK_WIDGET(g->bias_scale), TRUE, TRUE, 0);
+  /* strength */
+  g->strength_scale = dt_bauhaus_slider_new_with_range(self, 0.0, 100.0, 1, p->strength, 0);
+  dt_bauhaus_slider_set_format(g->strength_scale,"%.0f%%");
+  dt_bauhaus_widget_set_label(g->strength_scale,_("strength"));
   g_object_set(G_OBJECT(g->strength_scale), "tooltip-text", _("the strength of saturation boost"), (char *)NULL);
-  g_object_set(G_OBJECT(g->bias_scale), "tooltip-text", _("how much to spare highlights and shadows"), (char *)NULL);
-
   g_signal_connect (G_OBJECT (g->strength_scale), "value-changed",
                     G_CALLBACK (strength_callback), self);
+
+  /* bias */
+  g->bias_scale = dt_bauhaus_slider_new_with_range(self, 0.0, 1.0, 0.01, p->bias, 2);
+  dt_bauhaus_widget_set_label(g->bias_scale,_("mid-tones bias"));
+
+  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->strength_scale), TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->bias_scale), TRUE, TRUE, 0);
+  g_object_set(G_OBJECT(g->bias_scale), "tooltip-text", _("how much to spare highlights and shadows"), (char *)NULL);
   g_signal_connect (G_OBJECT (g->bias_scale), "value-changed",
                     G_CALLBACK (bias_callback), self);
 }
@@ -294,4 +365,6 @@ void gui_cleanup(struct dt_iop_module_t *self)
   self->gui_data = NULL;
 }
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
