@@ -26,6 +26,7 @@
 #include <gegl.h>
 #endif
 #include "common/darktable.h"
+#include "common/opencl.h"
 #include "develop/develop.h"
 #include "develop/imageop.h"
 #include "control/control.h"
@@ -65,6 +66,13 @@ void init_presets (dt_iop_module_so_t *self)
 }
 */
 
+typedef struct dt_iop_zonesystem_global_data_t
+{
+  int kernel_zonesystem;
+}
+dt_iop_zonesystem_global_data_t;
+
+
 typedef struct dt_iop_zonesystem_gui_data_t
 {
   guchar *preview_buffer;
@@ -88,7 +96,7 @@ const char *name()
 
 int flags()
 {
-  return IOP_FLAGS_INCLUDE_IN_STYLES;
+  return IOP_FLAGS_INCLUDE_IN_STYLES | IOP_FLAGS_ALLOW_TILING;
 }
 
 int
@@ -240,21 +248,96 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
   #pragma omp parallel for default(none) shared(roi_out, in, out, zonemap_scale,zonemap_offset) schedule(static)
 #endif
   for (int j=0; j<roi_out->height; j++)
-  for (int i=0; i<roi_out->width; i++)
-  {
-    /* remap lightness into zonemap and apply lightness */
-    const float *inp = in + ch*(j*roi_out->width+i);
-    float *outp = out + ch*(j*roi_out->width+i);
+    for (int i=0; i<roi_out->width; i++)
+    {
+      /* remap lightness into zonemap and apply lightness */
+      const float *inp = in + ch*(j*roi_out->width+i);
+      float *outp = out + ch*(j*roi_out->width+i);
 
-    const int rz = CLAMPS(inp[0]*rzscale, 0, size-2);  // zone index
+      const int rz = CLAMPS(inp[0]*rzscale, 0, size-2);  // zone index
 
-    const float zs = ((rz > 0) ? (zonemap_offset[rz]/inp[0]) : 0) + zonemap_scale[rz];
+      const float zs = ((rz > 0) ? (zonemap_offset[rz]/inp[0]) : 0) + zonemap_scale[rz];
 
-    _mm_stream_ps(outp,_mm_mul_ps(_mm_load_ps(inp),_mm_set1_ps(zs)));
-  }
+      _mm_stream_ps(outp,_mm_mul_ps(_mm_load_ps(inp),_mm_set1_ps(zs)));
+    }
 
   _mm_sfence();
 
+  if(piece->pipe->mask_display)
+    dt_iop_alpha_copy(ivoid, ovoid, roi_out->width, roi_out->height);
+}
+
+
+#ifdef HAVE_OPENCL
+int
+process_cl (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem dev_in, cl_mem dev_out, const dt_iop_roi_t *roi_in, const dt_iop_roi_t *roi_out)
+{
+  dt_iop_zonesystem_data_t *data = (dt_iop_zonesystem_data_t *)piece->data;
+  dt_iop_zonesystem_global_data_t *gd = (dt_iop_zonesystem_global_data_t *)self->data;
+  cl_mem dev_zmo, dev_zms = NULL;
+  cl_int err = -999;
+
+  const int devid = piece->pipe->devid;
+  const int width = roi_in->width;
+  const int height = roi_in->height;
+
+  /* calculate zonemap */
+  const int size = data->size;
+  float zonemap[MAX_ZONE_SYSTEM_SIZE] = {-1};
+  float zonemap_offset[ROUNDUP(MAX_ZONE_SYSTEM_SIZE, 16)] = {-1};
+  float zonemap_scale[ROUNDUP(MAX_ZONE_SYSTEM_SIZE, 16)] = {-1};
+
+  _iop_zonesystem_calculate_zonemap (data, zonemap);
+
+  /* precompute scale and offset */
+  for (int k=0; k < size-1; k++) zonemap_scale[k]  = (zonemap[k+1]-zonemap[k])*(size-1);
+  for (int k=0; k < size-1; k++) zonemap_offset[k] = 100.0f * ((k+1)*zonemap[k] - k*zonemap[k+1]) ;
+
+  dev_zmo = dt_opencl_copy_host_to_device_constant(devid, sizeof(float)*ROUNDUP(MAX_ZONE_SYSTEM_SIZE, 16), zonemap_offset);
+  if (dev_zmo == NULL) goto error;
+  dev_zms = dt_opencl_copy_host_to_device_constant(devid, sizeof(float)*ROUNDUP(MAX_ZONE_SYSTEM_SIZE, 16), zonemap_scale);
+  if (dev_zms == NULL) goto error;
+
+  size_t sizes[] = { ROUNDUPWD(width), ROUNDUPHT(height), 1};
+
+  dt_opencl_set_kernel_arg(devid, gd->kernel_zonesystem, 0, sizeof(cl_mem), (void *)&dev_in);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_zonesystem, 1, sizeof(cl_mem), (void *)&dev_out);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_zonesystem, 2, sizeof(int), (void *)&width);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_zonesystem, 3, sizeof(int), (void *)&height);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_zonesystem, 4, sizeof(int), (void *)&size);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_zonesystem, 5, sizeof(cl_mem), (void *)&dev_zmo);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_zonesystem, 6, sizeof(cl_mem), (void *)&dev_zms);
+  err = dt_opencl_enqueue_kernel_2d(devid, gd->kernel_zonesystem, sizes);
+
+  if(err != CL_SUCCESS) goto error;
+  dt_opencl_release_mem_object(dev_zmo);
+  dt_opencl_release_mem_object(dev_zms);
+  return TRUE;
+
+error:
+  if (dev_zmo != NULL) dt_opencl_release_mem_object(dev_zmo);
+  if (dev_zms != NULL) dt_opencl_release_mem_object(dev_zms);
+  dt_print(DT_DEBUG_OPENCL, "[opencl_zonesystem] couldn't enqueue kernel! %d\n", err);
+  return FALSE;
+}
+#endif
+
+
+
+void init_global(dt_iop_module_so_t *module)
+{
+  const int program = 2; // basic.cl, from programs.conf
+  dt_iop_zonesystem_global_data_t *gd = (dt_iop_zonesystem_global_data_t *)malloc(sizeof(dt_iop_zonesystem_global_data_t));
+  module->data = gd;
+  gd->kernel_zonesystem = dt_opencl_create_kernel(program, "zonesystem");
+}
+
+void cleanup_global(dt_iop_module_so_t *module)
+{
+  dt_iop_zonesystem_global_data_t *gd = (dt_iop_zonesystem_global_data_t *)module->data;
+  dt_opencl_free_kernel(gd->kernel_zonesystem);
+  free(module->data);
+  module->data = NULL;
 }
 
 
@@ -308,7 +391,7 @@ void init(dt_iop_module_t *module)
   module->params = malloc(sizeof(dt_iop_zonesystem_params_t));
   module->default_params = malloc(sizeof(dt_iop_zonesystem_params_t));
   module->default_enabled = 0;
-  module->priority = 588; // module order created by iop_dependencies.py, do not edit!
+  module->priority = 596; // module order created by iop_dependencies.py, do not edit!
   module->params_size = sizeof(dt_iop_zonesystem_params_t);
   module->gui_data = NULL;
   dt_iop_zonesystem_params_t tmp = (dt_iop_zonesystem_params_t)
@@ -387,18 +470,18 @@ void gui_init(struct dt_iop_module_t *self)
 
   /* add signal handler for preview pipe finish to redraw the preview */
   dt_control_signal_connect(darktable.signals,
-			    DT_SIGNAL_DEVELOP_PREVIEW_PIPE_FINISHED, 
-			    G_CALLBACK(_iop_zonesystem_redraw_preview_callback), 
-			    self);
+                            DT_SIGNAL_DEVELOP_PREVIEW_PIPE_FINISHED,
+                            G_CALLBACK(_iop_zonesystem_redraw_preview_callback),
+                            self);
 
 
 }
 
 void gui_cleanup(struct dt_iop_module_t *self)
 {
-  dt_control_signal_disconnect(darktable.signals, 
-			       G_CALLBACK(_iop_zonesystem_redraw_preview_callback), 
-			       self);
+  dt_control_signal_disconnect(darktable.signals,
+                               G_CALLBACK(_iop_zonesystem_redraw_preview_callback),
+                               self);
 
   dt_iop_zonesystem_gui_data_t *g = (dt_iop_zonesystem_gui_data_t *)self->gui_data;
   dt_pthread_mutex_destroy(&g->lock);
@@ -561,9 +644,9 @@ dt_iop_zonesystem_bar_scrolled (GtkWidget *widget, GdkEventScroll *event, dt_iop
   dt_iop_zonesystem_params_t *p = (dt_iop_zonesystem_params_t *)self->params;
   int cs = p->size;
   if(event->direction == GDK_SCROLL_UP)
-    p->size+=2;
+    p->size+=1;
   else if(event->direction == GDK_SCROLL_DOWN)
-    p->size-=2;
+    p->size-=1;
 
   /* sanity checks */
   p->size = p->size>MAX_ZONE_SYSTEM_SIZE?MAX_ZONE_SYSTEM_SIZE:p->size;
@@ -591,18 +674,16 @@ dt_iop_zonesystem_bar_motion_notify (GtkWidget *widget, GdkEventMotion *event, d
   const int inset = DT_ZONESYSTEM_INSET;
   int width = widget->allocation.width - 2*inset, height = widget->allocation.height - 2*inset;
 
+  /* calculate zonemap */
+  float zonemap[MAX_ZONE_SYSTEM_SIZE]= {-1};
+  _iop_zonesystem_calculate_zonemap (p,zonemap);
+
   /* record mouse position within control */
   g->mouse_x = CLAMP(event->x - inset, 0, width);
   g->mouse_y = CLAMP(height - 1 - event->y + inset, 0, height);
 
-  g->zone_under_mouse = (g->mouse_x/width) / (1.0/(p->size-1));
-
   if (g->is_dragging)
   {
-    /* calculate zonemap */
-    float zonemap[MAX_ZONE_SYSTEM_SIZE]= {-1};
-    _iop_zonesystem_calculate_zonemap (p,zonemap);
-
     if ( (g->mouse_x/width) > zonemap[g->current_zone-1] &&  (g->mouse_x/width) < zonemap[g->current_zone+1] )
     {
       p->zone[g->current_zone] = (g->mouse_x/width);
@@ -610,7 +691,24 @@ dt_iop_zonesystem_bar_motion_notify (GtkWidget *widget, GdkEventMotion *event, d
     }
   }
   else
-    g->hilite_zone = (g->mouse_y<(height/2.0))?TRUE:FALSE;
+  {
+    /* decide which zone the mouse is over */
+    if(g->mouse_y >= height*(1.0-DT_ZONESYSTEM_REFERENCE_SPLIT))
+      g->zone_under_mouse = (g->mouse_x/width) / (1.0/(p->size-1));
+    else
+    {
+      float xpos = g->mouse_x/width;
+      for(int z = 0; z < p->size-1; z++)
+      {
+        if(xpos >= zonemap[z] && xpos < zonemap[z+1])
+        {
+          g->zone_under_mouse = z;
+          break;
+        }
+      }
+    }
+    g->hilite_zone = (g->mouse_y<height)?TRUE:FALSE;
+  }
 
   gtk_widget_queue_draw (self->widget);
   gtk_widget_queue_draw (g->preview);
@@ -694,9 +792,11 @@ dt_iop_zonesystem_preview_expose (GtkWidget *widget, GdkEventExpose *event, dt_i
 void _iop_zonesystem_redraw_preview_callback(gpointer instance, gpointer user_data)
 {
   dt_iop_module_t *self = (dt_iop_module_t *)user_data;
-  dt_iop_zonesystem_gui_data_t *g = (dt_iop_zonesystem_gui_data_t *)self->gui_data;  
+  dt_iop_zonesystem_gui_data_t *g = (dt_iop_zonesystem_gui_data_t *)self->gui_data;
 
   dt_control_queue_redraw_widget(g->preview);
 }
 
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
