@@ -35,25 +35,25 @@ dt_lib_keywords_t;
 
 /* callback for drag and drop */
 static void _lib_keywords_drag_data_received_callback(GtkWidget *w,
-					  GdkDragContext *dctx,
-					  guint x,
-					  guint y,
-					  GtkSelectionData *data,
-					  guint info,
-					  guint time,
-					  gpointer user_data);
+    GdkDragContext *dctx,
+    guint x,
+    guint y,
+    GtkSelectionData *data,
+    guint info,
+    guint time,
+    gpointer user_data);
 
 /* set the data for drag and drop, eg the treeview path of drag source */
 static void _lib_keywords_drag_data_get_callback(GtkWidget *w,
-						 GdkDragContext *dctx,
-						 GtkSelectionData *data,
-						 guint info,
-						 guint time,
-						 gpointer user_data);
+    GdkDragContext *dctx,
+    GtkSelectionData *data,
+    guint info,
+    guint time,
+    gpointer user_data);
 
 /* add keyword to collection rules */
 static void _lib_keywords_add_collection_rule(GtkTreeView *view, GtkTreePath *tp,
-					      GtkTreeViewColumn *tvc, gpointer user_data);
+    GtkTreeViewColumn *tvc, gpointer user_data);
 
 const char* name()
 {
@@ -73,7 +73,7 @@ uint32_t container()
 
 int position()
 {
-  return 399;
+  return 300;
 }
 
 void init_key_accels(dt_lib_module_t *self)
@@ -93,22 +93,35 @@ void connect_key_accels(dt_lib_module_t *self)
 
 void gui_init(dt_lib_module_t *self)
 {
+  
   /* initialize ui widgets */
   dt_lib_keywords_t *d = (dt_lib_keywords_t *)g_malloc(sizeof(dt_lib_keywords_t));
   memset(d,0,sizeof(dt_lib_keywords_t));
   self->data = (void *)d;
   self->widget = gtk_vbox_new(FALSE, 5);
 
+  /* Create a new scrolled window, with scrollbars only if needed */
+  GtkWidget *scrolled_window;
+  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+  
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
+                                    GTK_POLICY_AUTOMATIC, 
+                                    GTK_POLICY_AUTOMATIC);
+
+
+  
+  
+
   GtkTreeStore *store = gtk_tree_store_new(1, G_TYPE_STRING);
 
   /* intialize the tree store with known tags */
   sqlite3_stmt *stmt;
 
-  GtkTreeIter uncategorized, temp;  
+  GtkTreeIter uncategorized, temp;
   memset(&uncategorized,0,sizeof(GtkTreeIter));
 
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), 
-			      "select name,icon,description from tags", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "SELECT name,icon,description FROM tags ORDER BY UPPER(name) DESC", -1, &stmt, NULL);
   while (sqlite3_step(stmt) == SQLITE_ROW)
   {
     if(strchr((const char *)sqlite3_column_text(stmt, 0),'|')==0)
@@ -116,13 +129,13 @@ void gui_init(dt_lib_module_t *self)
       /* add uncategorized root iter if not exists */
       if (!uncategorized.stamp)
       {
-	gtk_tree_store_insert(store, &uncategorized, NULL,0);
-	gtk_tree_store_set(store, &uncategorized, 0, _(UNCATEGORIZED_TAG), -1);
+        gtk_tree_store_insert(store, &uncategorized, NULL,0);
+        gtk_tree_store_set(store, &uncategorized, 0, _(UNCATEGORIZED_TAG), -1);
       }
 
       /* adding a uncategorized tag */
       gtk_tree_store_insert(store, &temp, &uncategorized,0);
-      gtk_tree_store_set(store, &temp, 0, sqlite3_column_text(stmt, 0), -1); 
+      gtk_tree_store_set(store, &temp, 0, sqlite3_column_text(stmt, 0), -1);
 
     }
     else
@@ -135,40 +148,40 @@ void gui_init(dt_lib_module_t *self)
       if (pch != NULL)
       {
         int j = 0;
-        while (pch[j] != NULL) 
+        while (pch[j] != NULL)
         {
-	  gboolean found=FALSE;
-	  int children = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(store),level>0?&current:NULL);
-	  /* find child with name, if not found create and continue */
-	  for (int k=0;k<children;k++)
-	  {
-	    if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(store), &iter, level>0?&current:NULL, k))
-	    {
-	      gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, 0, &value, -1);
-	      
-	      if (strcmp(value, pch[j])==0)
-	      {
-		current = iter;
-		found = TRUE;
-		break;
-	      }
-	    }
-	  }
-	  
-	  /* lets add new keyword and assign current */
-	  if (!found)
-	  {
-	    gtk_tree_store_insert(store, &iter, level>0?&current:NULL,0);
-	    gtk_tree_store_set(store, &iter, 0, pch[j], -1);
-	    current = iter;
-	  }
-	  
-	  level++;
-	  j++;
+          gboolean found=FALSE;
+          int children = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(store),level>0?&current:NULL);
+          /* find child with name, if not found create and continue */
+          for (int k=0; k<children; k++)
+          {
+            if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(store), &iter, level>0?&current:NULL, k))
+            {
+              gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, 0, &value, -1);
+
+              if (strcmp(value, pch[j])==0)
+              {
+                current = iter;
+                found = TRUE;
+                break;
+              }
+            }
+          }
+
+          /* lets add new keyword and assign current */
+          if (!found)
+          {
+            gtk_tree_store_insert(store, &iter, level>0?&current:NULL,0);
+            gtk_tree_store_set(store, &iter, 0, pch[j], -1);
+            current = iter;
+          }
+
+          level++;
+          j++;
         }
-	
+
         g_strfreev(pch);
-	
+
       }
     }
   }
@@ -177,14 +190,19 @@ void gui_init(dt_lib_module_t *self)
   GtkCellRenderer *renderer;
 
   d->view = GTK_TREE_VIEW (gtk_tree_view_new());
+  gtk_widget_set_size_request(GTK_WIDGET(d->view), -1, 300);
+
+  
+  gtk_container_add(GTK_CONTAINER(scrolled_window), GTK_WIDGET(d->view));
+  
 
   renderer = gtk_cell_renderer_text_new ();
   gtk_tree_view_insert_column_with_attributes(d->view,
-                                               -1,      
-                                               "",  
-                                               renderer,
-                                               "text", 0,
-                                               NULL);
+      -1,
+      "",
+      renderer,
+      "text", 0,
+      NULL);
 
   gtk_tree_view_set_headers_visible(d->view, FALSE);
 
@@ -192,41 +210,42 @@ void gui_init(dt_lib_module_t *self)
 
   /* setup dnd source and destination within treeview */
   static const GtkTargetEntry dnd_target = { "keywords-reorganize",
-					     GTK_TARGET_SAME_WIDGET, 0};
-  
-  gtk_tree_view_enable_model_drag_source(d->view, 
-					 GDK_BUTTON1_MASK, 
-					 &dnd_target, 1, GDK_ACTION_MOVE);
-  
+                              GTK_TARGET_SAME_WIDGET, 0
+                                           };
+
+  gtk_tree_view_enable_model_drag_source(d->view,
+                                         GDK_BUTTON1_MASK,
+                                         &dnd_target, 1, GDK_ACTION_MOVE);
+
   gtk_tree_view_enable_model_drag_dest(d->view, &dnd_target, 1, GDK_ACTION_MOVE);
-  
+
   /* setup drag and drop signals */
   g_signal_connect(G_OBJECT(d->view),"drag-data-received",
-		   G_CALLBACK(_lib_keywords_drag_data_received_callback),
-		   self);
-  
+                   G_CALLBACK(_lib_keywords_drag_data_received_callback),
+                   self);
+
   g_signal_connect(G_OBJECT(d->view),"drag-data-get",
-		   G_CALLBACK(_lib_keywords_drag_data_get_callback),
-		   self);
+                   G_CALLBACK(_lib_keywords_drag_data_get_callback),
+                   self);
 
   /* add callback when keyword is activated */
-  g_signal_connect(G_OBJECT(d->view), "row-activated", 
-		   G_CALLBACK(_lib_keywords_add_collection_rule), self); 
+  g_signal_connect(G_OBJECT(d->view), "row-activated",
+                   G_CALLBACK(_lib_keywords_add_collection_rule), self);
 
 
 
   /* free store, treeview has its own storage now */
   g_object_unref(store);
-  
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(d->view), TRUE,FALSE,0);
+
+  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(scrolled_window), TRUE, TRUE, 0);
 
   gtk_widget_show_all(GTK_WIDGET(d->view));
-  
+
 
 }
 
 void gui_cleanup(dt_lib_module_t *self)
-{ 
+{
   // dt_lib_import_t *d = (dt_lib_import_t*)self->data;
 
   /* cleanup mem */
@@ -244,16 +263,16 @@ static void _gtk_tree_move_iter(GtkTreeStore *store, GtkTreeIter *source, GtkTre
 
   gtk_tree_model_get_value(GTK_TREE_MODEL(store), source, 0, &value);
   gtk_tree_store_insert(store, &ni, dest,0);
-  gtk_tree_store_set(store, &ni, 0, g_strdup(g_value_get_string(&value)), -1);   
+  gtk_tree_store_set(store, &ni, 0, g_strdup(g_value_get_string(&value)), -1);
 
   /* for each children recurse into */
   int children = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(store), source);
-  for (int k=0;k<children;k++)
+  for (int k=0; k<children; k++)
   {
     GtkTreeIter child;
     if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(store), &child, source, k))
       _gtk_tree_move_iter(store, &child, &ni);
-  
+
   }
 
   /* iter copied lets remove source */
@@ -262,20 +281,20 @@ static void _gtk_tree_move_iter(GtkTreeStore *store, GtkTreeIter *source, GtkTre
 }
 
 static void _lib_keywords_drag_data_get_callback(GtkWidget *w,
-						 GdkDragContext *dctx,
-						 GtkSelectionData *data,
-						 guint info,
-						 guint time,
-						 gpointer user_data)
+    GdkDragContext *dctx,
+    GtkSelectionData *data,
+    guint info,
+    guint time,
+    gpointer user_data)
 {
   dt_lib_module_t *self = (dt_lib_module_t *)user_data;
   dt_lib_keywords_t *d = (dt_lib_keywords_t*)self->data;
-  
+
   /* get iter of item to drag to ssetup drag data */
   GtkTreeIter iter;
   GtkTreeModel *model = NULL;
   GtkTreeSelection *s = gtk_tree_view_get_selection(d->view);
-  
+
   if (gtk_tree_selection_get_selected(s,&model,&iter))
   {
 
@@ -283,16 +302,16 @@ static void _lib_keywords_drag_data_get_callback(GtkWidget *w,
     GtkTreePath *path = NULL;
     path = gtk_tree_model_get_path(model,&iter);
     gchar *sp = gtk_tree_path_to_string(path);
-   
+
     gtk_selection_data_set(data,data->target, 8, (const guchar *)sp, strlen(sp));
   }
 
 }
 
 /* builds a keyword string out of GtkTreePath */
-static void _lib_keywords_string_from_path(char *dest,size_t ds, 
-					   GtkTreeModel *model, 
-					   GtkTreePath *path)
+static void _lib_keywords_string_from_path(char *dest,size_t ds,
+    GtkTreeModel *model,
+    GtkTreePath *path)
 {
   g_assert(model!=NULL);
   g_assert(path!=NULL);
@@ -314,11 +333,11 @@ static void _lib_keywords_string_from_path(char *dest,size_t ds,
     /* add component to begin of list */
     gtk_tree_model_get_value(model, &iter, 0, &value);
     if ( !(gtk_tree_path_get_depth(wp) == 1 &&
-	   strcmp(g_value_get_string(&value), _(UNCATEGORIZED_TAG)) == 0))
+           strcmp(g_value_get_string(&value), _(UNCATEGORIZED_TAG)) == 0))
     {
-      components = g_list_insert(components, 
-				 g_strdup(g_value_get_string(&value)), 
-				 0);
+      components = g_list_insert(components,
+                                 g_strdup(g_value_get_string(&value)),
+                                 0);
     }
     g_value_unset(&value);
 
@@ -331,29 +350,29 @@ static void _lib_keywords_string_from_path(char *dest,size_t ds,
   int dcs = 0;
 
   if(g_list_length(components) == 0) dcs += g_snprintf(dest+dcs, ds-dcs," ");
-		      
-  for(int i=0;i<g_list_length(components);i++) 
+
+  for(int i=0; i<g_list_length(components); i++)
   {
     dcs += g_snprintf(dest+dcs, ds-dcs,
-		      "%s%s",
-		      (gchar *)g_list_nth_data(components, i),
-		      (i < g_list_length(components)-1) ? "|" : "");
+                      "%s%s",
+                      (gchar *)g_list_nth_data(components, i),
+                      (i < g_list_length(components)-1) ? "|" : "");
   }
-  
+
   /* free data */
   gtk_tree_path_free(wp);
-  
+
 
 }
 
 static void _lib_keywords_drag_data_received_callback(GtkWidget *w,
-					  GdkDragContext *dctx,
-					  guint x,
-					  guint y,
-					  GtkSelectionData *data,
-					  guint info,
-					  guint time,
-					  gpointer user_data)
+    GdkDragContext *dctx,
+    guint x,
+    guint y,
+    GtkSelectionData *data,
+    guint info,
+    guint time,
+    gpointer user_data)
 {
   dt_lib_module_t *self = (dt_lib_module_t *)user_data;
   dt_lib_keywords_t *d = (dt_lib_keywords_t*)self->data;
@@ -367,36 +386,36 @@ static void _lib_keywords_drag_data_received_callback(GtkWidget *w,
     if (gtk_tree_view_get_dest_row_at_pos(d->view, x, y, &dpath, &dpos))
     {
       /* fetch tree iter of source and dest dnd operation */
-      GtkTreePath *spath = gtk_tree_path_new_from_string((char *)data->data);      
-     
+      GtkTreePath *spath = gtk_tree_path_new_from_string((char *)data->data);
+
       char dtag[1024];
       char stag[1024];
-      
+
       _lib_keywords_string_from_path(dtag, 1024, model, dpath);
       _lib_keywords_string_from_path(stag, 1024, model, spath);
 
       /* reject drop onto ourself */
       if (strcmp(dtag,stag) == 0)
-	goto reject_drop;
+        goto reject_drop;
 
-       /* updated tags in database */
+      /* updated tags in database */
       dt_tag_reorganize(stag,dtag);
 
       /* lets move the source iter into dest iter */
       GtkTreeIter sit,dit;
       gtk_tree_model_get_iter(model, &sit, spath);
       gtk_tree_model_get_iter(model, &dit, dpath);
-     
+
       _gtk_tree_move_iter(GTK_TREE_STORE(model), &sit, &dit);
-      
+
       /* accept drop */
       gtk_drag_finish(dctx, TRUE, FALSE, time);
 
-      
+
     }
 
-  }   
- 
+  }
+
   /* reject drop */
 reject_drop:
   gtk_drag_finish (dctx, FALSE, FALSE, time);
@@ -404,11 +423,11 @@ reject_drop:
 
 
 static void _lib_keywords_add_collection_rule(GtkTreeView *view, GtkTreePath *tp,
-					      GtkTreeViewColumn *tvc, gpointer user_data)
+    GtkTreeViewColumn *tvc, gpointer user_data)
 {
-  char kw[1024]={0};
+  char kw[1024]= {0};
   _lib_keywords_string_from_path(kw, 1024, gtk_tree_view_get_model(view), tp);
-  
+
   /*
    * add a collection rule
    * TODO: move this into a dt_collection_xxx API to be used
@@ -428,10 +447,13 @@ static void _lib_keywords_add_collection_rule(GtkTreeView *view, GtkTreePath *tp
 
   /* set tag rule type */
   snprintf(confname, 200, "plugins/lighttable/collect/item%1d", rule);
-  dt_conf_set_int(confname, 2);
-  
+  dt_conf_set_int(confname, 3);
+
   dt_conf_set_int("plugins/lighttable/collect/num_rules", rule+1);
 
   dt_view_collection_update(darktable.view_manager);
   dt_collection_update_query(darktable.collection);
 }
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
+// kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
