@@ -24,17 +24,27 @@
 #include "common/history.h"
 #include "common/image_cache.h"
 #include "common/mipmap_cache.h"
+#include "common/selection.h"
 #include "control/control.h"
 #include "control/conf.h"
 #include "develop/develop.h"
 #include "libs/lib.h"
 #include "gui/accelerators.h"
 #include "gui/gtk.h"
+#include "gui/drag_and_drop.h"
 
 #include <gdk/gdkkeysyms.h>
 
 DT_MODULE(1)
 
+typedef enum dt_lib_filmstrip_select_t
+{
+  DT_LIB_FILMSTRIP_SELECT_NONE,
+  DT_LIB_FILMSTRIP_SELECT_SINGLE,
+  DT_LIB_FILMSTRIP_SELECT_TOGGLE,
+  DT_LIB_FILMSTRIP_SELECT_RANGE
+} dt_lib_filmstrip_select_t;
+
 typedef struct dt_lib_filmstrip_t
 {
   GtkWidget *filmstrip;
@@ -53,6 +63,8 @@ typedef struct dt_lib_filmstrip_t
   int32_t size_handle_height;
 
   int32_t activated_image;
+  dt_lib_filmstrip_select_t select;
+  int32_t select_id;
 }
 dt_lib_filmstrip_t;
 
@@ -75,25 +87,32 @@ static gboolean _lib_filmstrip_scroll_callback(GtkWidget *w,GdkEventScroll *e, g
 static gboolean _lib_filmstrip_expose_callback(GtkWidget *widget, GdkEventExpose *event, gpointer user_data);
 /* button press callback */
 static gboolean _lib_filmstrip_button_press_callback(GtkWidget *widget, GdkEventButton *event, gpointer user_data);
+/* button release callback */
+static gboolean _lib_filmstrip_button_release_callback(GtkWidget *widget, GdkEventButton *event, gpointer user_data);
 /* signal callback for collection change */
 static void _lib_filmstrip_collection_changed_callback(gpointer instance, gpointer user_data);
 
 /* key accelerators callback */
 static gboolean _lib_filmstrip_copy_history_key_accel_callback(GtkAccelGroup *accel_group,
-                                GObject *aceeleratable, guint keyval,
-                                GdkModifierType modifier, gpointer data);
+    GObject *aceeleratable, guint keyval,
+    GdkModifierType modifier, gpointer data);
 static gboolean _lib_filmstrip_paste_history_key_accel_callback(GtkAccelGroup *accel_group,
-                                GObject *aceeleratable, guint keyval,
-                                GdkModifierType modifier, gpointer data);
+    GObject *aceeleratable, guint keyval,
+    GdkModifierType modifier, gpointer data);
 static gboolean _lib_filmstrip_discard_history_key_accel_callback(GtkAccelGroup *accel_group,
-                                GObject *aceeleratable, guint keyval,
-                                GdkModifierType modifier, gpointer data);
+    GObject *aceeleratable, guint keyval,
+    GdkModifierType modifier, gpointer data);
 static gboolean _lib_filmstrip_ratings_key_accel_callback(GtkAccelGroup *accel_group,
-                                GObject *aceeleratable, guint keyval,
-                                GdkModifierType modifier, gpointer data);
+    GObject *aceeleratable, guint keyval,
+    GdkModifierType modifier, gpointer data);
 static gboolean _lib_filmstrip_colorlabels_key_accel_callback(GtkAccelGroup *accel_group,
-                                GObject *aceeleratable, guint keyval,
-                                GdkModifierType modifier, gpointer data);
+    GObject *aceeleratable, guint keyval,
+    GdkModifierType modifier, gpointer data);
+
+/* drag'n'drop callbacks */
+static void _lib_filmstrip_dnd_get_callback(GtkWidget *widget, GdkDragContext *context, GtkSelectionData *selection_data,
+    guint target_type, guint time, gpointer user_data);
+static void _lib_filmstrip_dnd_begin_callback(GtkWidget *widget, GdkDragContext *context, gpointer user_data);
 
 const char* name()
 {
@@ -102,7 +121,7 @@ const char* name()
 
 uint32_t views()
 {
-  return DT_VIEW_DARKROOM | DT_VIEW_TETHERING;
+  return DT_VIEW_DARKROOM | DT_VIEW_TETHERING | DT_VIEW_MAP;
 }
 
 uint32_t container()
@@ -110,7 +129,7 @@ uint32_t container()
   return DT_UI_CONTAINER_PANEL_BOTTOM;
 }
 
-int expandable() 
+int expandable()
 {
   return 0;
 }
@@ -131,7 +150,7 @@ void init_key_accels(dt_lib_module_t *self)
   dt_accel_register_lib(self, NC_("accel", "rate 5"), GDK_5, 0);
   dt_accel_register_lib(self, NC_("accel", "rate reject"), GDK_r, 0);
 
-  
+
   /* setup history key accelerators */
   dt_accel_register_lib(self, NC_("accel", "copy history"),
                         GDK_c, GDK_CONTROL_MASK);
@@ -140,8 +159,8 @@ void init_key_accels(dt_lib_module_t *self)
   dt_accel_register_lib(self, NC_("accel", "discard history"),
                         GDK_d, GDK_CONTROL_MASK);
 
-  
-  
+
+
   /* setup color label accelerators */
   dt_accel_register_lib(self, NC_("accel", "color red"), GDK_F1, 0);
   dt_accel_register_lib(self, NC_("accel", "color yellow"), GDK_F2, 0);
@@ -155,84 +174,84 @@ void connect_key_accels(dt_lib_module_t *self)
 
   // Rating accels
   dt_accel_connect_lib(
-      self, "rate desert",
-      g_cclosure_new(
-          G_CALLBACK(_lib_filmstrip_ratings_key_accel_callback),
-          (gpointer)DT_VIEW_DESERT,NULL));
+    self, "rate desert",
+    g_cclosure_new(
+      G_CALLBACK(_lib_filmstrip_ratings_key_accel_callback),
+      (gpointer)DT_VIEW_DESERT,NULL));
   dt_accel_connect_lib(
-      self, "rate 1",
-      g_cclosure_new(
-          G_CALLBACK(_lib_filmstrip_ratings_key_accel_callback),
-          (gpointer)DT_VIEW_STAR_1,NULL));
+    self, "rate 1",
+    g_cclosure_new(
+      G_CALLBACK(_lib_filmstrip_ratings_key_accel_callback),
+      (gpointer)DT_VIEW_STAR_1,NULL));
   dt_accel_connect_lib(
-      self, "rate 2",
-      g_cclosure_new(
-          G_CALLBACK(_lib_filmstrip_ratings_key_accel_callback),
-          (gpointer)DT_VIEW_STAR_2,NULL));
+    self, "rate 2",
+    g_cclosure_new(
+      G_CALLBACK(_lib_filmstrip_ratings_key_accel_callback),
+      (gpointer)DT_VIEW_STAR_2,NULL));
   dt_accel_connect_lib(
-      self, "rate 3",
-      g_cclosure_new(
-          G_CALLBACK(_lib_filmstrip_ratings_key_accel_callback),
-          (gpointer)DT_VIEW_STAR_3,NULL));
+    self, "rate 3",
+    g_cclosure_new(
+      G_CALLBACK(_lib_filmstrip_ratings_key_accel_callback),
+      (gpointer)DT_VIEW_STAR_3,NULL));
   dt_accel_connect_lib(
-      self, "rate 4",
-      g_cclosure_new(
-          G_CALLBACK(_lib_filmstrip_ratings_key_accel_callback),
-          (gpointer)DT_VIEW_STAR_4,NULL));
+    self, "rate 4",
+    g_cclosure_new(
+      G_CALLBACK(_lib_filmstrip_ratings_key_accel_callback),
+      (gpointer)DT_VIEW_STAR_4,NULL));
   dt_accel_connect_lib(
-      self, "rate 5",
-      g_cclosure_new(
-          G_CALLBACK(_lib_filmstrip_ratings_key_accel_callback),
-          (gpointer)DT_VIEW_STAR_5,NULL));
+    self, "rate 5",
+    g_cclosure_new(
+      G_CALLBACK(_lib_filmstrip_ratings_key_accel_callback),
+      (gpointer)DT_VIEW_STAR_5,NULL));
   dt_accel_connect_lib(
-      self, "rate reject",
-      g_cclosure_new(
-          G_CALLBACK(_lib_filmstrip_ratings_key_accel_callback),
-          (gpointer)DT_VIEW_REJECT,NULL));
+    self, "rate reject",
+    g_cclosure_new(
+      G_CALLBACK(_lib_filmstrip_ratings_key_accel_callback),
+      (gpointer)DT_VIEW_REJECT,NULL));
 
   // History key accels
   dt_accel_connect_lib(
-      self, "copy history",
-      g_cclosure_new(
-          G_CALLBACK(_lib_filmstrip_copy_history_key_accel_callback),
-          (gpointer)self->data,NULL));
+    self, "copy history",
+    g_cclosure_new(
+      G_CALLBACK(_lib_filmstrip_copy_history_key_accel_callback),
+      (gpointer)self->data,NULL));
   dt_accel_connect_lib(
-      self, "paste history",
-      g_cclosure_new(
-          G_CALLBACK(_lib_filmstrip_paste_history_key_accel_callback),
-          (gpointer)self->data,NULL));
+    self, "paste history",
+    g_cclosure_new(
+      G_CALLBACK(_lib_filmstrip_paste_history_key_accel_callback),
+      (gpointer)self->data,NULL));
   dt_accel_connect_lib(
-      self, "discard history",
-      g_cclosure_new(
-          G_CALLBACK(_lib_filmstrip_discard_history_key_accel_callback),
-          (gpointer)self->data,NULL));
+    self, "discard history",
+    g_cclosure_new(
+      G_CALLBACK(_lib_filmstrip_discard_history_key_accel_callback),
+      (gpointer)self->data,NULL));
 
   // Color label accels
   dt_accel_connect_lib(
-      self, "color red",
-      g_cclosure_new(
-          G_CALLBACK(_lib_filmstrip_colorlabels_key_accel_callback),
-          (gpointer)0,NULL));
+    self, "color red",
+    g_cclosure_new(
+      G_CALLBACK(_lib_filmstrip_colorlabels_key_accel_callback),
+      (gpointer)0,NULL));
   dt_accel_connect_lib(
-      self, "color yellow",
-      g_cclosure_new(
-          G_CALLBACK(_lib_filmstrip_colorlabels_key_accel_callback),
-          (gpointer)1,NULL));
+    self, "color yellow",
+    g_cclosure_new(
+      G_CALLBACK(_lib_filmstrip_colorlabels_key_accel_callback),
+      (gpointer)1,NULL));
   dt_accel_connect_lib(
-      self, "color green",
-      g_cclosure_new(
-          G_CALLBACK(_lib_filmstrip_colorlabels_key_accel_callback),
-          (gpointer)2,NULL));
+    self, "color green",
+    g_cclosure_new(
+      G_CALLBACK(_lib_filmstrip_colorlabels_key_accel_callback),
+      (gpointer)2,NULL));
   dt_accel_connect_lib(
-      self, "color blue",
-      g_cclosure_new(
-          G_CALLBACK(_lib_filmstrip_colorlabels_key_accel_callback),
-          (gpointer)3,NULL));
+    self, "color blue",
+    g_cclosure_new(
+      G_CALLBACK(_lib_filmstrip_colorlabels_key_accel_callback),
+      (gpointer)3,NULL));
   dt_accel_connect_lib(
-      self, "color purple",
-      g_cclosure_new(
-          G_CALLBACK(_lib_filmstrip_colorlabels_key_accel_callback),
-          (gpointer)4,NULL));
+    self, "color purple",
+    g_cclosure_new(
+      G_CALLBACK(_lib_filmstrip_colorlabels_key_accel_callback),
+      (gpointer)4,NULL));
 }
 
 void gui_init(dt_lib_module_t *self)
@@ -249,24 +268,36 @@ void gui_init(dt_lib_module_t *self)
 
   /* create drawingarea */
   self->widget = gtk_vbox_new(FALSE,0);
-  
-  
+
+
   /* createing filmstrip box*/
   d->filmstrip = gtk_event_box_new();
 
-  gtk_widget_add_events(d->filmstrip, 
-              GDK_POINTER_MOTION_MASK | 
-              GDK_POINTER_MOTION_HINT_MASK | 
-              GDK_BUTTON_PRESS_MASK | 
-              GDK_BUTTON_RELEASE_MASK |
-              GDK_SCROLL_MASK |
-              GDK_LEAVE_NOTIFY_MASK);
+  /* allow drag&drop of images from the filmstrip. this has to come before the other callbacks are registered! */
+  gtk_drag_source_set(d->filmstrip,
+                      GDK_BUTTON1_MASK,
+                      target_list_all,
+                      n_targets_all,
+                      GDK_ACTION_COPY);
+
+  g_signal_connect_after(d->filmstrip, "drag-begin", G_CALLBACK(_lib_filmstrip_dnd_begin_callback), self);
+  g_signal_connect(d->filmstrip, "drag-data-get", G_CALLBACK(_lib_filmstrip_dnd_get_callback), self);
+
+  gtk_widget_add_events(d->filmstrip,
+                        GDK_POINTER_MOTION_MASK |
+                        GDK_POINTER_MOTION_HINT_MASK |
+                        GDK_BUTTON_PRESS_MASK |
+                        GDK_BUTTON_RELEASE_MASK |
+                        GDK_SCROLL_MASK |
+                        GDK_LEAVE_NOTIFY_MASK);
 
   /* connect callbacks */
   g_signal_connect (G_OBJECT (d->filmstrip), "expose-event",
                     G_CALLBACK (_lib_filmstrip_expose_callback), self);
   g_signal_connect (G_OBJECT (d->filmstrip), "button-press-event",
                     G_CALLBACK (_lib_filmstrip_button_press_callback), self);
+  g_signal_connect (G_OBJECT (d->filmstrip), "button-release-event",
+                    G_CALLBACK (_lib_filmstrip_button_release_callback), self);
   g_signal_connect (G_OBJECT (d->filmstrip), "scroll-event",
                     G_CALLBACK (_lib_filmstrip_scroll_callback), self);
   g_signal_connect (G_OBJECT (d->filmstrip), "motion-notify-event",
@@ -274,7 +305,7 @@ void gui_init(dt_lib_module_t *self)
   g_signal_connect (G_OBJECT (d->filmstrip), "leave-notify-event",
                     G_CALLBACK(_lib_filmstrip_mouse_leave_callback), self);
 
-  
+
   /* set size of filmstrip */
   int32_t height = dt_conf_get_int("plugins/lighttable/filmstrip/height");
   gtk_widget_set_size_request(d->filmstrip, -1, CLAMP(height,64,400));
@@ -282,14 +313,14 @@ void gui_init(dt_lib_module_t *self)
   /* create the resize handle */
   GtkWidget *size_handle = gtk_event_box_new();
   gtk_widget_set_size_request(size_handle,-1,10);
-  gtk_widget_add_events(size_handle, 
-              GDK_POINTER_MOTION_MASK | 
-              GDK_POINTER_MOTION_HINT_MASK | 
-              GDK_BUTTON_PRESS_MASK | 
-              GDK_BUTTON_RELEASE_MASK |
-              GDK_ENTER_NOTIFY_MASK |
-              GDK_LEAVE_NOTIFY_MASK
-              );
+  gtk_widget_add_events(size_handle,
+                        GDK_POINTER_MOTION_MASK |
+                        GDK_POINTER_MOTION_HINT_MASK |
+                        GDK_BUTTON_PRESS_MASK |
+                        GDK_BUTTON_RELEASE_MASK |
+                        GDK_ENTER_NOTIFY_MASK |
+                        GDK_LEAVE_NOTIFY_MASK
+                       );
 
   g_signal_connect (G_OBJECT (size_handle), "button-press-event",
                     G_CALLBACK (_lib_filmstrip_size_handle_button_callback), self);
@@ -313,22 +344,22 @@ void gui_init(dt_lib_module_t *self)
   darktable.view_manager->proxy.filmstrip.activated_image = _lib_filmstrip_get_activated_imgid;
 
   /* connect signal handler */
-  dt_control_signal_connect(darktable.signals, 
-                  DT_SIGNAL_COLLECTION_CHANGED,
-                  G_CALLBACK(_lib_filmstrip_collection_changed_callback),
-                  (gpointer)self);
-  dt_control_signal_connect(darktable.signals, 
-                  DT_SIGNAL_DEVELOP_MIPMAP_UPDATED,
-                  G_CALLBACK(_lib_filmstrip_collection_changed_callback),
-                  (gpointer)self);
+  dt_control_signal_connect(darktable.signals,
+                            DT_SIGNAL_COLLECTION_CHANGED,
+                            G_CALLBACK(_lib_filmstrip_collection_changed_callback),
+                            (gpointer)self);
+  dt_control_signal_connect(darktable.signals,
+                            DT_SIGNAL_DEVELOP_MIPMAP_UPDATED,
+                            G_CALLBACK(_lib_filmstrip_collection_changed_callback),
+                            (gpointer)self);
 }
 
 void gui_cleanup(dt_lib_module_t *self)
 {
   /* disconnect from signals */
   dt_control_signal_disconnect(darktable.signals,
-                    G_CALLBACK(_lib_filmstrip_collection_changed_callback),
-                    (gpointer)self);
+                               G_CALLBACK(_lib_filmstrip_collection_changed_callback),
+                               (gpointer)self);
 
   /* unset viewmanager proxy */
   darktable.view_manager->proxy.filmstrip.module = NULL;
@@ -373,7 +404,7 @@ static gboolean _lib_filmstrip_size_handle_button_callback(GtkWidget *w, GdkEven
       gtk_widget_get_size_request(d->filmstrip, NULL, &d->size_handle_height);
       d->size_handle_is_dragging = TRUE;
     }
-    else if (e->type == GDK_BUTTON_RELEASE) 
+    else if (e->type == GDK_BUTTON_RELEASE)
       d->size_handle_is_dragging = FALSE;
   }
   return TRUE;
@@ -419,9 +450,9 @@ static gboolean _lib_filmstrip_scroll_callback(GtkWidget *w,GdkEventScroll *e, g
   dt_lib_filmstrip_t *strip = (dt_lib_filmstrip_t *)self->data;
 
   /* change the offset */
-  if (strip->offset > 0 && (e->direction == GDK_SCROLL_UP || e->direction == GDK_SCROLL_LEFT)) 
+  if (strip->offset > 0 && (e->direction == GDK_SCROLL_UP || e->direction == GDK_SCROLL_LEFT))
     strip->offset--;
-  else if(strip->offset < strip->collection_count-1 && (e->direction == GDK_SCROLL_DOWN || e->direction == GDK_SCROLL_RIGHT)) 
+  else if(strip->offset < strip->collection_count-1 && (e->direction == GDK_SCROLL_DOWN || e->direction == GDK_SCROLL_RIGHT))
     strip->offset++;
   else
     return TRUE;
@@ -436,49 +467,99 @@ static gboolean _lib_filmstrip_button_press_callback(GtkWidget *w, GdkEventButto
   dt_lib_filmstrip_t *strip = (dt_lib_filmstrip_t *)self->data;
 
   int32_t mouse_over_id = strip->mouse_over_id;
+  strip->select = DT_LIB_FILMSTRIP_SELECT_NONE;
 
-  /* is this an activation of image */
-  if (e->button == 1 && e->type == GDK_2BUTTON_PRESS)
-    if (mouse_over_id > 0)
-    {
-      strip->activated_image = mouse_over_id;
-      dt_control_signal_raise(darktable.signals, DT_SIGNAL_VIEWMANAGER_FILMSTRIP_ACTIVATE);
-      return TRUE;
-    }
-
-
-  /* let check if any thumb controls was clicked */
-  switch(strip->image_over)
+  if (e->button == 1)
   {
-    case DT_VIEW_DESERT:
-      break;
-    case DT_VIEW_REJECT:
-    case DT_VIEW_STAR_1:
-    case DT_VIEW_STAR_2:
-    case DT_VIEW_STAR_3:
-    case DT_VIEW_STAR_4:
-    case DT_VIEW_STAR_5:
+    if(e->type == GDK_BUTTON_PRESS)
     {
-      const dt_image_t *cimg = dt_image_cache_read_get(darktable.image_cache, mouse_over_id);
-      dt_image_t *image = dt_image_cache_write_get(darktable.image_cache, cimg);
-      image->dirty = 1;
-      if(strip->image_over == DT_VIEW_STAR_1 && ((image->flags & 0x7) == 1)) image->flags &= ~0x7;
-      else if(strip->image_over == DT_VIEW_REJECT && ((image->flags & 0x7) == 6)) image->flags &= ~0x7;
-      else
+      /* let check if any thumb controls was clicked */
+      switch(strip->image_over)
       {
-        image->flags &= ~0x7;
-        image->flags |= strip->image_over;
-      }
-      dt_image_cache_write_release(darktable.image_cache, image, DT_IMAGE_CACHE_SAFE);
-      dt_image_cache_read_release(darktable.image_cache, image);
-      break;
-    }
+        case DT_VIEW_DESERT:
+          /* is this an activation of image */
+          if ((e->state & (GDK_SHIFT_MASK|GDK_CONTROL_MASK)) == 0)
+            strip->select = DT_LIB_FILMSTRIP_SELECT_SINGLE;
+          else if ((e->state & (GDK_CONTROL_MASK)) == GDK_CONTROL_MASK)
+            strip->select = DT_LIB_FILMSTRIP_SELECT_TOGGLE;
+          else if ((e->state & (GDK_SHIFT_MASK)) == GDK_SHIFT_MASK)
+            strip->select = DT_LIB_FILMSTRIP_SELECT_RANGE;
+          if(strip->select != DT_LIB_FILMSTRIP_SELECT_NONE)
+          {
+            strip->select_id = mouse_over_id;
+            return TRUE;
+          }
+          break;
+        case DT_VIEW_REJECT:
+        case DT_VIEW_STAR_1:
+        case DT_VIEW_STAR_2:
+        case DT_VIEW_STAR_3:
+        case DT_VIEW_STAR_4:
+        case DT_VIEW_STAR_5:
+        {
+          const dt_image_t *cimg = dt_image_cache_read_get(darktable.image_cache, mouse_over_id);
+          dt_image_t *image = dt_image_cache_write_get(darktable.image_cache, cimg);
+          if(strip->image_over == DT_VIEW_STAR_1 && ((image->flags & 0x7) == 1)) image->flags &= ~0x7;
+          else if(strip->image_over == DT_VIEW_REJECT && ((image->flags & 0x7) == 6)) image->flags &= ~0x7;
+          else
+          {
+            image->flags &= ~0x7;
+            image->flags |= strip->image_over;
+          }
+          dt_image_cache_write_release(darktable.image_cache, image, DT_IMAGE_CACHE_SAFE);
+          dt_image_cache_read_release(darktable.image_cache, image);
+          gtk_widget_queue_draw(darktable.view_manager->proxy.filmstrip.module->widget);
+          return TRUE;
+        }
 
-    default:
-      return FALSE;
+        default:
+          return FALSE;
+      }
+    }
+    else if(e->type == GDK_2BUTTON_PRESS)
+    {
+      if (mouse_over_id > 0)
+      {
+        strip->activated_image = mouse_over_id;
+        dt_control_signal_raise(darktable.signals, DT_SIGNAL_VIEWMANAGER_FILMSTRIP_ACTIVATE);
+        return TRUE;
+      }
+    }
   }
 
-  return TRUE;
+  return FALSE;
+}
+
+static gboolean _lib_filmstrip_button_release_callback(GtkWidget *w, GdkEventButton *e, gpointer user_data)
+{
+  dt_lib_module_t *self = (dt_lib_module_t *)user_data;
+  dt_lib_filmstrip_t *strip = (dt_lib_filmstrip_t *)self->data;
+
+  int32_t mouse_over_id = strip->mouse_over_id;
+  int32_t select_id = strip->select_id;
+  gboolean result = FALSE;
+
+  if(mouse_over_id == select_id && mouse_over_id > 0)
+  {
+    result = TRUE;
+    if(strip->select == DT_LIB_FILMSTRIP_SELECT_SINGLE)
+      dt_selection_select_single(darktable.selection, mouse_over_id);
+    else if(strip->select == DT_LIB_FILMSTRIP_SELECT_TOGGLE)
+      dt_selection_toggle(darktable.selection, mouse_over_id);
+    else if(strip->select == DT_LIB_FILMSTRIP_SELECT_RANGE)
+      dt_selection_select_range(darktable.selection, mouse_over_id);
+    else
+      result = FALSE;
+  }
+
+  strip->select = DT_LIB_FILMSTRIP_SELECT_NONE;
+  strip->select_id = -1;
+
+  /* redraw filmstrip */
+  if(result && darktable.view_manager->proxy.filmstrip.module)
+    gtk_widget_queue_draw(darktable.view_manager->proxy.filmstrip.module->widget);
+
+  return result;
 }
 
 static gboolean _lib_filmstrip_expose_callback(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
@@ -499,7 +580,7 @@ static gboolean _lib_filmstrip_expose_callback(GtkWidget *widget, GdkEventExpose
   DT_CTL_SET_GLOBAL(lib_image_mouse_over_id, -1);
 
   /* create cairo surface */
-  cairo_t *cr = gdk_cairo_create(widget->window); 
+  cairo_t *cr = gdk_cairo_create(widget->window);
 
   /* fill background */
   cairo_set_source_rgb (cr, .2, .2, .2);
@@ -535,9 +616,9 @@ static gboolean _lib_filmstrip_expose_callback(GtkWidget *widget, GdkEventExpose
   if(!query)
     return FALSE;
 
-  if(offset < 0)                
+  if(offset < 0)
     strip->offset = offset = 0;
-  if(offset > strip->collection_count-1) 
+  if(offset > strip->collection_count-1)
     strip->offset = offset = strip->collection_count-1;
 
   // dt_view_set_scrollbar(self, offset, count, max_cols, 0, 1, 1);
@@ -545,18 +626,20 @@ static gboolean _lib_filmstrip_expose_callback(GtkWidget *widget, GdkEventExpose
   DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), query, -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, offset - max_cols/2);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, max_cols);
-  
+
 
   cairo_save(cr);
   cairo_translate(cr, empty_edge, 0.0f);
   for(int col = 0; col < max_cols; col++)
   {
-    if(col < col_start) {
+    if(col < col_start)
+    {
       cairo_translate(cr, wd, 0.0f);
       continue;
     }
 
-    if(step_res != SQLITE_DONE) {
+    if(step_res != SQLITE_DONE)
+    {
       step_res = sqlite3_step(stmt);
     }
 
@@ -573,10 +656,11 @@ static gboolean _lib_filmstrip_expose_callback(GtkWidget *widget, GdkEventExpose
       // FIXME find out where the y translation is done, how big the value is and use it directly instead of getting it from the matrix ...
       cairo_matrix_t m;
       cairo_get_matrix(cr, &m);
-      dt_view_image_expose(&(strip->image_over), id, cr, wd, ht, max_cols, img_pointerx, img_pointery);
+      dt_view_image_expose(&(strip->image_over), id, cr, wd, ht, max_cols, img_pointerx, img_pointery, FALSE);
       cairo_restore(cr);
     }
-    else if (step_res == SQLITE_DONE) {
+    else if (step_res == SQLITE_DONE)
+    {
       /* do nothing, just add some empty thumb frames */
     }
     else goto failure;
@@ -594,7 +678,8 @@ failure:
       g_object_set(G_OBJECT(strip->filmstrip), "tooltip-text", tooltip, (char *)NULL);
       g_free(tooltip);
     }
-  } else if(darktable.gui->center_tooltip == 2) // not set in this round
+  }
+  else if(darktable.gui->center_tooltip == 2)   // not set in this round
   {
     darktable.gui->center_tooltip = 0;
     g_object_set(G_OBJECT(strip->filmstrip), "tooltip-text", "", (char *)NULL);
@@ -606,8 +691,8 @@ failure:
 #endif
 
   /* cleanup */
-  cairo_destroy(cr); 
-    
+  cairo_destroy(cr);
+
   return TRUE;
 }
 
@@ -620,28 +705,13 @@ static void _lib_filmstrip_collection_changed_callback(gpointer instance, gpoint
 static void _lib_filmstrip_scroll_to_image(dt_lib_module_t *self, gint imgid, gboolean activate)
 {
   dt_lib_filmstrip_t *strip = (dt_lib_filmstrip_t *)self->data;
- 
+
   /* if no imgid just bail out */
   if(imgid <= 0) return;
 
   strip->activated_image = imgid;
 
-  char query[1024];
-  const gchar *qin = dt_collection_get_query (darktable.collection);
-  if(qin)
-  {
-    snprintf(query, 1024, "select rowid from (%s) where id=?3", qin);
-    sqlite3_stmt *stmt;
-    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), query, -1, &stmt, NULL);
-    DT_DEBUG_SQLITE3_BIND_INT(stmt, 1,  0);
-    DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, -1);
-    DT_DEBUG_SQLITE3_BIND_INT(stmt, 3, imgid);
-    if(sqlite3_step(stmt) == SQLITE_ROW)
-    {
-      strip->offset = sqlite3_column_int(stmt, 0) - 1;
-    }
-    sqlite3_finalize(stmt);
-  }
+  strip->offset = dt_collection_image_offset(imgid);
 
   /* activate the image if requested */
   if (activate)
@@ -663,8 +733,8 @@ int32_t _lib_filmstrip_get_activated_imgid(dt_lib_module_t *self)
 }
 
 static gboolean _lib_filmstrip_copy_history_key_accel_callback(GtkAccelGroup *accel_group,
-                                                           GObject *aceeleratable, guint keyval,
-                                                           GdkModifierType modifier, gpointer data)
+    GObject *aceeleratable, guint keyval,
+    GdkModifierType modifier, gpointer data)
 {
   dt_lib_filmstrip_t *strip = (dt_lib_filmstrip_t *)data;
   int32_t mouse_over_id;
@@ -679,8 +749,8 @@ static gboolean _lib_filmstrip_copy_history_key_accel_callback(GtkAccelGroup *ac
 }
 
 static gboolean _lib_filmstrip_paste_history_key_accel_callback(GtkAccelGroup *accel_group,
-                                                            GObject *aceeleratable, guint keyval,
-                                                            GdkModifierType modifier, gpointer data)
+    GObject *aceeleratable, guint keyval,
+    GdkModifierType modifier, gpointer data)
 {
   dt_lib_filmstrip_t *strip = (dt_lib_filmstrip_t *)data;
   if (strip->history_copy_imgid==-1) return FALSE;
@@ -697,8 +767,8 @@ static gboolean _lib_filmstrip_paste_history_key_accel_callback(GtkAccelGroup *a
 }
 
 static gboolean _lib_filmstrip_discard_history_key_accel_callback(GtkAccelGroup *accel_group,
-                                                              GObject *aceeleratable, guint keyval,
-                                                              GdkModifierType modifier, gpointer data)
+    GObject *aceeleratable, guint keyval,
+    GdkModifierType modifier, gpointer data)
 {
   dt_lib_filmstrip_t *strip = (dt_lib_filmstrip_t *)data;
   if (strip->history_copy_imgid==-1) return FALSE;
@@ -713,8 +783,8 @@ static gboolean _lib_filmstrip_discard_history_key_accel_callback(GtkAccelGroup
 }
 
 static gboolean _lib_filmstrip_ratings_key_accel_callback(GtkAccelGroup *accel_group,
-                                                      GObject *aceeleratable, guint keyval,
-                                                      GdkModifierType modifier, gpointer data)
+    GObject *aceeleratable, guint keyval,
+    GdkModifierType modifier, gpointer data)
 {
   long int num = (long int)data;
   switch (num)
@@ -734,9 +804,9 @@ static gboolean _lib_filmstrip_ratings_key_accel_callback(GtkAccelGroup *accel_g
       /* get image from cache */
       const dt_image_t *cimg = dt_image_cache_read_get(darktable.image_cache, mouse_over_id);
       dt_image_t *image = dt_image_cache_write_get(darktable.image_cache, cimg);
-      if (num == 666) 
+      if (num == 666)
         image->flags &= ~0xf;
-      else if (num == DT_VIEW_STAR_1 && ((image->flags & 0x7) == 1)) 
+      else if (num == DT_VIEW_STAR_1 && ((image->flags & 0x7) == 1))
         image->flags &= ~0x7;
       else
       {
@@ -752,13 +822,13 @@ static gboolean _lib_filmstrip_ratings_key_accel_callback(GtkAccelGroup *accel_g
     }
     default:
       break;
-    }
-    return TRUE;
+  }
+  return TRUE;
 }
 
 static gboolean _lib_filmstrip_colorlabels_key_accel_callback(GtkAccelGroup *accel_group,
-                                GObject *acceleratable, guint keyval,
-                                GdkModifierType modifier, gpointer data)
+    GObject *acceleratable, guint keyval,
+    GdkModifierType modifier, gpointer data)
 {
   dt_colorlabels_key_accel_callback(NULL, NULL, 0, 0, data);
   /* redraw filmstrip */
@@ -767,4 +837,124 @@ static gboolean _lib_filmstrip_colorlabels_key_accel_callback(GtkAccelGroup *acc
   return TRUE;
 }
 
+static void
+_lib_filmstrip_dnd_get_callback(GtkWidget *widget, GdkDragContext *context, GtkSelectionData *selection_data,
+                                guint target_type, guint time, gpointer user_data)
+{
+  dt_lib_module_t *self = (dt_lib_module_t *)user_data;
+  dt_lib_filmstrip_t *strip = (dt_lib_filmstrip_t *)self->data;
+
+  g_assert (selection_data != NULL);
+
+  int mouse_over_id = strip->mouse_over_id;
+  int count = dt_collection_get_selected_count(NULL);
+  switch (target_type)
+  {
+    case DND_TARGET_IMGID:
+    {
+      int id = ((count == 1) ? mouse_over_id : -1);
+      gtk_selection_data_set(selection_data, selection_data-> target, _DWORD, (guchar*) &id, sizeof(id));
+      break;
+    }
+    default: // return the location of the file as a last resort
+    case DND_TARGET_URI: // TODO: add all images from the selection
+    {
+      if(count == 1)
+      {
+        gchar pathname[DT_MAX_PATH_LEN] = {0};
+        dt_image_full_path(mouse_over_id, pathname, DT_MAX_PATH_LEN);
+        gchar *uri = g_strdup_printf("file://%s", pathname); // TODO: should we add the host?
+        gtk_selection_data_set(selection_data, selection_data-> target, _BYTE, (guchar*) uri, strlen(uri));
+        g_free(uri);
+      }
+      else
+      {
+        sqlite3_stmt *stmt;
+        GList *images = NULL;
+        DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select distinct imgid from selected_images", -1, &stmt, NULL);
+        while (sqlite3_step (stmt) == SQLITE_ROW)
+        {
+          int id = sqlite3_column_int(stmt, 0);
+          gchar pathname[DT_MAX_PATH_LEN] = {0};
+          dt_image_full_path(id, pathname, DT_MAX_PATH_LEN);
+          gchar *uri = g_strdup_printf("file://%s", pathname); // TODO: should we add the host?
+          images = g_list_append(images, uri);
+        }
+        sqlite3_finalize(stmt);
+        gchar* uri_list = dt_util_glist_to_str("\r\n", images, count);
+        gtk_selection_data_set(selection_data, selection_data-> target, _BYTE, (guchar*) uri_list, strlen(uri_list));
+        g_free(uri_list);
+      }
+      break;
+    }
+  }
+}
+
+static void
+_lib_filmstrip_dnd_begin_callback(GtkWidget *widget, GdkDragContext *context, gpointer user_data)
+{
+  const int ts = 64;
+
+  dt_lib_module_t *self = (dt_lib_module_t *)user_data;
+  dt_lib_filmstrip_t *strip = (dt_lib_filmstrip_t *)self->data;
+
+  int imgid = strip->mouse_over_id;
+
+  // imgid part of selection -> do nothing
+  // otherwise               -> select the current image
+  strip->select = DT_LIB_FILMSTRIP_SELECT_NONE;
+  sqlite3_stmt *stmt;
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select imgid from selected_images where imgid=?1", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, imgid);
+  if(sqlite3_step(stmt) != SQLITE_ROW)
+  {
+    dt_selection_select_single(darktable.selection, imgid);
+    /* redraw filmstrip */
+    if(darktable.view_manager->proxy.filmstrip.module)
+      gtk_widget_queue_draw(darktable.view_manager->proxy.filmstrip.module->widget);
+  }
+  sqlite3_finalize(stmt);
+
+  // if we are dragging a single image -> use the thumbnail of that image
+  // otherwise use the generic d&d icon
+  // TODO: have something pretty in the 2nd case, too.
+  if(dt_collection_get_selected_count(NULL) == 1)
+  {
+    dt_mipmap_buffer_t buf;
+    dt_mipmap_size_t mip = dt_mipmap_cache_get_matching_size(darktable.mipmap_cache, ts, ts);
+    dt_mipmap_cache_read_get(darktable.mipmap_cache, &buf, imgid, mip, DT_MIPMAP_BLOCKING);
+
+    if(buf.buf)
+    {
+      uint8_t *scratchmem = dt_mipmap_cache_alloc_scratchmem(darktable.mipmap_cache);
+      uint8_t *buf_decompressed = dt_mipmap_cache_decompress(&buf, scratchmem);
+
+      uint8_t *rgbbuf = g_malloc((buf.width+2)*(buf.height+2)*3);
+      memset(rgbbuf, 64, (buf.width+2)*(buf.height+2)*3);
+      for(int i=1; i<=buf.height; i++)
+        for(int j=1; j<=buf.width; j++)
+          for(int k=0; k<3; k++)
+            rgbbuf[(i*(buf.width+2)+j)*3+k] = buf_decompressed[((i-1)*buf.width+j-1)*4+2-k];
+
+      int w=ts, h=ts;
+      if(buf.width < buf.height) w = (buf.width*ts)/buf.height; // portrait
+      else                       h = (buf.height*ts)/buf.width; // landscape
+
+      GdkPixbuf *source = gdk_pixbuf_new_from_data(rgbbuf, GDK_COLORSPACE_RGB, FALSE, 8, (buf.width+2), (buf.height+2), (buf.width+2)*3, NULL, NULL);
+      GdkPixbuf *scaled = gdk_pixbuf_scale_simple(source, w, h, GDK_INTERP_HYPER);
+      gtk_drag_set_icon_pixbuf(context, scaled, 0, 0);
+
+      if(source)
+        g_object_unref(source);
+      if(scaled)
+        g_object_unref(scaled);
+      g_free(rgbbuf);
+    }
+
+    dt_mipmap_cache_read_release(darktable.mipmap_cache, &buf);
+  }
+}
+
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
